
#region

//        Copyright (C) 2004, 2005. Antonio Gutiérrez Mayoral
//                 <agutierr@gsyc.escet.urjc.es>
// 
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License as
//  published by the Free Software Foundation; either version 2 of the
//  License, or (at your option) any later version.
// 
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  General Public License for more details.
// 
//  You should have received a copy of the GNU General Public
//  License along with this program; if not, write to the
//  Free Software Foundation, Inc., 59 Temple Place - Suite 330,
//  Boston, MA 02111-1307, USA.

#endregion

using Gtk;
using Gdk;
using Glade;
using Vte;
using Gecko;
using GNU.Gettext;

using System;
using System.Drawing;
using System.Collections;
using System.Threading;

using pfc.Svn;

/// <summary>
/// Main Namespace.
/// <summary>

namespace pfc {

	/// <summary> Main App Class </summary>
	public class gsvn {
	
		/// <summary> Main Window Gtk Widgets. We have to declare it to can use them on
		/// App. 
		/// </summary>
		
		# region
		//[Widget] Gtk.Widget mainWindow;
		[Widget] Gtk.Toolbar mainToolbar;
		
		/// <summary> Main Notebook, where we dock the Gsvn Log Window and user's Shells
		/// </summary>
		[Widget] Gtk.Notebook inf_notebook;
		/// <summary> Status Bar from main window. We use it for write messages when user
		/// pases mouse over menu entries. 
		/// </summary>
		[Widget] Gtk.Statusbar statusBar;
		/// <summary> ToolButtons from Toolbar. Due a Glade bug, we have to declarate it 
		/// for event's connect :-( We cant do it through Glade. </summary>
		[Widget] Gtk.Button addDirToolbarButton;
		[Widget] Gtk.Button checkoutToolbarButton;
		[Widget] Gtk.Button importToolbarButton;
		[Widget] Gtk.Button exportToolbarButton;
		[Widget] Gtk.Button commitToolbarButton;
		[Widget] Gtk.Button preferencesToolbarButton;
		[Widget] Gtk.Button helpToolbarButton;
		[Widget] Gtk.Button quitToolbarButton;
		[Widget] Gtk.Button shellToolbarButton;
		/// <summary> The Gsvn Log Console. </summary>
		[Widget] Gtk.TextView gsvnLogConsole;
		/// <summary> The Menu items. </summary>
		// From File Menu
		[Widget] Gtk.MenuItem MenuFileAddWorkingDirectory;
		[Widget] Gtk.MenuItem MenuFileExit;
		[Widget] Gtk.MenuItem MenuFileCloseCurrentWorkSpace;
		// From View Menu
		[Widget] Gtk.MenuItem MenuViewRefresh;
		[Widget] Gtk.MenuItem MenuViewClear;
		// From Repository Menu
		[Widget] Gtk.MenuItem MenuRepositoryCheckout;
		[Widget] Gtk.MenuItem MenuRepositoryImport;
		[Widget] Gtk.MenuItem MenuRepositoryExport;
		//[Widget] Gtk.MenuItem MenuRepositoryBrowse;		
		// From Modify Menu
		[Widget] Gtk.MenuItem MenuModifyUpdate;
		[Widget] Gtk.MenuItem MenuModifyUpdateRevision;
		[Widget] Gtk.MenuItem MenuModifyCommit;
		[Widget] Gtk.MenuItem MenuModifyMakeDirectory;
		[Widget] Gtk.MenuItem MenuModifySwitch;
		[Widget] Gtk.MenuItem MenuModifyCleanup;
		[Widget] Gtk.MenuItem MenuModifyCopyRemotetoRemote;
		[Widget] Gtk.MenuItem MenuModifyCopyRemotetoLocal;
		[Widget] Gtk.MenuItem MenuModifyMoveOnRepository;
		// From Extras Menu
		[Widget] Gtk.MenuItem MenuExtrasShell;
		[Widget] Gtk.MenuItem MenuHelpContents;
		// from query menu
		[Widget] Gtk.MenuItem MenuQueryLog;
		//[Widget] Gtk.MenuItem MenuQueryDiff;
		[Widget] Gtk.MenuItem MenuQueryInfo;
		// From Help Menu
		[Widget] Gtk.MenuItem MenuHelpWebSite;
		[Widget] Gtk.MenuItem MenuHelpUpdates;
		[Widget] Gtk.MenuItem MenuHelpAbout;
		/// <summary> Scrolled Windows uses in the app. There are two main windows, the window
		/// where display the directory tree and the window where we display the file tree.
		/// </summary>
		[Widget] Gtk.ScrolledWindow workingDirectoryWindow;
		[Widget] Gtk.ScrolledWindow browseFilesWindow;		

		//[Widget] Gtk.Notebook workingDirectorysNoteBook;		
		#endregion // End of widget's declarative zone
		
		/// <summary> Declarative zone, private members. </summary>		
		#region 
		private string wc_dir;		
		private checkOutDialog checkoutdialog;
		private AboutWindow aboutwindow;
		private HelpWindow helpwindow;
		private ImportDialog importdialog;
		private ExportDialog exportdialog;
		private CommitDialog commitdialog;
		private CommitEntryDialog commitEntryDialog;
		private UpdateRevisionDialog updaterevisiondialog;
		private ExitDialog exitdialog;
		private WorkingDirectoryDialog workingdirectorydialog;
		//private WaitWindow waitwindow;
		private PropertyEditor propertyeditor;	
		private RevertConfirmDialog revertConfirmDialog;
		private RevertCompletedDialog revertCompletedDialog;	
		private AddConfirmDialog addConfirmDialog;
		private AddRecursiveConfirmDialog addRecursiveConfirmDialog;
		private DeleteConfirmDialog deleteConfirmDialog;
		private ViewDialog viewDialog;
		//private BlameDialog blameDialog;
		private CopyLocaltoLocalDialog copyLocaltoLocalDialog;
		private CopyLocaltoRemoteDialog copyLocaltoRemoteDialog;
		private CopyRemotetoLocalDialog copyRemotetoLocalDialog;
		private CopyRemotetoRemoteDialog copyRemotetoRemoteDialog;
		private MakeDirectoryDialog makeDirectoryDialog;
		private RenameDialog renameDialog;
		private InfoDialog infoDialog;
		//private LogDialog logDialog;
		private MoveLocalDialog moveLocalDialog;
		private MoveRemoteDialog moveRemoteDialog;
		//private CommandOutputWindow commandOutputWindow;
		private CleanupConfirmDialog cleanupConfirmDialog;
		private SwitchDialog switchDialog;
		private PreferencesDialog preferencesDialog;
		private NotTextFileDialog notTextFileDialog;
		private CreateRepositoryDialog createRepositoryDialog;
		
		private Log log;
		private Config config;
		
		////////////////////////////////////////////////////////	
		// MAIN CLIENT										  //  		
		////////////////////////////////////////////////////////
		
		//private Gtk.HBox hbox;
	    //private	Terminal term;
	    //private Gtk.Label ShellLabel; 
	    //private string selected_dir;
	    //private string selected_entry;
	    private TreeView DirTree;
	    private TreeView FileTree;
	    #endregion // End of Private Members zone.
	    
	    /// <summary> Public members used on App.
	    #region 
	    public WorkingCopy workingcopy;
	    public SvnClient svn;	
	    public SvnConfig svnconfig;
	    #endregion // End of Public members.

		/// <summary> Gettext Resource. Is used for translate Strings from gsvn. </summary>
		private static GettextResourceManager catalogo = new GettextResourceManager("Gsvn");  
	
		/// <summary> App's entry point </summary>
		/// <param name="args">Command line args</param>
		/// <returns> void </returns>
	    public static void Main (string[] args)
	    
	    {
        	
			new gsvn (args);       
	    }
	    
	    /// <sumary> Inicialize path for icons </sumary>
		public static string getPath() {
			
			string[] argv;
			argv = Environment.GetCommandLineArgs();
			
			return (argv[0].Substring(0, argv[0].LastIndexOf('/') + 1) + "doc/");
		}
	    
	    /// <summary> Show Welcome to user on a page. </summary>	   
	    public void showWelcome() {
	    
	    	Gtk.Frame frame;			
			WebControl web;			
			
			web = new WebControl();
			web.Show();
						
			frame = new Frame();
			frame.Add(web);
			//frame.HeightRequest = 600;
			
			web.LoadUrl( getPath() + "welcome.html");
			//web.LoadUrl( "/home/miguel/Master2/Projecte/src/doc/welcome.html");
			web.ShowAll();   
    	
	    	Gtk.Label label;
	    	label = new Gtk.Label(catalogo.GetString("Welcome!"));
	    		
	    	//browseFilesWindow.Add(frame);
	    	//browseFilesWindow.Add(label);
			browseFilesWindow.AddWithViewport(frame);
	    	browseFilesWindow.AddWithViewport(label);
						
	    	browseFilesWindow.ShowAll();
	    	
	    
	    }

	    /// <summary> This method is called when user logout on a tab shell. </summary>	 
	    private void OnShellExited (object o, EventArgs args) {
	   
                Console.WriteLine (catalogo.GetString("shell exited"));
				inf_notebook.RemovePage( inf_notebook.CurrentPage );
                
        }	        
  	    
  	    /// <summary> Show Welcome to user on a page. </summary>	 
	    public void prepareTerminal() {
	    	    
	    	HBox shellHBox;
	    
	    	Gtk.Alignment alig1;
	    	Gtk.Alignment alig2;
	    
	    	shellHBox = new HBox(false, 0); 
	    	//shellHBox = new HBox();
	    	
	    	shellHBox.BorderWidth = 2;
	    	   		       		    
	    	Terminal terminal = new Terminal();
	    	
	    	alig1 = new Gtk.Alignment(0,0,0,0);
	    	alig2 = new Gtk.Alignment(0,0,0,0);
	    	
	    	shellHBox.Spacing = 0;
	    	
			terminal.CursorBlinks = true;
			terminal.MouseAutohide = true;
			terminal.ScrollOnKeystroke = true;
			terminal.DeleteBinding = TerminalEraseBinding.Auto;
			terminal.BackspaceBinding = TerminalEraseBinding.Auto;
			terminal.Encoding = "iso-8859-1";
			terminal.FontFromString = "Monospace 8";
			terminal.ChildExited += new EventHandler (OnShellExited);
					
       		Gdk.Color black = new Gdk.Color ();
            Gdk.Color.Parse ("black", ref black);
            Gdk.Color white = new Gdk.Color ();
            Gdk.Color[] palette = new Gdk.Color[128];
            Gdk.Color.Parse ("white", ref white);
                    	
            terminal.SetColors(black, white, palette, 0);
            //terminal.SetDefaultColors();
			
			string[] argv = Environment.GetCommandLineArgs ();

			string[] envv = new string [Environment.GetEnvironmentVariables ().Count];
			int i = 0;
		
			foreach (DictionaryEntry e in Environment.GetEnvironmentVariables ()) {
				if (e.Key == "" || e.Value == "")
					continue;
					
				string tmp = String.Format ("{0}={1}", e.Key, e.Value);
				envv[i] = tmp;
				i ++;
			}
		
			int pid = terminal.ForkCommand (Environment.GetEnvironmentVariable ("SHELL"), argv, envv, Environment.CurrentDirectory, false, true, true);
			Console.WriteLine("Pid Terminal {0} ", pid );

			VScrollbar vscroll = new VScrollbar (terminal.Adjustment);		
			Console.WriteLine("VScrollbar ok" );
			
			shellHBox.PackStart(terminal,true,true,0);
			shellHBox.PackStart(vscroll,false,false,0);
			
			//shellHBox.PackStart(terminal);
			//shellHBox.PackStart(vscroll);
			
			inf_notebook.AppendPage(shellHBox, new Label("Shell") );
			//inf_notebook.Add(shellHBox);
			Console.WriteLine("AppendPage ok" );
			
			shellHBox.ShowAll();  		
        
        } 
        
        /// <summary> Prepare Tool bar items and connect their events. </summary>	 
		void prepareToolbar() {
    
	    	addDirToolbarButton.Clicked 	+= new EventHandler(OnMenuFileAddWorkingDirectoryActivate);
	    	checkoutToolbarButton.Clicked 	+= new EventHandler(OnMenuRepositoryCheckoutActivate);
	    	importToolbarButton.Clicked		+= new EventHandler(OnMenuRepositoryImportActivate);
	    	exportToolbarButton.Clicked		+= new EventHandler(OnMenuRepositoryExportActivate);
	    	commitToolbarButton.Clicked		+= new EventHandler(OnMenuModifyCommitActivate);
	    	preferencesToolbarButton.Clicked += new EventHandler(OnMenuExtrasPreferencesActivate);
	    	helpToolbarButton.Clicked		+= new EventHandler(OnMenuHelpContentsActivate);
	    	quitToolbarButton.Clicked 		+= new EventHandler(OnMenuFileExitActivate);
	    	shellToolbarButton.Clicked		+= new EventHandler(OnMenuExtrasShellActivate);
    	}  
    	
    	/// <summary> Change the sensitive value from toolbar Items export and commit.
    	/// When Main program starts, and no have working directoyr, user can not realize
    	/// commit and export operations.
    	void prepareToolbarItems(bool _value) {
    	
    		exportToolbarButton.Sensitive = _value;
    		commitToolbarButton.Sensitive = _value;
    		
    	}
    	
    	/// <summary> Set menu sensitive property. </summary>	
    	/// <param name="_value">Value to set on Buttons.</param> 
    	void prepareMenuItems(bool _value) {
    	
    		MenuFileAddWorkingDirectory.Sensitive = !(_value);
    		MenuRepositoryExport.Sensitive = _value;
    		MenuModifyCommit.Sensitive = _value;
    		MenuModifyCleanup.Sensitive = _value;
    		MenuModifySwitch.Sensitive = _value;
    		MenuModifyMakeDirectory.Sensitive = _value;
    		MenuQueryLog.Sensitive = _value;
    		MenuQueryInfo.Sensitive = _value;
    		MenuFileCloseCurrentWorkSpace.Sensitive = _value;
    		MenuModifyUpdate.Sensitive = _value;
    		MenuModifyUpdateRevision.Sensitive = _value;
    		MenuModifyCopyRemotetoRemote.Sensitive = _value;
    	   	MenuModifyCopyRemotetoLocal.Sensitive = _value;
			MenuModifyMoveOnRepository.Sensitive = _value;

    	}
    	
    	/// <summary> Connect a menu entry with the associated status bar message.</summary>	         
		void prepareMenuItemsEvents() {

	        // connect menu items selected with status bar messages.
	        MenuFileExit.Selected 					+= new EventHandler(MessageBarExit);
	        MenuFileExit.Deselected					+= new EventHandler(MessageBarClear);       	
	        MenuFileAddWorkingDirectory.Selected 	+= new EventHandler(MessageBarAddWorking);
	        MenuFileAddWorkingDirectory.Deselected	+= new EventHandler(MessageBarClear);       
	        MenuViewRefresh.Selected				+= new EventHandler(MessageBarRefresh);
	        MenuViewRefresh.Deselected				+= new EventHandler(MessageBarClear);       
	        MenuViewClear.Selected 					+= new EventHandler(MessageBarClearLog);
	        MenuViewClear.Deselected				+= new EventHandler(MessageBarClear);       
	        MenuRepositoryCheckout.Selected 		+= new EventHandler(MessageBarCheckout);
	        MenuRepositoryCheckout.Deselected		+= new EventHandler(MessageBarClear);       
	        MenuRepositoryImport.Selected 			+= new EventHandler(MessageBarImport);
	        MenuRepositoryImport.Deselected			+= new EventHandler(MessageBarClear);       
	        MenuRepositoryExport.Selected			+= new EventHandler(MessageBarExport);
	        MenuRepositoryExport.Deselected			+= new EventHandler(MessageBarClear); 
	        MenuModifyUpdate.Selected				+= new EventHandler(MessageBarUpdate);
	        MenuModifyUpdate.Deselected				+= new EventHandler(MessageBarClear);       
	        MenuModifyUpdateRevision.Selected		+= new EventHandler(MessageBarUpdateRev);
	        MenuModifyUpdateRevision.Deselected		+= new EventHandler(MessageBarClear);       
	        MenuModifyCommit.Selected				+= new EventHandler(MessageBarCommit);
	        MenuModifyCommit.Deselected				+= new EventHandler(MessageBarClear);       
	        MenuExtrasShell.Selected				+= new EventHandler(MessageBarShell);
	        MenuExtrasShell.Deselected				+= new EventHandler(MessageBarClear);       
	        MenuHelpContents.Selected				+= new EventHandler(MessageBarContents);
	        MenuHelpContents.Deselected				+= new EventHandler(MessageBarClear);       
	        MenuHelpWebSite.Selected				+= new EventHandler(MessageBarWebSite);
	        MenuHelpWebSite.Deselected				+= new EventHandler(MessageBarClear);       
	        MenuHelpUpdates.Selected				+= new EventHandler(MessageBarUpdates);
	        MenuHelpUpdates.Deselected				+= new EventHandler(MessageBarClear);       
	        MenuHelpAbout.Selected					+= new EventHandler(MessageBarAbout);
	        MenuHelpAbout.Deselected				+= new EventHandler(MessageBarClear);       
	       
	    }       

		/// <summary> Class constructor </summary>
	    public gsvn (string[] args) 
	    
	    {
	    	//uint id1, id2;  
	    	string welcomeMessage = catalogo.GetString("Welcome to Gsvn!");   	
  	      
	    	Application.Init();
	    	
	    	Glade.XML gxml = new Glade.XML ("gui.glade", "mainWindow");
	        gxml.Autoconnect (this);
	        log = new Log(1, gsvnLogConsole);
	    	
	    	log.Debug(catalogo.GetString("Creating main window..."));
	    	log.Debug(catalogo.GetString("Starting Gsvn..."));
	    	
	    	config = new Config();
	        svnconfig = new SvnConfig();
	               
	        // Definining status bar messages for menu entrys
	        
			prepareMenuItemsEvents();
			prepareToolbar();
			prepareToolbarItems(false);
			prepareMenuItems(false);
			
			showWelcome();

	        log.Debug(catalogo.GetString("Application Started..."));    

	        Application.Run();	  

	    }
	    
		/// <summary> Clear the Gsvn status bar. </summary>
		public void MessageBarClear(object o, EventArgs args) {
		    
		  	uint id1, id2;
		    		    
		   	id1 = statusBar.GetContextId("");
		   	id2 = statusBar.Push(id1,"");
		    	
		}	
	    
	   	/// <summary> Clear the Gsvn status bar. </summary>
	    public void MessageBarClear_() {
	    
	    	uint id1, id2;
	    		    
	    	id1 = statusBar.GetContextId("");
	    	id2 = statusBar.Push(id1,"");
	    	
	    }
	    
	    public void OnGsvnMainWindowDestroy(object o, EventArgs args)
	    
	    {	
	    	exitdialog = new ExitDialog(this);
	    	exitdialog.Show();
	    }
	    
	    /// <summary> Confirm user wants to exit when destroy window event is called. </summary>
	    public void MessageBarExit(object o, EventArgs args) {
	    
	    	uint id1, id2;
	    	string message = catalogo.GetString("Exit Gsvn");
	    
	    	id1 = statusBar.GetContextId(message);
	    	id2 = statusBar.Push(id1, message); 
	    
	    }
	    
	    /// <summary> 'Add Working Directory' message bar </summary>
	    public void MessageBarAddWorking(object o, EventArgs args) {
	    
	    	uint id1, id2;
	    	string message = catalogo.GetString("Add working directory to project");
	    	
	    	id1 = statusBar.GetContextId(message);
	    	id2 = statusBar.Push(id1, message); 
	    
	    }	    
	    
	    /// <summary> 'Refresh' message bar </summary>
	    public void MessageBarRefresh(object o, EventArgs args) {
	    
	    	uint id1, id2;
	    	string message = catalogo.GetString("Refresh files and directories view");
	    	
	    	id1 = statusBar.GetContextId(message);
	    	id2 = statusBar.Push(id1, message); 
	    
	    }	    
	    
	    /// <summary> 'Clear log' message bar </summary>
	    public void MessageBarClearLog(object o, EventArgs args) {
	    
	    	uint id1, id2;
	    	string message = catalogo.GetString("Clear Gsvn log console");
	    	
	    	id1 = statusBar.GetContextId(message);
	    	id2 = statusBar.Push(id1, message); 
	    
	    }	    	    
	    
	    /// <summary> 'Checkout' message bar </summary>
	    public void MessageBarCheckout(object o, EventArgs args) {
	    
	    	uint id1, id2;
	    	string message = catalogo.GetString("Checkout lastest copy from repository");
	    	
	    	id1 = statusBar.GetContextId(message);
	    	id2 = statusBar.Push(id1, message); 
	    
	    }
	    
	    /// <summary> 'Import' status bar message </summary>
	    public void MessageBarImport(object o, EventArgs args) {
	    
	    	uint id1, id2;
	    	string message = catalogo.GetString("Import local directory to repository");
	    	
	    	id1 = statusBar.GetContextId(message);
	    	id2 = statusBar.Push(id1, message); 
	    
	    }	    	    	    	    	 
	    
	    /// <summary> 'Export' status bar message </summary>
	    public void MessageBarExport(object o, EventArgs args) {
	    
	    	uint id1, id2;
	    	string message = catalogo.GetString("Export a clean copy of a working directory");
	    	
	    	id1 = statusBar.GetContextId(message);
	    	id2 = statusBar.Push(id1, message); 
	    
	    }	    
	    
 		/// <summary> 'Update' status bar message </summary>   
	    public void MessageBarUpdate(object o, EventArgs args) {
	    
	    	uint id1, id2;
	    	string message = catalogo.GetString("Update local copy with latest version on repository");
	    	
	    	id1 = statusBar.GetContextId(message);
	    	id2 = statusBar.Push(id1, message); 
	    
	    }	    	    	    	    
	    
	    /// <summary> 'Update revision' status bar message </summary>
	    public void MessageBarUpdateRev(object o, EventArgs args) {
	    
	    	uint id1, id2;
	    	string message = catalogo.GetString("Update local copy with a particular revision on repository");
	    	
	    	id1 = statusBar.GetContextId(message);
	    	id2 = statusBar.Push(id1, message); 
	    
	    }	    
	    
	    /// <summary> 'Commit' status bar message </summary>
	    public void MessageBarCommit(object o, EventArgs args) {
	    
	    	uint id1, id2;
	    	string message = catalogo.GetString("Commit changes to repository");
	    	
	    	id1 = statusBar.GetContextId(message);
	    	id2 = statusBar.Push(id1, message); 
	    
	    }	    	    	    	
    	
    	/// <summary> 'Shell' status bar message </summary>
	    public void MessageBarShell(object o, EventArgs args) {
	    
	    	uint id1, id2;
	    	string message = catalogo.GetString("Open an embedded shell on Gsvn");
	    	
	    	id1 = statusBar.GetContextId(message);
	    	id2 = statusBar.Push(id1, message); 
	    
	    }    
  
	    /// <summary> 'Help contets' status bar message </summary>
	    public void MessageBarContents(object o, EventArgs args) {
	    
	    	uint id1, id2;
	    	string message = catalogo.GetString("Open a window with help topics");
	    	
	    	id1 = statusBar.GetContextId(message);
	    	id2 = statusBar.Push(id1, message); 
	    
	    }	    	     	   
	    
	    /// <summary> 'Web Site' status bar message </summary>
	    public void MessageBarWebSite(object o, EventArgs args) {
	    
	    	uint id1, id2;
	    	string message = catalogo.GetString("Visit Gsvn web site");
	    	
	    	id1 = statusBar.GetContextId(message);
	    	id2 = statusBar.Push(id1, message); 
	    
	    }	    	      	
	    
	    /// <summary> 'Updates' status bar message </summary>
	    public void MessageBarUpdates(object o, EventArgs args) {
	    
	    	uint id1, id2;
	    	string message = catalogo.GetString("Check for gsvn updates");
	    	
	    	id1 = statusBar.GetContextId(message);
	    	id2 = statusBar.Push(id1, message); 
	    
	    }
	    
	   	/// <summary> 'About' status bar message </summary> 
	   	public void MessageBarAbout(object o, EventArgs args) {
	    
	    	uint id1, id2;
	    	string message = catalogo.GetString("Shows an about window");
	    	
	    	id1 = statusBar.GetContextId(message);
	    	id2 = statusBar.Push(id1, message); 
	    
	    }	
	   
	    
	    // buttons events from toolbar.
	    
	    /// <summary> This event is called when user clicks on Toolbar -> exit </summary>
		public void On_Toolbar_Exit_Clicked(object o, EventArgs args) {
		
			exitdialog = new ExitDialog(this);
			exitdialog.Show();
			
		}
      
	    /// <summary>	Method asociated with glade graphical interface. 
	    /// 			This method is invoked when user clicks on Close App.
	    /// </summary>
	    public void OnWindowDeleteEvent (object o, DeleteEventArgs args) 
	    
	    {

	    	log.Debug(catalogo.GetString("Quit Gsvn"));
  			    
	    	Application.Quit ();
	        args.RetVal = true;
	    }

		
		
		/// <summary>	Method asociated with glade graphical interface.
		///				This method is invoked when user clicks on app menu -> file -> Add Working Directory
		/// </summary>	
		public void OnMenuFileAddWorkingDirectoryActivate(object o, EventArgs args)
		
		{
			workingdirectorydialog = new WorkingDirectoryDialog(this);
			workingdirectorydialog.Show();
			
		}
		
		/// <summary> Clear Widgtes from BrowseFileScrolledWindow </summary>
		public void clearBrowseFilesWindow() {
			
				
			foreach (Gtk.Widget w in browseFilesWindow.Children) {
				browseFilesWindow.Remove(w);
			}
			browseFilesWindow.VscrollbarPolicy = Gtk.PolicyType.Automatic;
			browseFilesWindow.HscrollbarPolicy = Gtk.PolicyType.Automatic;
			browseFilesWindow.HideAll();
				
		}		
		
		/// <summary> Clear Widgtes from BrowseDirScrolledWindow </summary>	
		public void clearBrowseDirWindow() {
			
			foreach (Gtk.Widget w in workingDirectoryWindow.Children) {
				workingDirectoryWindow.Remove(w);
			}
			
		}
		
		/// <summary> This event is called when user clicks on File -> 'Close current workspace' entry. </summary>
		public void OnMenuFileCloseCurrentWorkSpaceActivate(object o, EventArgs args)
		
		{	
				this.clearBrowseDirWindow();
				this.clearBrowseFilesWindow();
				this.workingcopy = null;
				this.svn = null;
				this.prepareMenuItems(false);
		}	

		/// <summary>	Method asociated with glade graphical interface.
		///				This method is invoked when user clicks on app menu -> file -> Exit
		/// </summary>	
		public void OnMenuFileExitActivate(object o, EventArgs args)
		
		{		
			exitdialog = new ExitDialog(this);
			exitdialog.Show();

		}
		
		/// <summary> This method is used to draw the File Tree List. </summary>
		/// <param name="dir"> Dir to browse </param>
		/// <returns> TreeView with File Tree List. </return>
		private TreeView GetFileList(string dir) {			
			
			TreeStore FileList = null;
			
			workingcopy = new WorkingCopy(this.wc_dir);
			
			FileList = new TreeStore(typeof(Pixbuf), typeof(string), typeof(string), typeof(string),
									 typeof(string), typeof(string), typeof(string), typeof(string), typeof(string),
									 typeof(string), typeof(string), typeof(string), typeof(string), typeof(string));
									 
			// primero meto los directorios.
			string[] dirEntries = System.IO.Directory.GetDirectories(dir);	
					
			foreach (string _dir in dirEntries) {
				if (_dir.IndexOf(".svn") == -1) {
				
					// get entry info with svn info
					InfoList info;			
					info = workingcopy.info(_dir);
				
					string _entry = _dir.Substring(_dir.LastIndexOf('/')+1, _dir.Length - _dir.LastIndexOf('/') -1);
					string __entry = _dir.Substring(0, _dir.LastIndexOf('/'));

					FileList.AppendValues(new Gdk.Pixbuf(GsvnIcons.Folder(config)),
											 __entry, 
											 _entry, 
											 info.getInfoElement("URL"), 
											 info.getInfoElement("Revision"), 
											 info.getInfoElement("Status"), 
											 info.getInfoElement("Last Changed Author"), 
											 info.getInfoElement("Last Changed Rev"), 
											 info.getInfoElement("Last Changed Date"), 
											 info.getInfoElement("Text Last Updated"), 
											 info.getInfoElement("Properties Last Updated"), 
											 info.getInfoElement("Checksum"), 
											 info.getInfoElement("Schedule"));					

				}
			}
						
			string[] fileEntries = System.IO.Directory.GetFileSystemEntries(dir);	
			foreach (string file in fileEntries) {	
				if (System.IO.Directory.Exists(file)) ;
				else {	
					if (file.IndexOf(".svn") == -1) {
						
						// get entry info with svn info
						InfoList info;			
						info = workingcopy.info(file);
					
						string _entry = file.Substring(file.LastIndexOf('/')+1, file.Length - file.LastIndexOf('/') -1);
						string __entry = file.Substring(0, file.LastIndexOf('/'));
						FileList.AppendValues(new Gdk.Pixbuf(GsvnIcons.getIcon(file,config)),
															 __entry, 
															 _entry, 
															 info.getInfoElement("URL"), 
															 info.getInfoElement("Revision"), 
															 info.getInfoElement("Status"), 
															 info.getInfoElement("Last Changed Author"), 
															 info.getInfoElement("Last Changed Rev"), 
															 info.getInfoElement("Last Changed Date"), 
															 info.getInfoElement("Text Last Updated"), 
															 info.getInfoElement("Properties Last Updated"), 
															 info.getInfoElement("Checksum"), 
															 info.getInfoElement("Schedule"));
					}
				}
			}
				
			MessageBarClear_();
				
			FileTree = new TreeView (FileList);
			return FileTree;	
									 							 
		}			
		
		/// <summary> This method is used for browse a Dir on Working Directory Window. </summary>
		public void browseDir(TreeStore store, TreeIter parent, string dir) {
		
			string[] entries = System.IO.Directory.GetDirectories(dir);
			
			foreach (string entry in entries) {
				if (entry.IndexOf(".svn") != -1 )  { ; }
				else {				
					string _entry = entry.Substring(dir.LastIndexOf('/')+1, entry.Length - dir.LastIndexOf('/') -1);
					TreeIter iter = store.AppendValues(parent, new Gdk.Pixbuf(GsvnIcons.Folder(config)), _entry, entry );
					browseDir(store, iter, entry+"/");
				}			
			}				
		}
		
		/// <summary> This method is called when user clicks on Property Editor,
		/// in the entry context pop up menu. </summary>
		public void OnPopupMenuPropertyEditorActivated(object o, EventArgs args) {
		
			PropList properties;
			string selected = "";
			TreeIter iter;
            TreeModel model;
			
			// we load the properties list. 
			// we search what entry on browse file is activated
			if (FileTree.Selection.GetSelected (out model, out iter)) {    
            	selected = (string) model.GetValue (iter, 1) + "/" + (string) model.GetValue (iter, 2);
            }
			
			properties = workingcopy.propList(selected);
			
			System.Console.WriteLine(properties.lengthProperties());
			
			propertyeditor = new PropertyEditor(properties, this);
			propertyeditor.Show();				
			
		}
		
		/// <summary> This method is called when user clicks on Revert,
		/// in the entry context pop up menu. </summary>
		public void OnPopupMenuRevertActivated(object o, EventArgs args) {
		
			revertConfirmDialog = new RevertConfirmDialog(this);
			revertConfirmDialog.Show();
			
		}
		
		/// <summary> This method is called when user clicks on Add
		/// in the entry context pop up menu. </summary>
		public void OnPopupMenuAddActivated(object o, EventArgs args) {
		
			// we seek what entry on filetree is selected.
			string _entry = "";
			TreeIter iter;
            TreeModel model;
			
			// we load the properties list. 
			// we search what entry on browse file is activated
			if (FileTree.Selection.GetSelected (out model, out iter)) {    
            	_entry = (string) model.GetValue (iter, 1) + "/" + (string) model.GetValue (iter, 2);
            }
            
			workingcopy._add(_entry, false);
			addConfirmDialog = new AddConfirmDialog();
			addConfirmDialog.Show();
			
		}

		/// <summary> This method is called when user clicks on 'Add Recursive' entry,
		/// in the entry context pop up menu. </summary>				
		public void OnPopupMenuAddRecursiveActivated(object o, EventArgs args) {
		
			// we seek what entry on filetree is selected.
			string _entry = "";
			TreeIter iter;
            TreeModel model;
			
			// we load the properties list. 
			// we search what entry on browse file is activated
			if (FileTree.Selection.GetSelected (out model, out iter)) {    
            	_entry = (string) model.GetValue (iter, 1) + "/" + (string) model.GetValue (iter, 2);
            }
            
			workingcopy._add(_entry, true);			
			addRecursiveConfirmDialog = new AddRecursiveConfirmDialog();
			addRecursiveConfirmDialog.Show();
			
		}
		
		/// <summary> This method is called when user clicks on 'View' entry,
		/// in the entry context pop up menu. </summary>	
		public void OnPopupMenuViewActivated(object o, EventArgs args) {
		
			string _selected = "";
			
			_selected = this.getFileTreeSelected();
		
			if (!WorkingCopy.isTextFile(_selected)) {
				notTextFileDialog = new NotTextFileDialog();
				notTextFileDialog.Show();			
			}		
			else {
				viewDialog = new ViewDialog(workingcopy.cat(_selected));
				viewDialog.Show();
			}
			
		}

		/// <summary> This method is called when user clicks on 'Cat' entry,
		/// in the entry context pop up menu. </summary>					
		public void OnPopupMenuCatActivated(object o, EventArgs args) {
		
			string _selected = "";
			
			_selected = this.getFileTreeSelected();		
		
			if (!WorkingCopy.isTextFile(_selected)) {
				notTextFileDialog = new NotTextFileDialog();
				notTextFileDialog.Show();			
			}
			
			else {		
				this.postCatDialog(_selected);
			}
			
		}
			
		/// <summary> This method is called when user clicks on 'Blame' entry,
		/// in the entry context pop up menu. </summary>							
		public void OnPopupMenuBlameActivated(object o, EventArgs args) {
		
			postBlameDialog();
			
		}
		
		/// <summary> This method is called when user clicks on 'Delete' entry,
		/// in the entry context pop up menu. </summary>			
		public void OnPopupMenuDeleteActivated(object o, EventArgs args) {
		
			string _selected = "";
			
			_selected = this.getFileTreeSelected();
			
			workingcopy.delete(_selected);
			
			deleteConfirmDialog = new DeleteConfirmDialog();
			deleteConfirmDialog.Show();
			
			
		}
		
		/// <summary> This method is called when user clicks on 'Update' entry,
		/// in the entry context pop up menu. </summary>			
		public void OnPopupMenuUpdateActivated(object o, EventArgs args) {
		
			string[] _params = new string[10];
			string _selected = "";
			int i = 0;
			
			_selected = this.getFileTreeSelected();
			
			if (_selected == "") 
				_selected = this.workingcopy.workingdirectory;
				
			_params[i++] = _selected;
			
			if (svn == null) {
				svn = new SvnClient();
			}	
			
			svn.execSubcommand("UPDATE_ENTRY", _params);			
			
		}
		
		/// <summary> This method is called when user clicks on 'Move Local' entry,
		/// in the entry context pop up menu. </summary>	
		public void OnPopupMenuMoveLocalActivated(object o, EventArgs args) {
		
			moveLocalDialog = new MoveLocalDialog(this, getFileTreeSelected() );
			moveLocalDialog.Show();
			
		}
		
		/// <summary> This method is called when user clicks on 'Rename' entry,
		/// in the entry context pop up menu. </summary>			
		public void OnPopupMenuRenameActivated(object o, EventArgs args) {
		
			renameDialog = new RenameDialog(this);
			renameDialog.Show();
			
		}		
		
		/// <summary> This method is called when user clicks on 'Log' entry,
		/// in the entry context pop up menu. </summary>			
		public void OnPopupMenuLogActivated(object o, EventArgs args) {
		
			postLogDialog();
			
		}		
		
		/// <summary> This method is called when user clicks on 'Info' entry,
		/// in the entry context pop up menu. </summary>			
		public void OnPopupMenuInfoActivated(object o, EventArgs args) {
		
			string selected = getFileTreeSelected();
		
			infoDialog = new InfoDialog(this.workingcopy, selected);
			infoDialog.Show();
			
		}	
		
		/// <summary> This method is called when user clicks on 'Edit' entry,
		/// in the entry context pop up menu. </summary>			
		public void OnPopupMenuEditActivated(object o, EventArgs args) {
		
			string _selected = "";
			
			_selected = this.getFileTreeSelected();		
		
			if (!WorkingCopy.isTextFile(_selected)) {
				notTextFileDialog = new NotTextFileDialog();
				notTextFileDialog.Show();			
			}
			
			else {			
				workingcopy.edit(_selected);		
			}
			
			
		}		
		
		/// <summary> This method is called when user clicks on 'Commit' entry,
		/// in the entry context pop up menu. </summary>			
		public void OnPopupMenuCommitActivated(object o, EventArgs args) {
		
			commitEntryDialog = new CommitEntryDialog(this);
			commitEntryDialog.Show();				
			
		}	
		
		/// <summary> This method is called when user clicks on 'Copy local to local' entry,
		/// in the entry context pop up menu. </summary>			
		public void OnPopupMenuCopyLocaltoLocalActivated(object o, EventArgs args) {
		
			string _selected = "";
			
			_selected = this.getFileTreeSelected();
		
			copyLocaltoLocalDialog = new CopyLocaltoLocalDialog(_selected, this);
			copyLocaltoLocalDialog.Show();
			
		}
		
		/// <summary> This method is called when user clicks on 'Copy local to remote' entry,
		/// in the entry context pop up menu. </summary>			
		public void OnPopupMenuCopyLocaltoRemoteActivated(object o, EventArgs args) {
		
			string _selected = "";
			
			_selected = this.getFileTreeSelected();
		
			copyLocaltoRemoteDialog = new CopyLocaltoRemoteDialog(_selected, this);
			copyLocaltoRemoteDialog.Show();
			
		}
		
		/// <summary> This method define the "Copy Menu" </summary>	
		/// <returns> A menu that contains the different Copy types. </returns>		
		public Menu MenuCopy(bool _sensitive) {
		
			Gtk.Menu menu;
			
			menu = new Menu();
			
			ImageMenuItem item;
			
			item = new ImageMenuItem(catalogo.GetString("Local to local"));
			item.Sensitive = _sensitive;
			item.Activated += new EventHandler(OnPopupMenuCopyLocaltoLocalActivated);
			menu.Append(item);
			
			item = new ImageMenuItem(catalogo.GetString("Local to remote"));
			item.Activated += new EventHandler(OnPopupMenuCopyLocaltoRemoteActivated);
			item.Sensitive = _sensitive;
			menu.Append(item);
			
			return menu;
			
		 }
															
		
		/// <summary> This method defines the Main Popup Menu for a Versioned item
		/// on that Dir. </summary>		
		public void FileEntryPopupMenuVersioned(EventButton ev) {
			
			Gtk.Menu menu;		
			menu = new Menu ();
			
			ImageMenuItem item;
			Gtk.Image image;
			image = new Gtk.Image();
			
			TreeIter iter;
			TreeModel model;
			string _entry = "";
			bool _sensitive = false;
			           
			if (FileTree.Selection.GetSelected (out model, out iter))     
				_entry = (string) model.GetValue (iter, 1) + "/" + (string) model.GetValue (iter, 2);
				
			// if entry is directory, we append "Del Recursive" Menu Entry.
			if (System.IO.Directory.Exists(_entry))
				_sensitive = true;			
		
			// view entry
			item = new ImageMenuItem (catalogo.GetString("View (Local)"));
			image = new Gtk.Image();
			image.SetFromStock(Stock.ZoomIn, IconSize.SmallToolbar);
			item.Activated += new EventHandler (OnPopupMenuViewActivated);	
			item.Image = image;
			menu.Append(item);
			
			// view entry
			item = new ImageMenuItem (catalogo.GetString("Cat (Remote)"));
			image = new Gtk.Image();
			image.SetFromStock(Stock.ZoomIn, IconSize.SmallToolbar);
			item.Activated += new EventHandler (OnPopupMenuCatActivated);	
			item.Image = image;
			menu.Append(item);			
			
			// blame entry
			item = new ImageMenuItem (catalogo.GetString("Blame"));
			item.Activated += new EventHandler (OnPopupMenuBlameActivated);	
			menu.Append(item);		
				
			// edit entry
			item = new ImageMenuItem (catalogo.GetString("Edit"));
			item.Activated += new EventHandler(OnPopupMenuEditActivated);
			menu.Append(item);
			menu.Append(new Gtk.SeparatorMenuItem() );

			// add entry
			item = new ImageMenuItem (catalogo.GetString("Add")); // only if is not versioned
			item.Sensitive = false;
			image = new Gtk.Image();
			image.SetFromStock(Stock.Add, IconSize.SmallToolbar);
			item.Image = image;
			menu.Append(item);
			
			// add recursive entry
			item = new ImageMenuItem (catalogo.GetString("Add Recursive")); // only if is directory
			item.Sensitive = false;
			menu.Append(item);				
			menu.Append(new Gtk.SeparatorMenuItem() );		
			
			// delete entry
			item = new ImageMenuItem (catalogo.GetString("Delete")); // only if it is versioned
			image = new Gtk.Image();
			image.SetFromStock(Stock.Delete, IconSize.SmallToolbar);
			item.Image = image;
			item.Activated += new EventHandler (OnPopupMenuDeleteActivated);
			menu.Append(item);
			menu.Append(new Gtk.SeparatorMenuItem() );

			// update entry
			item = new ImageMenuItem (catalogo.GetString("Update"));
			image = new Gtk.Image();
			image.SetFromStock(Stock.Refresh, IconSize.SmallToolbar);
			item.Image = image;
			item.Activated += new EventHandler (OnPopupMenuUpdateActivated);
			menu.Append(item);			
			
			menu.Append(new Gtk.SeparatorMenuItem() );
			
			// Commit Entry
			item = new ImageMenuItem (catalogo.GetString("Commit"));
			image = new Gtk.Image();
			image.SetFromStock(Stock.GotoTop, IconSize.SmallToolbar);
			item.Image = image;
			item.Activated += new EventHandler (OnPopupMenuCommitActivated);
			menu.Append(item);	
			
			menu.Append(new Gtk.SeparatorMenuItem() );													
			
			// revert entry
			item = new ImageMenuItem (catalogo.GetString("Revert"));
			image = new Gtk.Image();
			image.SetFromStock(Stock.Undo, IconSize.SmallToolbar);
			item.Image = image;
			item.Activated += new EventHandler (OnPopupMenuRevertActivated);
			menu.Append(item);			
			menu.Append(new Gtk.SeparatorMenuItem() );
			
			// copy local entry
			item = new ImageMenuItem(catalogo.GetString("Copy"));
			image = new Gtk.Image();
			image.SetFromStock(Stock.Copy, IconSize.SmallToolbar);
			item.Image = image;
			item.Submenu = MenuCopy(true);
			menu.Append(item);
			
			menu.Append(new Gtk.SeparatorMenuItem() );
			
			// move local entry
			item = new ImageMenuItem (catalogo.GetString("Move Local"));
			image = new Gtk.Image();
			image.SetFromStock(Stock.Cut, IconSize.SmallToolbar);
			item.Image = image;
			item.Activated += new EventHandler (OnPopupMenuMoveLocalActivated);
			menu.Append(item);
			
			menu.Append(new Gtk.SeparatorMenuItem() );
			
			// rename entry
			item = new ImageMenuItem (catalogo.GetString("Rename"));
			item.Activated += new EventHandler (OnPopupMenuRenameActivated);
			menu.Append(item);
			menu.Append(new Gtk.SeparatorMenuItem() );
			
			// log entry 
			item = null;
			item = new ImageMenuItem (catalogo.GetString("Log"));
			image = new Gtk.Image();
			image.SetFromStock(Stock.Print, IconSize.SmallToolbar);
			item.Image = image;
			item.Activated += new EventHandler (OnPopupMenuLogActivated);
			menu.Append(item);			
			
			// info entry
			item = null;
			item = new ImageMenuItem (catalogo.GetString("Info"));
			item.Activated += new EventHandler (OnPopupMenuInfoActivated);
			menu.Append(item);
			menu.Append(new Gtk.SeparatorMenuItem() );
			
			// info entry
			item = null;
			item = new ImageMenuItem (catalogo.GetString("Property Editor"));
			image = new Gtk.Image();
			image.SetFromStock(Stock.Preferences, IconSize.SmallToolbar);
			item.Image = image;
			item.Activated += new EventHandler (OnPopupMenuPropertyEditorActivated);
			menu.Append(item);	
		
			menu.ShowAll ();
			
			menu.Popup (null, null, null, IntPtr.Zero, (ev != null) ? ev.Button : 0, (ev != null) ? ev.Time : 0);
			
		}
		
		/// <summary> This method defines the Main Popup Menu for a non-versioned item
		/// on that Dir. </summary>			
		public void FileEntryPopupMenuUnversioned(EventButton ev) {
			
			Gtk.Menu menu;		
			menu = new Menu ();
			
			ImageMenuItem item;
			Gtk.Image image;
			image = new Gtk.Image();
			
			
			TreeIter iter;
			TreeModel model;
			string _entry = "";
			bool _sensitive = false;
			           
			if (FileTree.Selection.GetSelected (out model, out iter))     
				_entry = (string) model.GetValue (iter, 1) + "/" + (string) model.GetValue (iter, 2);
						
			// view entry
			item = new ImageMenuItem (catalogo.GetString("View"));
			image = new Gtk.Image();
			image.SetFromStock(Stock.ZoomIn, IconSize.SmallToolbar);
			item.Image = image;
			item.Sensitive = false;
			item.Activated += new EventHandler(OnPopupMenuViewActivated);
			menu.Append(item);
			
			// view entry
			item = new ImageMenuItem (catalogo.GetString("Cat (Remote)"));
			image = new Gtk.Image();
			image.SetFromStock(Stock.ZoomIn, IconSize.SmallToolbar);
			item.Activated += new EventHandler (OnPopupMenuCatActivated);	
			item.Image = image;
			menu.Append(item);				
			
			// blame entry
			item = new ImageMenuItem (catalogo.GetString("Blame"));
			item.Sensitive = false;
			item.Activated += new EventHandler(OnPopupMenuBlameActivated);
			menu.Append(item);	
					
			// edit entry
			item = new ImageMenuItem (catalogo.GetString("Edit"));
			item.Sensitive = false;
			item.Activated += new EventHandler(OnPopupMenuEditActivated);
			menu.Append(item);
			menu.Append(new Gtk.SeparatorMenuItem() );
			
			// add entry
			item = new ImageMenuItem (catalogo.GetString("Add")); // only if is not versioned
			item.Activated += new EventHandler (OnPopupMenuAddActivated);
			image = new Gtk.Image();
			image.SetFromStock(Stock.Add, IconSize.SmallToolbar);
			item.Sensitive = true;
			item.Image = image;
			menu.Append(item);
			
			// add recursive entry
			item = new ImageMenuItem (catalogo.GetString("Add Recursive")); // only if is directory
			item.Sensitive = true;
			item.Activated += new EventHandler (OnPopupMenuAddRecursiveActivated);
			menu.Append(item);	
			menu.Append(new Gtk.SeparatorMenuItem() );		
			
			// delete entry
			item = new ImageMenuItem (catalogo.GetString("Delete")); // only if it is versioned
			image = new Gtk.Image();
			image.SetFromStock(Stock.Delete, IconSize.SmallToolbar);
			item.Sensitive = false;
			item.Image = image;
			item.Activated += new EventHandler (OnPopupMenuDeleteActivated);
			menu.Append(item);		
			menu.Append(new Gtk.SeparatorMenuItem() );
			
			// update entry
			item = new ImageMenuItem (catalogo.GetString("Update"));
			item.Sensitive = false;
			image = new Gtk.Image();
			image.SetFromStock(Stock.Refresh, IconSize.SmallToolbar);
			item.Image = image;			
			item.Activated += new EventHandler (OnPopupMenuUpdateActivated);			
			menu.Append(item);			
			menu.Append(new Gtk.SeparatorMenuItem() );
			
			// Commit Entry
			item = new ImageMenuItem (catalogo.GetString("Commit"));
			image = new Gtk.Image();
			item.Sensitive = false;
			image.SetFromStock(Stock.GotoTop, IconSize.SmallToolbar);
			item.Image = image;
			item.Activated += new EventHandler (OnPopupMenuCommitActivated);
			menu.Append(item);	
			
			menu.Append(new Gtk.SeparatorMenuItem() );				
			
			// revert entry
			item = new ImageMenuItem (catalogo.GetString("Revert"));
			item.Sensitive = false;
			image = new Gtk.Image();
			image.SetFromStock(Stock.Undo, IconSize.SmallToolbar);
			item.Image = image;
			menu.Append(item);
			menu.Append(new Gtk.SeparatorMenuItem() );
			
			// copy local entry
			item = new ImageMenuItem (catalogo.GetString("Copy"));
			image = new Gtk.Image();
			image.SetFromStock(Stock.Copy, IconSize.SmallToolbar);
			item.Image = image;
			item.Sensitive = false;
			item.Submenu = MenuCopy(false);
			menu.Append(item);
			
			menu.Append(new Gtk.SeparatorMenuItem() );
			
			// move entry
			item = new ImageMenuItem (catalogo.GetString("Move Local"));
			item.Sensitive = false;
			image = new Gtk.Image();
			image.SetFromStock(Stock.Cut, IconSize.SmallToolbar);
			item.Image = image;
			menu.Append(item);
			
			menu.Append(new Gtk.SeparatorMenuItem() );
			
			// rename entry
			item = new ImageMenuItem (catalogo.GetString("Rename"));
			item.Sensitive = false;

			menu.Append(item);
			menu.Append(new Gtk.SeparatorMenuItem() );

			// log entry 
			item = null;
			item = new ImageMenuItem (catalogo.GetString("Log"));
			item.Sensitive = false;
			image = new Gtk.Image();
			image.SetFromStock(Stock.Print, IconSize.SmallToolbar);
			item.Image = image;

			menu.Append(item);
						
			// info entry
			item = null;
			item = new ImageMenuItem (catalogo.GetString("Info"));
			item.Sensitive = false;

			menu.Append(item);
			menu.Append(new Gtk.SeparatorMenuItem() );
			
			// info entry
			item = null;
			item = new ImageMenuItem (catalogo.GetString("Property Editor"));
			item.Sensitive = false;
			image = new Gtk.Image();
			image.SetFromStock(Stock.Preferences, IconSize.SmallToolbar);
			item.Image = image;
			item.Activated += new EventHandler (OnPopupMenuPropertyEditorActivated);
			menu.Append(item);	
		
			menu.ShowAll ();
			
			menu.Popup (null, null, null, IntPtr.Zero, (ev != null) ? ev.Button : 0, (ev != null) ? ev.Time : 0);
			
		}		
		
		/// <summary> Defines the File Entry Popup Menu. </summary>
		public void FileEntryPopupMenu_(object o, PopupMenuArgs args) {
			

            TreeIter iter;
            TreeModel model;
            string _entry = "";
            
            if (FileTree.Selection.GetSelected (out model, out iter))   
            	_entry = (string) model.GetValue (iter, 1) + "/" + (string) model.GetValue (iter, 2);
            
            if (workingcopy.isVersioned(_entry)) 	            	
				FileEntryPopupMenuVersioned(null);
			else 			
				FileEntryPopupMenuUnversioned(null);
						
		}
		
		/// <summary> Defines the FileEntryButtonPressed Event. </summary>
		[GLib.ConnectBefore]
		void FileEntryButtonPressed(object o, ButtonPressEventArgs args)
		{		
			
			switch (args.Event.Button)
			{
				case 3: {	

				            TreeIter iter;
				            TreeModel model;
				            string _entry = "";
				            
				            if (FileTree.Selection.GetSelected (out model, out iter)) {    
				            	_entry = (string) model.GetValue (iter, 1) + "/" + (string) model.GetValue (iter, 2);

				            }
							
							if (workingcopy.isVersioned(_entry)) 		
								FileEntryPopupMenuVersioned(args.Event);
							else 
								FileEntryPopupMenuUnversioned(args.Event); 
							break;
						}
			}	
		
		}
		
		/// <summary> Defines the DirTreeButtonPressed Event. </summary>
		[GLib.ConnectBefore] 
		public void DirTreeButtonPressed (object o, EventArgs args) {
		
			foreach (Gtk.Widget w in browseFilesWindow.Children) {
				browseFilesWindow.Remove(w);
			}

            TreeIter iter;
            TreeModel model;
            string dir = "";
            
            if (DirTree.Selection.GetSelected (out model, out iter)) {    
            	dir = (string) model.GetValue (iter, 2);
             
            }
                        
            if (dir == "") 
            	dir = this.wc_dir;							
			
			TreeView FileTree = GetFileList(dir);
			FileTree.HeadersVisible = true;
			
			FileTree.AppendColumn(catalogo.GetString("Icon"), new CellRendererPixbuf (), "pixbuf", 0);
			FileTree.AppendColumn(catalogo.GetString("Path"), new CellRendererText (), "text", 1);
			FileTree.AppendColumn(catalogo.GetString("Name"), new CellRendererText (), "text", 2);
			if (config.isSetviewUrl() )
				FileTree.AppendColumn(catalogo.GetString("URL"), new CellRendererText (), "text", 3);
			if (config.isSetviewRevision() )
				FileTree.AppendColumn(catalogo.GetString("Revision"), new CellRendererText (), "text", 4);
			if (config.isSetviewStatus() )
				FileTree.AppendColumn(catalogo.GetString("Status"), new CellRendererText (), "text", 5);
			if (config.isSetviewLCAuthor() )
				FileTree.AppendColumn(catalogo.GetString("Last Changed Author"), new CellRendererText(), "text", 6);
			if (config.isSetviewLCRevision() )
				FileTree.AppendColumn(catalogo.GetString("Last Changed Revision"), new CellRendererText(), "text", 7);
			if (config.isSetviewLCDate() )
				FileTree.AppendColumn(catalogo.GetString("Last Changed Date"), new CellRendererText(), "text", 8);
			if (config.isSetviewTextUpdated() )
				FileTree.AppendColumn(catalogo.GetString("Text Last Updated"), new CellRendererText(), "text", 9);
			if (config.isSetviewPropLastChanged() )
				FileTree.AppendColumn(catalogo.GetString("Prop last changed"), new CellRendererText(), "text", 10);
			if (config.isSetviewChecksum() )
				FileTree.AppendColumn(catalogo.GetString("Checksum"), new CellRendererText(), "text", 11);	
			FileTree.AppendColumn(catalogo.GetString("Schedule"), new CellRendererText(), "text", 12);		
			
			FileTree.ButtonPressEvent += new ButtonPressEventHandler(FileEntryButtonPressed);
			FileTree.PopupMenu += new PopupMenuHandler (FileEntryPopupMenu_);
			//browseFilesWindow.Add(FileTree);
			browseFilesWindow.AddWithViewport(FileTree);
			browseFilesWindow.ShowAll();			
			MessageBarClear_();
					
		}

		/// <summary> This method is called when user Add a working copy. It draws the file
		/// Tree on Working Space  Window. </summary>
		public void createTreeCopy(string dir) {
		
			this.prepareToolbarItems(true);
			prepareMenuItems(true);		
			TreeStore tree = null;	

			tree = new TreeStore( typeof(Pixbuf), typeof(string), typeof(string) );			

			TreeIter iter = tree.AppendValues(new Gdk.Pixbuf(GsvnIcons.Server(config)), dir, dir); 			

			if (dir[dir.Length-1] != '/')
				dir = System.String.Concat(dir,"/");			

			browseDir(tree, iter, dir);	

			DirTree = new TreeView(tree);	

			DirTree.HeadersVisible = false;			
		
			DirTree.AppendColumn("", new CellRendererPixbuf (), "pixbuf", 0);
			DirTree.AppendColumn(catalogo.GetString("Directorio"), new CellRendererText(), "text", 1);
			

			DirTree.Selection.Changed += new EventHandler(DirTreeButtonPressed);			

			workingDirectoryWindow.Add(DirTree);
			workingDirectoryWindow.ShowAll();			
			TreeView FileTree = GetFileList(dir);
			FileTree.HeadersVisible = true;
			FileTree.RulesHint = true;			

			FileTree.AppendColumn(catalogo.GetString("Icon"), new CellRendererPixbuf (), "pixbuf", 0);
			FileTree.AppendColumn(catalogo.GetString("Path"), new CellRendererText (), "text", 1);
			FileTree.AppendColumn(catalogo.GetString("Name"), new CellRendererText (), "text", 2);
			if (config.isSetviewUrl() )
				FileTree.AppendColumn(catalogo.GetString("URL"), new CellRendererText (), "text", 3);
			if (config.isSetviewRevision() )
				FileTree.AppendColumn(catalogo.GetString("Revision"), new CellRendererText (), "text", 4);
			if (config.isSetviewStatus() )
				FileTree.AppendColumn(catalogo.GetString("Status"), new CellRendererText (), "text", 5);
			if (config.isSetviewLCAuthor() )
				FileTree.AppendColumn(catalogo.GetString("Last Changed Author"), new CellRendererText(), "text", 6);
			if (config.isSetviewLCRevision() )
				FileTree.AppendColumn(catalogo.GetString("Last Changed Revision"), new CellRendererText(), "text", 7);
			if (config.isSetviewLCDate() )
				FileTree.AppendColumn(catalogo.GetString("Last Changed Date"), new CellRendererText(), "text", 8);
			if (config.isSetviewTextUpdated() )
				FileTree.AppendColumn(catalogo.GetString("Text Last Updated"), new CellRendererText(), "text", 9);
			if (config.isSetviewPropLastChanged() )
				FileTree.AppendColumn(catalogo.GetString("Prop last changed"), new CellRendererText(), "text", 10);
			if (config.isSetviewChecksum() )
				FileTree.AppendColumn(catalogo.GetString("Checksum"), new CellRendererText(), "text", 11);	
			FileTree.AppendColumn(catalogo.GetString("Schedule"), new CellRendererText(), "text", 12);		
			
			FileTree.ButtonPressEvent += new ButtonPressEventHandler(FileEntryButtonPressed);
			FileTree.PopupMenu += new PopupMenuHandler (FileEntryPopupMenu_);
					
			//browseFilesWindow.Add(FileTree);
			browseFilesWindow.AddWithViewport(FileTree);
			browseFilesWindow.ShowAll();
						
		}	

		
		/// <summary> Events from View Menu (on Main Window) </summary>		
		
		/// <summary>	Method asociated with glade graphical interface.
		///				This method is invoked when user clicks on app menu -> view -> Refresh
		/// </summary>			
		public void OnMenuViewRefreshActivate(object o, EventArgs args)
		
		{
			TreeIter iter;
            TreeModel model;
            string selected = "";

			if (DirTree != null) {
				if (DirTree.Selection.GetSelected (out model, out iter)) 
            		selected = (string) model.GetValue (iter, 2);

            	if (selected != "") {
	            	this.clearBrowseDirWindow();
	            	this.clearBrowseFilesWindow();

	            	this.createTreeCopy(this.wc_dir);

	            	//browseFilesWindow.Add(this.GetFileList(selected));
	            	browseFilesWindow.AddWithViewport(this.GetFileList(selected));
	            	browseFilesWindow.ShowAll();
	            }
            }            
			
		}
		
		/// <summary>	Method asociated with glade graphical interface.
		///				This method is invoked when user clicks on app menu -> view -> clear
		/// </summary>			
		public void OnMenuViewClearActivate(object o, EventArgs args)
		
		{
			log.clearLog();
		}
		
		/// <summary>	Method asociated with glade graphical interface.
		///				This method is invoked when user clicks on app menu -> view -> columns -> path
		/// </summary>	
		public void OnMenuColumnsSetPath(object o, EventArgs args) {
			
			config.permutViewPath();
			refreshFileTree();
		
		}
		
		/// <summary>	Method asociated with glade graphical interface.
		///				This method is invoked when user clicks on app menu -> view -> columns -> name
		/// </summary>	
		public void OnMenuColumnsSetName(object o, EventArgs args){
			
			config.permutViewName();
		
		}
		
		/// <summary>	Method asociated with glade graphical interface.
		///				This method is invoked when user clicks on app menu -> view -> columns -> url
		/// </summary>			
		public void OnMenuColumnsSetUrl(object o, EventArgs args) {
			
			config.permutViewUrl();
		
		}

		/// <summary>	Method asociated with glade graphical interface.
		///				This method is invoked when user clicks on app menu -> view -> columns -> path -> revision
		/// </summary>	
		public void OnMenuColumnsSetRevision(object o, EventArgs args) {
			
			config.permutViewRevision();
		
		}
		
		/// <summary>	Method asociated with glade graphical interface.
		///				This method is invoked when user clicks on app menu -> view -> columns -> path
		///				status
		/// </summary>		
		public void OnMenuColumnsSetStatus(object o, EventArgs args) {
			
			config.permutViewStatus();
		
		}
		
		/// <summary>	Method asociated with glade graphical interface.
		///				This method is invoked when user clicks on app menu -> view -> columns -> path
		///				last changed author
		/// </summary>			
		public void OnMenuColumnsSetLastChangedAuthor(object o, EventArgs args) {
			
			config.permutViewLCAuthor();
		
		}
		
		/// <summary>	Method asociated with glade graphical interface.
		///				This method is invoked when user clicks on app menu -> view -> columns -> path
		/// </summary>			
		public void OnMenuColumnsSetLastChangedRevision(object o, EventArgs args) {
			
			config.permutViewLCRevision();
		
		}

		/// <summary>	Method asociated with glade graphical interface.
		///				This method is invoked when user clicks on app menu -> view -> columns -> 
		///				last changed date.
		/// </summary>			
		public void OnMenuColumnsSetLastChangedDate(object o, EventArgs args) {
			
			config.permutViewLCDate();
		
		}

		/// <summary>	Method asociated with glade graphical interface.
		///				This method is invoked when user clicks on app menu -> view -> columns -> text last
		/// 			updated.
		/// </summary>			
		public void OnMenuColumnsSetTextLastUpdated (object o, EventArgs args) {
			
			config.permutViewTextUpdated();
		
		}

		/// <summary>	Method asociated with glade graphical interface.
		///				This method is invoked when user clicks on app menu -> view -> columns -> p.last changed
		/// </summary>	
		public void OnMenuColumnsSetPropLastChanged (object o, EventArgs args) {
			
			config.isSetviewPropLastChanged();
		
		}
		
		/// <summary>	Method asociated with glade graphical interface.
		///				This method is invoked when user clicks on app menu -> view -> columns -> checksum
		/// </summary>			
		public void OnMenuColumnsSetChecksum(object o, EventArgs args) {
			
			config.permutViewChecksum();
		
		}
		
		public void OnMenuViewToolbarActivate(object o, EventArgs args) {
		
			if (config.isSetviewToolbar())
				mainToolbar.HideAll();
			else
				mainToolbar.ShowAll();
				
			config.permutViewToolbar();
			
		}
		
		/// <summary> Events from Repository Menu (on Main Window) </summary>
		
		/// <summary>	Method asociated with glade graphical interface.
		///				This method is invoked when user clicks on app menu -> repository -> checkout
		/// </summary>		
		public void OnMenuRepositoryCheckoutActivate(object o, EventArgs args)
		
		{
			log.Debug("User click on repository -> checkout");

			log.Debug(catalogo.GetString("User click Repository -> Checkout"));
			log.Debug(catalogo.GetString("Creating CheckOut Dialog"));

			checkoutdialog = new checkOutDialog(this);
			checkoutdialog.Show();		

		}
		

		/// <summary>	Method asociated with glade graphical interface.
		///				This method is invoked when user clicks on app menu -> repository -> create
		/// </summary>				
		public void OnMenuRepositoryCreateActivate(object o, EventArgs args)
		
		{
			createRepositoryDialog = new CreateRepositoryDialog(this);
			createRepositoryDialog.Show();
		}

		/// <summary>	Method asociated with glade graphical interface.
		///				This method is invoked when user clicks on app menu -> repository -> import
		/// </summary>				
		public void OnMenuRepositoryImportActivate(object o, EventArgs args)
		
		{

			log.Debug(catalogo.GetString("User click Repository -> Import"));
			log.Debug(catalogo.GetString("Creating Import Dialog"));			
			importdialog = new ImportDialog(this);
			importdialog.Show();			
		
		}

		/// <summary>	Method asociated with glade graphical interface.
		///				This method is invoked when user clicks on app menu -> repository -> export
		/// </summary>			
		public void OnMenuRepositoryExportActivate(object o, EventArgs args)
		
		{
			exportdialog = new ExportDialog(this);
			exportdialog.Show();
		}
		
		/// <summary>	Method asociated with glade graphical interface.
		///				This method is invoked when user clicks on app menu -> Modify -> Update
		/// </summary>			
		public void OnMenuModifyUpdateActivate(object o, EventArgs args) {
		
			updaterevisiondialog = new UpdateRevisionDialog(this, 0);
			updaterevisiondialog.Show();			
			
		}
		
		/// <summary>	Method asociated with glade graphical interface.
		///				This method is invoked when user clicks on app menu -> Modify -> Update Revision
		/// </summary>			
		public void OnMenuModifyUpdateRevisionActivate(object o, EventArgs args) {
		
			updaterevisiondialog = new UpdateRevisionDialog(this, 1);
			updaterevisiondialog.Show();
			
		}
		
		/// <summary>	Method asociated with glade graphical interface.
		///				This method is invoked when user clicks on app menu -> Modify -> 
		/// </summary>			
		public void OnMenuModifyCommitActivate(object o, EventArgs args) {
		
			commitdialog = new CommitDialog(this);
			commitdialog.Show();
		}
		
		public void OnMenuModifyMakeDirectoryActivate(object o, EventArgs args) {
		
			makeDirectoryDialog = new MakeDirectoryDialog(this);
			makeDirectoryDialog.Show();
			
		}
		
		/// <summary>	Method asociated with glade graphical interface.
		///				This method is invoked when user clicks on app menu -> Modify -> Copy Remote to Remote
		/// </summary>			
		public void OnMenuModifyCopyRemotetoRemoteActivate(object o, EventArgs args)  {
		
			copyRemotetoRemoteDialog = new CopyRemotetoRemoteDialog(this );
			copyRemotetoRemoteDialog.Show();
			
			
		}
		
		
		/// <summary>	Method asociated with glade graphical interface.
		///				This method is invoked when user clicks on app menu -> Modify -> Copy Remote to Local
		/// </summary>			
		public void OnMenuModifyCopyRemotetoLocalActivate(object o, EventArgs args) {
		
			copyRemotetoLocalDialog = new CopyRemotetoLocalDialog(this, this.getDirTreeSelected() );
			copyRemotetoLocalDialog.Show();
			
		}
		
		/// <summary>	Method asociated with glade graphical interface.
		///				This method is invoked when user clicks on app menu -> Modify -> Cleanup
		/// </summary>			
		public void OnMenuModifyCleanupActivate(object o, EventArgs args) {
		
			cleanupConfirmDialog = new CleanupConfirmDialog(this);
			cleanupConfirmDialog.Show();
			
		}
		
		public void OnMenuModifyMoveOnRepositoryActivate(object o, EventArgs args) {
		
			moveRemoteDialog = new MoveRemoteDialog(this);
			moveRemoteDialog.Show();
		
		}
		
		/// <summary>	Method asociated with glade graphical interface.
		///				This method is invoked when user clicks on app menu -> Modify -> Switch
		/// </summary>			
		public void OnMenuModifySwitchActivate(object o, EventArgs args) {
		
			switchDialog = new SwitchDialog(this);
			switchDialog.Show();
			
		}
		

		/// <summary>	Method asociated with glade graphical interface.
		///				This method is invoked when user clicks on App's Menu -> query -> Log
		/// </summary>			
		public void OnMenuQueryLogActivate(object o, EventArgs args) {
		
			string[] _params = new string[10];
			int i = 0;
			string _selected = "";
			
			_params[i++] = this.workingcopy.workingdirectory;
			
			if (svn == null) {
				svn = new SvnClient();
			}	
						
			svn.execSubcommand("LOG", _params);		
		
		}
		
		/// <summary>	Method asociated with glade graphical interface.
		///				This method is invoked when user clicks on App's Menu -> Query -> Info
		/// </summary>			
		public void OnMenuQueryInfoActivate(object o, EventArgs args) {
		
			infoDialog = new InfoDialog(this.workingcopy, this.workingcopy.workingdirectory);
			infoDialog.Show();
		
		}
		 
	
		/// <summary>	Method asociated with glade graphical interface.
		///				This method is invoked when user clicks on app menu -> extras -> Shell
		/// </summary>			
		public void OnMenuExtrasShellActivate(object o, EventArgs args)	{

			log.Debug(catalogo.GetString("User click Extras -> Shell"));
			log.Debug(catalogo.GetString("Opening a shell on a tab..."));

			prepareTerminal();
			inf_notebook.CurrentPage = (inf_notebook.NPages-1);
		}
		
		/// <summary>	Method asociated with glade graphical interface.
		///				This method is invoked when user clicks on app menu -> extras -> preferences
		/// </summary>	
		public void OnMenuExtrasPreferencesActivate(object o, EventArgs args) {
		
			preferencesDialog = new PreferencesDialog(svnconfig);
			preferencesDialog.Show();
		
		}
		
		/// <summary> Events from help Menu (on Main Window) </summary>
		
		/// <summary>	Method asociated with glade graphical interface.
		///				This method is invoked when user clicks on apps's menu -> help -> about.	
		/// </summary>	
		public void OnMenuHelpAboutActivate(object o, EventArgs args) {
		

			log.Debug(catalogo.GetString("User click Help -> About"));
			log.Debug(catalogo.GetString("Creating about Window"));
			
			aboutwindow = new AboutWindow();
			aboutwindow.Show();

		}
		
				
		/// <summary> Events from help Menu (on Main Window) </summary>
		
		/// <summary>	Method asociated with glade graphical interface.
		///				This method is invoked when user clicks on apps's menu -> help -> Quick Launch.
		/// </summary>	
	    public void OnMenuHelpQuickLaunchActivate(object o, EventArgs args) {
	    
			;	
	    	    
	    }		
		

		/// <summary>	Method asociated with glade graphical interface.
		///				This method is invoked when user clicks on app menu -> help -> contents
		/// </summary>		
		public void OnMenuHelpContentsActivate(object o, EventArgs args) {
		

			log.Debug(catalogo.GetString("User click Help -> Contents"));
			log.Debug(catalogo.GetString("Creating Help contents Window"));

			helpwindow = new HelpWindow();
			helpwindow.Show();
			
		}
		
		public void OnMenuHelpWebSiteActivate(object o, EventArgs args)
		
		{
			;
		}
		
		////////////////////////////////////////////////////////////////////////////////////
		// AUXILIAR METHODS
		////////////////////////////////////////////////////////////////////////////////////
		
		/// <summary> Returns Working Directory </summary>
		/// <returns> Working Directory </returns>
		public string getWorkingDirectory() {
		
			return this.wc_dir;
			
		}
		
		/// <summary> Set Working Directory </summary>
		/// <param name="dir">The working directory to set.</param>
		public void setWorkingDirectory(string dir) {
		
			wc_dir = dir;
			
		}		
		
		/// <summary> Set the property list that user has introduced throught property editor. </summary>
		/// <param name="list">The property List.</param>
		public void saveProperties(PropList list) {
		
			int _entries = list.lengthProperties();
			TreeIter iter;
            TreeModel model;
            string selected = "";
            
			// search for what entry on WorkingDirectory frame  is selected.
			
			// we load the properties list. 
			// we search what entry on browse file is activated
			if (FileTree != null) 
				if (FileTree.Selection.GetSelected (out model, out iter)) 
            		selected = 	(string) model.GetValue (iter, 1) + "/" + (string) model.GetValue (iter, 2);
			// we search the entry on file tree is selected.
			
			for (int i = 0; i < _entries; i++) {
				if (list.getPropertyName(i) != "")
					workingcopy.propSet(selected,list.getPropertyName(i), list.getPropertyValue(i));
			
			}
			
		}
		
		/// <summary> Edit the property list that user has introduced throught property editor. </summary>
		/// <param name="list">The property List that contains the properties to edit.</param>
		public void editProperties(PropList list) {
		
			int _entries = list.lengthProperties();
			TreeIter iter;
            TreeModel model;
            string selected = "";
            
			// search for what entry on WorkingDirectory frame  is selected.
			
			// we load the properties list. 
			// we search what entry on browse file is activated
			if (FileTree != null) 
				if (FileTree.Selection.GetSelected (out model, out iter)) 
            		selected = 	(string) model.GetValue (iter, 1) + "/" + (string) model.GetValue (iter, 2);
			// we search the entry on file tree is selected.		
			
			for (int i = 0; i < _entries; i++) {
				if (list.getPropertyName(i) != "")
					workingcopy.propEdit(selected,list.getPropertyName(i), list.getPropertyValue(i));
			
			}
			
		}
		
		/// <summary> Delete the property list that user has introduced throught property editor. </summary>
		/// <param name="list">The property List that contains the properties to delete.</param>
		public void deleteProperties(PropList list) {
		
			int _entries = list.lengthProperties();
			TreeIter iter;
            TreeModel model;
            string selected = "";
            
			// search for what entry on WorkingDirectory frame  is selected.
			
			// we load the properties list. 
			// we search what entry on browse file is activated
			if (FileTree != null) 
				if (FileTree.Selection.GetSelected (out model, out iter)) 
            		selected = 	(string) model.GetValue (iter, 1) + "/" + (string) model.GetValue (iter, 2);
			// we search the entry on file tree is selected.		
			
			for (int i = 0; i < _entries; i++) {
				System.Console.WriteLine(selected);
				if (list.getPropertyName(i) != "")
					workingcopy.propDel(selected,list.getPropertyName(i));
			
			}
			
		}		
		
		/// <summary> Refresh the file tree window. </summary>
		void refreshFileTree() {
		
			TreeIter iter;
            TreeModel model;
            string selected = "";
            
			// search for what entry on WorkingDirectory frame  is selected.
			
			// we load the properties list. 
			// we search what entry on browse file is activated
			if (DirTree != null) {
				if (DirTree.Selection.GetSelected (out model, out iter)) 
            		selected = (string) model.GetValue (iter, 2);
            	// clear both frames
            	if (selected != "") {
	            	this.clearBrowseFilesWindow();
	            	// we populate the dir tree for the working directory tree	            	
	            	// we populate the file tree for the selected directory
	            	//browseFilesWindow.Add(this.GetFileList(selected));
	            	browseFilesWindow.AddWithViewport(this.GetFileList(selected));
	            	browseFilesWindow.ShowAll();
	            }
	         }
	         
	     }
	     
	     /// <summary> This method is called when user do Revert throught Contextual Menu.
	     /// </summary>
	     public void doRevert() {
	     
	     	// search what entry on filetree is selected.
			TreeIter iter;
            TreeModel model;
            string _entry = "";
            
			// search for what entry on WorkingDirectory frame  is selected.
			
			// we load the properties list. 
			// we search what entry on browse file is activated
			if (FileTree != null) {
				if (FileTree.Selection.GetSelected (out model, out iter)) 
            		_entry = (string) model.GetValue (iter, 1) + "/" + (string) model.GetValue (iter, 2);
            	
            	workingcopy.revert(_entry);
            	revertCompletedDialog = new RevertCompletedDialog();
            	revertCompletedDialog.Show();		
            		 
            } 
            
         }
         	
         /// <summary> Refresh the file tree window. </summary>
         public void refreshFileList() {
         
         	// search what entry on filetree is selected.
			TreeIter iter;
            TreeModel model;
            string _entry = "";
            
			// search for what entry on WorkingDirectory frame  is selected.
			
			// we load the properties list. 
			// we search what entry on browse file is activated
			if (DirTree != null) {
				if (DirTree.Selection.GetSelected (out model, out iter)) 
            		_entry = (string) model.GetValue (iter, 1) + "/" + (string) model.GetValue (iter, 2);
            	else 
            		_entry = workingcopy.workingdirectory;
           	}
           	else
           		_entry = workingcopy.workingdirectory;
           		    	
           	this.clearBrowseFilesWindow();
           	//browseFilesWindow.Add(this.GetFileList(_entry));
           	browseFilesWindow.AddWithViewport(this.GetFileList(_entry));
	       	browseFilesWindow.ShowAll();
	       
	    }
	    
	    /// <summary> Returns the Entry on File Tree wich is selected. </summary>
	    /// <returns> Selected entry on File tree. </returns>
	    public string getFileTreeSelected() {
	    
	    	string _selected = "";
			TreeIter iter;
            TreeModel model;
            	    	
	    	if (FileTree != null) 
				if (FileTree.Selection.GetSelected (out model, out iter)) 
            		_selected = (string) model.GetValue (iter, 1) + "/" + (string) model.GetValue (iter, 2);
         	
         	return _selected;
         	
         }
                                 
		/// <summary> Returns the Entry on Dir Tree wich is selected. </summary> 
		/// <returns> Selected entry on Dir tree. </returns>                                              
		public string getDirTreeSelected() {
		
			string _selected = "";
			TreeIter iter;
            TreeModel model;
            	    	
	    	if (DirTree != null) 
				if (DirTree.Selection.GetSelected (out model, out iter)) 
            		_selected = (string) model.GetValue (iter, 2);
         	
         	return _selected;
	
		}
	
	
		/////////////////////////////////////////////////////////////////////////////////////////////////
		// POST-EVENTS METHODS. THIS METHODS ARE USED WHEN A DIALOG IS CALLED, FOR FURTHER OPERATIONS. //
		/////////////////////////////////////////////////////////////////////////////////////////////////
		
		/// <summary> This method is called when user call to checkout dialog. After, checkout is 
		/// realized throught this method. </summary>
		/// <param name="SVN_URL"> The Subversion resource </param>
		/// <param name="SVN_REMOTE_DIR"> The directory on Subversion repository </param>
		/// <param name="LOCAL_PATH"> Local path to download checkout </param>
		/// <param name="USERNAME"> Username on repository if neccesary </param>
		/// <param name="LAST_REVISION"> True if checkout must download last revision </param>
		/// <param name="REVISION"> Revision to download if Checkout musnt download lastest revision. </param>
		public void postCheckoutDialog(	string 	SVN_URL,
										string  SVN_REMOTE_DIR,
										string 	LOCAL_PATH,
										string  USERNAME,
										bool	LAST_REVISION, 	
										string 	REVISION) {

			string[] _params;
			//string output;
			
			_params = new string[10];
			
			svn = new SvnClient(LOCAL_PATH, SVN_URL, SVN_REMOTE_DIR, this);

			_params[0] = LAST_REVISION.ToString();
			_params[1] = REVISION;	
			_params[2] = USERNAME;		
	
			svn.execSubcommand("CHECKOUT", _params);
			
			this.setWorkingDirectory(LOCAL_PATH);				
			
			workingcopy = new WorkingCopy(LOCAL_PATH);

		}
		
		/// <summary> This method is called when user call to export dialog. After, export is 
		/// realized throught this method with the parameters user's check in. </summary>
		/// <paran name="FROM_URL">True if Export must be realized from an URL.</param>
		/// <param name="SVN_URL"> The Subversion resource </param>
		/// <param name="LOCAL_DIR"> Directory on repository to export </param>
		/// <param name="REV"> Revision to export, if lastest doesnt export </param>
		/// <param name="SAVE_TO"> Local dir to save the export </param>
		/// <param name="COMPRESS_SAVE_TO"> Local dir to save the compress export </param>
		/// <param name="COMPRESS_FORMAT"> Format to save the export </param>		
		public void postExportDialog(	bool FROM_URL,
										string SVN_URL,
										string LOCAL_DIR,
										string REV,
										string SAVE_TO,
										string COMPRESS_SAVE_TO,
										string COMPRESS_FORMAT) {
		
			string[] _params = new string[10];
			
			int i = 0;
			
			_params[i++] = FROM_URL.ToString();
			_params[i++] = SVN_URL;
			_params[i++] = LOCAL_DIR;
			_params[i++] = REV;
			_params[i++] = SAVE_TO;
			_params[i++] = COMPRESS_SAVE_TO;
			_params[i++] = COMPRESS_FORMAT;
			
			svn = new SvnClient();
				
			svn.execSubcommand("EXPORT", _params);
				
		}
		
		/// <summary> This method is called when user call to import dialog. After, import is 
		/// realized throught this method with the parameters user's check in. </summary>
		/// <paran name="LOCAL_PATH">Path to import on server.</param>
		/// <param name="SVN_URL"> Subversion URL Resource.</param>
		/// <param name="REPOSITORY_DIR"> Repository Dir on Server. </param>
		/// <param name="IMPORT_MESSAGE"> Import message. </param>
		/// <param name="USERNAME"> Username to do import, if neccesary </param>
		/// <param name="NON_RECURSIVE"> True if import must be non-recursive over LOCAL_PATH. </param>				
		public void postImportDialog(	string LOCAL_PATH,
										string SVN_URL,
										string REPOSITORY_DIR,
										string IMPORT_MESSAGE,
										string USERNAME,
										bool NON_RECURSIVE) {
										
			string[] _params;
			//string output;
			
			_params = new string[10];
		
			_params[0] = SVN_URL;
			_params[1] = LOCAL_PATH;	
			_params[2] = REPOSITORY_DIR;
			_params[3] = IMPORT_MESSAGE;
			_params[4] = USERNAME;
			_params[5] = NON_RECURSIVE.ToString();
			
			svn = new SvnClient();
			
			svn.execSubcommand("IMPORT", _params);		
				
		}	
		
		/// <summary> This method is called when user call to Commit dialog. After, commit is 
		/// realized throught this method with the parameters user's check in. </summary>
		/// <paran name="MESSAGE">Commit log.</param>
		public void postCommitDialog(	string MESSAGE ) {
		
			string[] _params = new string[10];
			int i = 0;
			
			_params[i++] = workingcopy.workingdirectory;
			_params[i++] = MESSAGE;		
			
			svn = new SvnClient();	
			
			svn.execSubcommand("COMMIT", _params);	
			
		}
		
		/// <summary> This method is called when user call to Commit dialog over an individual entry. After, commit is 
		/// realized throught this method with the parameters user's check in. </summary>		
		/// <paran name="MESSAGE">Commit log.</param>		
		public void postCommitEntryDialog(	string MESSAGE ) {
		
			string[] _params = new string[10];
			int i = 0;
			string _selected = "";
			
			_selected = this.getFileTreeSelected();
			
			if (_selected == "") 
				_selected = workingcopy.workingdirectory;			
			
			_params[i++] = _selected;
			_params[i++] = MESSAGE;		
			
			svn = new SvnClient();	
			
			svn.execSubcommand("COMMIT_ENTRY", _params);	
			
		}		
		
		/// <summary> This method is called when user call to Update Dialog. After, update is 
		/// realized throught this method with the parameters user's check in. </summary>	
		/// <param name="lastest"> True if Update must set working copy to lastest repository revision.</param>
		/// <param name="revision"> In other case, revision to update the working copy.</param>		
		public void postUpdateDialog( bool lastest, string revision) {
		
			string[] _params = new string[10];
			int i = 0;
			
			_params[i++] = workingcopy.workingdirectory;
			_params[i++] = lastest.ToString();
			_params[i++] = revision;
			
			if (svn == null) {
				svn = new SvnClient();
			}	
			
			svn.execSubcommand("UPDATE", _params);	
			
		}
		
		/// <summary> This method is called when user call to Make Directory Dialog. After, operation is 
		/// realized throught this method with the parameters user's check in. </summary>	
		/// <param name="dirName"> Name of directory to create.</param>
		public void postMakeDirectoryDialog(string dirName) {		
		
			string _selected = this.getDirTreeSelected();
			
			if (_selected == "") _selected = workingcopy.workingdirectory + "/" ;
		
			workingcopy.makeDirectory( _selected, dirName);
			this.clearBrowseDirWindow();
	        this.createTreeCopy(this.wc_dir);
	        // we populate the file tree for the selected directory
	        this.clearBrowseFilesWindow();
	        //browseFilesWindow.Add(this.GetFileList(_selected));
	        browseFilesWindow.AddWithViewport(this.GetFileList(_selected));
	        browseFilesWindow.ShowAll();
		
		}
		
		/// <summary> This method is called when user call to Clenaup Dialog. After, operation is 
		/// realized throught this method with the parameters user's check in. </summary>	
		public void postCleanupDialog() {
		
			string _selected = this.getDirTreeSelected();
		
			if (_selected == "") _selected = workingcopy.workingdirectory;
			
			workingcopy.cleanup(_selected);
			
		}
		
		/// <summary> This method is called when user call to Make Switch Dialog. After, operation is 
		/// realized throught this method with the parameters user's check in. </summary>	
		/// <param name="NEW_URL"> New Subversion URL to switch the working copy. </param>		
		public void postSwitchDialog(string NEW_URL)  {
		
			string[] _params = new string[10];
			int i = 0;
			
			_params[i++] = NEW_URL;
			_params[i++] = this.workingcopy.workingdirectory;
			
			if (svn == null) {
				svn = new SvnClient();
			}	
			
			svn.execSubcommand("SWITCH", _params);	
		
		}		
		
		/// <summary> This method is called when user call to Move on Repository Dialog. After, operation is 
		/// realized throught this method with the parameters user's check in. </summary>	
		/// <param name="FROM"> The source file. </param>				
		/// <param name="TO"> The dest file. </param>
		/// <param name="MESSAGE"> Operation Log Message. </param>
		public void postMoveOnRepository(string FROM, string TO, string MESSAGE) {
		
			string[] _params = new string[10];
			int i = 0;
			
			_params[i++] = FROM;
			_params[i++] = TO;
			_params[i++] = MESSAGE;
			
			if (svn == null) {
				svn = new SvnClient();
			}	
			
			svn.execSubcommand("MOVE_REMOTE", _params);
			
		}			
		
		/// <summary> This method is called when user call to Copy Remote to Remote Dialog. After, operation is 
		/// realized throught this method with the parameters user's check in. </summary>	
		/// <param name="FROM"> The source file. </param>				
		/// <param name="TO"> The dest file. </param>
		/// <param name="MESSAGE"> Operation Log Message. </param>		
		public void postCopyRemotetoRemoteDialog(string FROM, string TO, string MESSAGE) {
		
			string[] _params = new string[10];
			int i = 0;
			
			_params[i++] = FROM;
			_params[i++] = TO;
			_params[i++] = MESSAGE;
			
			if (svn == null) {
				svn = new SvnClient();
			}	
			
			svn.execSubcommand("COPY_REMOTE", _params);
			
		}
		
		/// <summary> This method is called when user call to Rename Dialog. After, operation is 
		/// realized throught this method with the parameters user's check in. </summary>	
		/// <param name="TO"> The file or directory new name. </param>				
		/// <param name="MESSAGE"> Operation Log Message. </param>		
		public void postRenameDialog(string TO, string MESSAGE) {
		
			string[] _params = new string[10];
			int i = 0;
			string _selected = "";
			
			_selected = this.getFileTreeSelected();
			
			_params[i++] = _selected;
			_params[i++] = TO;
			_params[i++] = MESSAGE;
			
			if (svn == null) {
				svn = new SvnClient();
			}	
						
			svn.execSubcommand("MOVE_REMOTE", _params);
		
		}
		
		/// <summary> This method is called when user call to Cat Entry Option. After, operation is 
		/// realized throught this method with the parameters user's check in. </summary>	
		/// <param name="_selected"> The _entry wich is selected on file tree.</param>		
		public void postCatDialog(string _selected) {
		
			string[] _params = new string[10];
			int i = 0;
			
			_params[i++] = _selected;
			
			if (svn == null) {
				svn = new SvnClient();
			}	
						
			svn.execSubcommand("CAT", _params);			
			
		}			
		
		/// <summary> This method is called when user call to Blame Entry Option. After, operation is 
		/// realized throught this method with the parameters user's check in. </summary>	
		public void postBlameDialog() {
		
			string[] _params = new string[10];
			int i = 0;
			string _selected = "";
			
			_selected = this.getFileTreeSelected();
			
			_params[i++] = _selected;
			
			if (svn == null) {
				svn = new SvnClient();
			}	
						
			svn.execSubcommand("BLAME", _params);			
			
		}				
		
		/// <summary> This method is called when user call to 'Log' Entry Option. After, operation is 
		/// realized throught this method with the parameters user's check in. </summary>	
		public void postLogDialog() {
		
			string[] _params = new string[10];
			int i = 0;
			string _selected = "";
			
			_selected = this.getFileTreeSelected();
			
			_params[i++] = _selected;
			
			if (svn == null) {
				svn = new SvnClient();
			}	
						
			svn.execSubcommand("LOG", _params);
			
		}	
		
		/// <summary> This method is called when user call to Create Dialog. After, operation is 
		/// realized throught this method with the parameters user's check in. </summary>	
		/// <param name="PATH"> The PATH to the new Repository. </param>			
		public void postCreateDialog(string PATH) {
		
			string[] _params = new string[10];
			int i = 0;

			_params[i++] = PATH;
			
			if (svn == null) {
				svn = new SvnClient();
			}	
						
			svn.execSubcommand("CREATE", _params);
			
		}
	
	} // Closing main class

} // Closing pfc namespace

