#region

//        Copyright (C) 2004, 2005. Antonio Gutiérrez Mayoral
//                 <agutierr@gsyc.escet.urjc.es>
// 
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License as
//  published by the Free Software Foundation; either version 2 of the
//  License, or (at your option) any later version.
// 
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  General Public License for more details.
// 
//  You should have received a copy of the GNU General Public
//  License along with this program; if not, write to the
//  Free Software Foundation, Inc., 59 Temple Place - Suite 330,
//  Boston, MA 02111-1307, USA.

#endregion

using System;
using System.IO;
using System.Xml;

using Gtk;
using Glade;

namespace pfc {

	/// <summary>
	/// This class defines a dialog to the user, about Welcome Info to Gsvn.
	/// <summary>
	public class WelcomeDialog {
	
		[Widget] Dialog welcomeDialog;
		
		public WelcomeDialog() {
		
			Glade.XML gxml = Glade.XML.FromAssembly ("gui.glade","welcomeDialog",null);
       		gxml.Autoconnect (this);     		
       		
       	}
       	
       	public void Show() {
       	
       		welcomeDialog.Show();
       		
       	}
       	
       	public void On_welcomeDialog_Close_Clicked(object o , EventArgs args) {
       		welcomeDialog.Hide();
       		welcomeDialog.Dispose();
       		
       	}
       	
	} // Closing WelcomeDialog class

	/// <summary> This class is used for configurate Gsvn throught xml config files or
	/// set preferences in view menu. 
	/// </summary>
	public class Config {
	
		// program options.
		
		private string svnExecutable = "/usr/bin/svn";
	
		// General options.
	
		//private bool savePreferencesOnExit;
		
		//private bool saveViewColumnsOnExit;
		
		// From View Menu
		
		private bool viewPath;
		
		private bool viewName;
		
		private bool viewUrl;
		
		private bool viewRevision;
		
		private bool viewStatus;
		
		private bool viewLCAuthor;
		
		private bool viewLCRevision;
		
		private bool viewLCDate;
		
		private bool viewTextUpdated;
		
		private bool viewPropLastChanged;
		
		private bool viewChecksum;
		
		private bool viewSaveOnExit;
		
		private bool viewToolBar;
	
		private WelcomeDialog welcomedialog;
		
		// from view menu
		
		// true = big
		// false = small.
		private bool iconSize;
	
		/// <summary> class constructor </summary>
		public Config() {
		
			if (!checkConfigFiles()) {
			
				// show welcome dialog to user
				welcomedialog = new WelcomeDialog();
				welcomedialog.Show();
				
				// set defaults view propertys
				this.viewPath = true;
				this.viewName = true;
				this.viewUrl = true;
				this.viewRevision = true;
				this.viewStatus = true;
				this.viewLCAuthor = true;
				this.viewLCRevision = true;
				this.viewTextUpdated  = true;
				this.viewPropLastChanged = true;
				this.viewChecksum = true;
				this.viewSaveOnExit = true;
				this.viewToolBar = true;
				
			}		
				
		}
		
		/// <summary> This function check that files options.xml is in user's home directory.
		/// </summary>
		/// <return> True if files options.xml is in user's home directory. </returns>
		public bool checkConfigFiles() {
			
			string homePath = "";
			
			try {
			  	homePath = Environment.GetEnvironmentVariable("HOME");
			}
			catch (Exception ex) {
				Console.WriteLine("Exception: " + ex.Message );
			}
			
			
			try {
				XmlTextReader reader = new XmlTextReader( homePath + "/.gsvn/options.xml");
				return true;			
			}
			catch (System.IO.FileNotFoundException) {
				return false;
			}
			catch (System.IO.DirectoryNotFoundException) {
				System.IO.Directory.CreateDirectory( homePath + "/.gsvn");
				return false;
			}
			
		}
		
		// consult member variables.
		public bool isSetviewPath() {
		
			return viewPath;
			
		}
		
		/// <summary> Permut value of ViewPath. </summary>
		public void permutViewPath() {
		
			viewPath = ! viewPath;
			
		}
		
		/// <summary> Return true if ViewName is checked. </summary>
		public bool isSetviewName() {
		
			return viewName;
			
		}
		
		/// <summary> Permut value of ViewName. </summary>
		public void permutViewName() {
		
			viewName = ! viewName;
			
		}
		
		/// <summary> Return true if ViewUrl is checked. </summary>
		public bool isSetviewUrl() {
		
			return viewUrl;
			
		}
		
		/// <summary> Permut value of ViewURL. </summary>
		public void permutViewUrl() {
		
			viewUrl = ! viewUrl;
			
		}
		
		/// <summary> Return true if ViewRevision is checked. </summary>
		public bool isSetviewRevision() {
		
			return viewRevision;
			
		}
		
		/// <summary> Permut value of ViewRevision. </summary>
		public void permutViewRevision() {
		
			viewRevision = ! viewRevision;
			
		}
		
		/// <summary> Return true if ViewStatus is checked. </summary>
		public bool isSetviewStatus() {
		
			return viewStatus;
			
		}
		
		/// <summary> Permut value of view status. </summary>
		public void permutViewStatus() {
		
			viewStatus = ! viewStatus;
			
		}
		
		/// <summary> Return true if ViewLCAuthor is checked. </summary>
		public bool isSetviewLCAuthor() {
		
			return viewLCAuthor;
			
		}
		
		/// <summary> Permut value of ViewLCAuthor. </summary>
		public void permutViewLCAuthor() {
		
			viewLCAuthor = ! viewLCAuthor;
			
		}
		
		/// <summary> Return true if ViewLCRevision is checked. </summary>
		public bool isSetviewLCRevision() {
		
			return viewPath;
			
		}
		
		/// <summary> Return true if ViewLCRevision is checked. </summary>
		public void permutViewLCRevision() {
		
			viewLCRevision = !viewLCRevision;
			
		}
		
		/// <summary> Return true if ViewLCDate is checked. </summary>
		public bool isSetviewLCDate() {
		
			return viewLCDate;
			
		}
		
		public void permutViewLCDate() {
		
			viewLCDate = ! viewLCDate;
			
		}
		
		/// <summary> Return true if ViewTextUpdated is checked. </summary>
		public bool isSetviewTextUpdated() {
		
			return viewTextUpdated;
			
		}
		
		public void permutViewTextUpdated() {
		
			viewTextUpdated = ! viewTextUpdated;
			
		}
		
		/// <summary> Return true if ViewPropLastChanged is checked. </summary>
		public bool isSetviewPropLastChanged() {
		
			return viewPropLastChanged;
			
		}
		
		public void permutViewPropLastChanged() {
		
			viewPropLastChanged = ! viewPropLastChanged;
			
		}
		
		/// <summary> Return true if ViewChecksum is checked. </summary>
		public bool isSetviewChecksum() {
		
			return viewChecksum;
			
		}
		
		
		/// <summary> Return true if ViewSaveOnExitis checked. </summary>
		public void permutViewChecksum() {
		
			viewChecksum = ! viewChecksum;
			
		}
		
		/// <summary> Return true if ViewSaveOnExitis checked. </summary>
		public bool isSetviewSaveOnExit() {
		
			return viewSaveOnExit;
			
		}
		
		/// <summary> Return true if ViewSaveOnExitis checked. </summary>
		public bool isSetviewToolbar() {
		
			return viewToolBar;
			
		}
		
		/// <summary> Changes the value of viewToolBar member. </summary>
		public void permutViewToolbar() {
		
			viewToolBar = ! viewToolBar;
	
		}
		
		/// <summary> Return the icon size. </summary>
		/// <returns> Icon size on config. </returns>
		public bool getIconSize() {
		
			return this.iconSize;
			
		}
		
		/// <summary> Return the path to SvnExec file. </summary>
		/// <returns> Path to svn exec file. </returns>
		public string getSvnExecutable() {
		
			return this.svnExecutable;
			
		}
		
	} // closing config class
	
} // closing namespace
																						
																																																															
		