#region

//        Copyright (C) 2004, 2005. Antonio Gutiérrez Mayoral
//                 <agutierr@gsyc.escet.urjc.es>
// 
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License as
//  published by the Free Software Foundation; either version 2 of the
//  License, or (at your option) any later version.
// 
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  General Public License for more details.
// 
//  You should have received a copy of the GNU General Public
//  License along with this program; if not, write to the
//  Free Software Foundation, Inc., 59 Temple Place - Suite 330,
//  Boston, MA 02111-1307, USA.

#endregion

using System;
using System.Threading;
using Gtk;
using Glade;

using pfc.Svn;

/// <summary> 
/// On this file we define class to show Dialogs to user. 
/// </summary><
namespace pfc {


	/// <summary> This class handleds the Checkout Dialog. 
	/// </summary>
	public class checkOutDialog {
	
		[Widget] Dialog checkoutDialog;
		[Widget] Entry checkoutDialog_serverHost;
		[Widget] Entry checkoutDialog_localdir;
		[Widget] Entry checkoutDialog_localDirOnServer;
		[Widget] Entry checkoutDialog_rev;
		[Widget] Entry checkoutDialog_username;
		
		//[Widget] CheckButton checkoutDialog_lastest;
	
		private fileSelection checkoutfileselection;
		
		private checkOutAlertDialog checkoutalertdialog;
		
		private DirectoryNotExistsAlertDialog directoryNotExistsAlertDialog; 
		
		private gsvn app;
		
		public checkOutDialog(gsvn _app) {
		
			Glade.XML gxml = Glade.XML.FromAssembly ("gui.glade","checkoutDialog",null);
       		gxml.Autoconnect (this);
       		app = _app;
       		
       	}
       	
       	public void Show() {
       	
       		checkoutDialog.Show();
       		
       	}
       	
		/// Methods asociated with events on Dialogs.
		
		public void On_CheckoutDialog_OkButton_Clicked(object o, EventArgs args) {
		
			//SvnClient svn;
		
			/// Check All Entrys are Ok.
			int errors = 0;
			if (checkoutDialog_serverHost.Text == "") {
				errors++;
			}
			if (checkoutDialog_localdir.Text == "") {
				errors++;
			}
			if (checkoutDialog_localDirOnServer.Text == "") {
				errors++;
			}
			if (System.IO.Directory.Exists(checkoutDialog_localdir.Text)==false) {
           		directoryNotExistsAlertDialog = new DirectoryNotExistsAlertDialog();
           		directoryNotExistsAlertDialog.Show();
           		errors++;
           	}
           	if (errors > 0) {
				checkoutalertdialog = new checkOutAlertDialog();
				checkoutalertdialog.Show();
           	}
           	else {
           		// procedeed to checkout.		    
           		checkoutDialog.Hide();          
           		
           		if (this.checkoutDialog_rev.Text == "") {
           			app.postCheckoutDialog(	this.checkoutDialog_serverHost.Text,
           									this.checkoutDialog_localDirOnServer.Text,
           									this.checkoutDialog_localdir.Text,
           									this.checkoutDialog_username.Text,
           									true,
           									"");
           		}
           		else {
            		app.postCheckoutDialog(	this.checkoutDialog_serverHost.Text,
            								this.checkoutDialog_localDirOnServer.Text,
           									this.checkoutDialog_localdir.Text,
           									this.checkoutDialog_username.Text,
           									false,
           									this.checkoutDialog_rev.Text);  
           		}        			
           	}				
			
		}
		
		public void On_CheckoutDialog_CancelButton_Clicked(object o, EventArgs args) {
			
			checkoutDialog.Hide();
			checkoutDialog.Dispose();
			
		}
		
		public void On_CheckoutDialog_Latest_Toggled(object o, EventArgs args) {
		
			if (checkoutDialog_rev.IsEditable == true) {
				checkoutDialog_rev.IsEditable = false;
				checkoutDialog_rev.Text = "" ; 
			}
			else {
				checkoutDialog_rev.IsEditable = true;
			}
			
		}     
		
		// select local dir button activated
		public void OnFileNavigationClicked (object o, EventArgs args) {	

			checkoutfileselection = new fileSelection();
			checkoutfileselection.Show();
			checkoutDialog_localdir.Text = checkoutfileselection.path;
		
		}
		
		public void On_CheckoutDialog_Focus(object o, EventArgs args) {
			
			if (checkoutfileselection != null) {
				checkoutDialog_localdir.Text = checkoutfileselection.path;
			}			
		}
	
	} //closing checkOutDialog Class
	
	/// <summary> 
	/// This class is used for Show a FileSelection Widget to user. 
	/// </summary>
	public class fileSelection {
	
		private FileSelection fileselection;
		
		public string path = "";
		
		public fileSelection () {
		
			fileselection = new FileSelection("Please select local dir");  
			fileselection.Modal = true;	
			fileselection.CancelButton.Clicked += new EventHandler (Close_Button_Clicked);
			fileselection.OkButton.Clicked += new EventHandler (Ok_Button_Clicked);						

		}
		
		public void Show() {
		
			fileselection.Show();
			
		}
		
		public void Ok_Button_Clicked(object o, EventArgs args) { 
		
			fileselection.Hide();
			this.path = fileselection.Filename;
			
		}
		
		public void Close_Button_Clicked(object o, EventArgs args) {
		
			fileselection.Hide();
			
		}
				
	} // closing FileSelection Class.
	
	/// <summary> 
	/// This class is used for show the import dialog to user.
	/// </summary>
	public class ImportDialog {
	
		[Widget] Dialog importDialog;
		[Widget] Entry importDialog_localpath;
		[Widget] Entry importDialog_url;
		[Widget] Gtk.TextView  importDialog_message;
		[Widget] Gtk.CheckButton importDialog_nonrecursive;
		[Widget] Entry importDialog_username;
		[Widget] Entry importDialog_repositoryDir;
		
		private fileSelection fileselection;
		private ImportAlertDialog importalertdialog;
		
		private gsvn app;

		public ImportDialog(gsvn _app) {
		
			Glade.XML gxml = new Glade.XML ("gui.glade","importDialog");
       		gxml.Autoconnect (this);
       		
       		app = _app;
       		
       	}
       	
       	public void Show() {
       	
       		importDialog.Show();
       		
       	}	
       	
       	public void On_importDialog_OkButton_Clicked(object o, EventArgs args) {
       		
       		if ((importDialog_localpath.Text == "") | (importDialog_url.Text == "") | (importDialog_repositoryDir.Text == "")) {
       			importalertdialog = new ImportAlertDialog();
       			importalertdialog.Show();
       		}
       		else {       		
       			app.postImportDialog(importDialog_localpath.Text,
       								 importDialog_url.Text,
       								 importDialog_repositoryDir.Text,
       								 importDialog_message.Buffer.Text,
       								 importDialog_username.Text,
       								 importDialog_nonrecursive.Active);
       		
       			importDialog.Hide();
       			importDialog.Dispose();
       		}       
       		
       	}
       	
       	public void On_importDialog_CancelButton_Clicked(object o, EventArgs args) {
       	
			importDialog.Hide();
			importDialog.Dispose();
       		
       	}
       	
       	public void On_importDialog_Text_Focus(object o, EventArgs args) {
       	
       		importDialog_message.Buffer.Text = "" ; 
       		
       	}
       	
       	public void OnFileNavigationClicked (object o, EventArgs args) {	

			fileselection = new fileSelection();
			fileselection.Show();
		
		}
		
		public void On_importDialog_Focus (object o, EventArgs args) {
		
			if (fileselection != null) {				
				importDialog_localpath.Text = fileselection.path;
			}	
			
		}
       	
    }  // closing ImportDialog Class.	
    
	/// <summary> 
	/// This class is used for show the export dialog to user. 
	/// </summary>    
	public class ExportDialog {
	
		[Widget] Dialog exportDialog;
		// widgets from importdialog dialog
		[Widget] Entry exportDialog_workingcopy;
		[Widget] Entry exportDialog_url;
		//[Widget] Entry exportDialog_user;
		//[Widget] Entry exportDialog_password;		
		[Widget] Entry exportDialog_saveto;
		[Widget] Entry exportDialog_savetofile;
		[Widget] Entry exportDialog_rev;
		
		[Widget] Gtk.RadioButton exportDialog_checkurl;
		[Widget] Gtk.RadioButton exportDialog_checkworking;
		[Widget] Gtk.CheckButton exportDialog_checkcompress;
		//[Widget] Gtk.CheckButton exportDialog_checklastest;
		
		//[Widget] Gtk.Combo exportDialog_combocompress;		
		
		private fileSelection fileselection1;
		private fileSelection fileselection2;
		private fileSelection fileselection3;

		private ExportAlertDialog exportalertdialog;
		
		private gsvn app;

		public ExportDialog(gsvn _app) {
		
			Glade.XML gxml = Glade.XML.FromAssembly ("gui.glade","exportDialog",null);
       		gxml.Autoconnect (this);
       		
       		app = _app;
       		
       	}
       	
       	public void Show() {
       	
       		exportDialog.Show();
       		
       	}	
       	
       	public void OnFileNavigationClicked (object o, EventArgs args) {	
			
			fileselection1 = new fileSelection();
			fileselection1.Show();	
			exportDialog_checkworking.Activate();		
		
		}       	
		
		public void On_exportDialog_OkButton_Clicked(object o, EventArgs args) {
		
			// check all fills are ok
			if ( ((exportDialog_checkurl.Active == true) && (exportDialog_url.Text != "" ) && 
				  (exportDialog_saveto.Text != ""))
				 |
				 ((exportDialog_workingcopy.Text != "") && (exportDialog_saveto.Text != "") &&
				 (exportDialog_checkworking.Active == true)) 
			   ) { // all ok
			   
			   	if (this.exportDialog_checkurl.Active == false) {
			   		
			   		if (WorkingCopy.is_Working_Copy(exportDialog_workingcopy.Text) == false) {
			    		WorkingCopyErrorDialog alertDialog; 					
 						alertDialog = new WorkingCopyErrorDialog();
 						alertDialog.Show();
 					}
 					else {	app.postExportDialog(	this.exportDialog_checkurl.Active,
					   								this.exportDialog_url.Text,
					   								this.exportDialog_workingcopy.Text,
					   								this.exportDialog_rev.Text,
					   								this.exportDialog_saveto.Text,
					   								this.exportDialog_savetofile.Text,
					   								"");
			   								
			   				exportDialog.Hide();	   	
			   		}			

			}
			else {
				exportalertdialog = new ExportAlertDialog();
			}
		
			}
		}
		
		
		public void On_exportDialog_CancelButton_Clicked(object o, EventArgs args) {
		
			exportDialog.Hide();
			exportDialog.Dispose();
			
		}
		
		public void On_exportDialog_Savetofile_Clicked(object o, EventArgs args) {
		
			fileselection2 = new fileSelection();
			fileselection2.Show();	
			exportDialog_checkcompress.Activate();
		
		}
		
       	public void OnFileNavigation2Clicked (object o, EventArgs args) {	

			fileselection3 = new fileSelection();
			fileselection3.Show();		
		
		}   		
		
		public void On_exportDialog_Focus(object o, EventArgs args) {
		
			if (fileselection1 != null) {
				exportDialog_workingcopy.Text = fileselection1.path;
			}
			if (fileselection2 != null) {
				exportDialog_savetofile.Text = fileselection2.path;
			}
			if (fileselection3 != null) {
				exportDialog_saveto.Text = fileselection3.path;
			}
		}
       	
	} // closing ExportDialog class
	
	/// <summary> 
	/// This class shows the update revision dialog to user.
	/// </summary>	
	public class UpdateRevisionDialog {
	
		[Widget] Gtk.Dialog updateRevisionDialog;
		[Widget] Gtk.CheckButton updateRevisionDialog_toLastestCheck;
		[Widget] Gtk.CheckButton updateRevisionDialog_toRevisionCheck;
		[Widget] Gtk.Entry updateRevisionDialog_revision;
		
		private gsvn app;
		
		public UpdateRevisionDialog(gsvn _app, int lastest) {
		
			Glade.XML gxml = Glade.XML.FromAssembly ("gui.glade", "updateRevisionDialog", null);
			gxml.Autoconnect(this);
			app = _app;
			
			switch (lastest) {
			
				case 0: 	updateRevisionDialog_toLastestCheck.Active = true;
							updateRevisionDialog_toRevisionCheck.Active = false;
							updateRevisionDialog_revision.IsEditable = false;
							break;
				case 1:		updateRevisionDialog_toRevisionCheck.Active = true;
							updateRevisionDialog_toLastestCheck.Active = false;
							updateRevisionDialog_revision.IsEditable = true;
							break;
				default: 	updateRevisionDialog_toRevisionCheck.Active = true;
							updateRevisionDialog_toLastestCheck.Active = false;
							updateRevisionDialog_revision.IsEditable = true;
							break;			
			}
			
		}
		
		public void Show() {
		
			updateRevisionDialog.Show();
			
		}
		
		public void On_UpdateRevisionDialog_CancelButton_Clicked(object o, EventArgs args) {
		
			updateRevisionDialog.Hide();
			updateRevisionDialog.Dispose();
			
		}
		
		public void On_UpdateRevisionDialog_UpdateButton_Clicked(object o, EventArgs args) {
		
			app.postUpdateDialog( updateRevisionDialog_toLastestCheck.Active, updateRevisionDialog_revision.Text);
			updateRevisionDialog.Hide();
			updateRevisionDialog.Dispose();
			
		}
		
		public void On_UpdateRevisionDialog_ToRevisionToogled(object o, EventArgs args) {
		
			updateRevisionDialog_revision.IsEditable = true;
			
		}
		
		public void On_UpdateRevisionDialog_ToLastestToogled(object o, EventArgs args) {
		
			updateRevisionDialog_revision.IsEditable = false;
			
		}
		
	} // closing UpdateRevisionDialog class.
	
	/// <summary> 
	/// This class shows the commit dialog to user.
	/// </summary>	
	public class CommitDialog {
	
		[Widget] Gtk.Dialog commitDialog;
		// the commit message
		[Widget] Gtk.TextView commitDialog_message;
		
		private gsvn app;
		
		public CommitDialog(gsvn _app) {
		
			Glade.XML gxml = Glade.XML.FromAssembly ("gui.glade", "commitDialog", null);
			gxml.Autoconnect(this);
			
			app = _app;
			
		}
		
		public void Show() {
		
			commitDialog.Show();
			
		}
		
		public void On_commitDialog_Cancel_Clicked(object o, EventArgs args) {
		
			commitDialog.Hide();
			commitDialog.Dispose();
			
		}
		
		public void On_commitDialog_Commit_Clicked(object o, EventArgs args) {
		
			commitDialog.Hide();
			commitDialog.Dispose();
			
			app.postCommitDialog(commitDialog_message.Buffer.Text);
			
			
		}
		
		public void On_commitDialog_Clear_Clicked(object o, EventArgs args) {
		
			commitDialog_message.Buffer.Text = "";
			
		}
		
	} // closing CommitDialog class.

	/// <summary> 
	/// This class shows the commit entry to user.
	/// </summary>	
	public class CommitEntryDialog {
	
		[Widget] Gtk.Dialog commitEntryDialog;
		// the commit message
		[Widget] Gtk.TextView commitEntryDialog_message;
		
		private gsvn app;
		
		public CommitEntryDialog(gsvn _app) {
		
			Glade.XML gxml = Glade.XML.FromAssembly ("gui.glade", "commitEntryDialog", null);
			gxml.Autoconnect(this);
			
			app = _app;
			
		}
		
		public void Show() {
		
			commitEntryDialog.Show();
			
		}
		
		public void On_commitDialog_Cancel_Clicked(object o, EventArgs args) {
		
			commitEntryDialog.Hide();
			commitEntryDialog.Dispose();
			
		}
		
		public void On_commitDialog_Commit_Clicked(object o, EventArgs args) {
		
			commitEntryDialog.Hide();
			commitEntryDialog.Dispose();
			
			app.postCommitEntryDialog(commitEntryDialog_message.Buffer.Text);
			
			
		}
		
		public void On_commitDialog_Clear_Clicked(object o, EventArgs args) {
		
			commitEntryDialog_message.Buffer.Text = "";
			
		}
		
	}	
			
	/// <summary> 
	/// This class shows the Working Directory dialog to user.
	/// </summary>	
	public class WorkingDirectoryDialog {

		[Widget] Window addWorkingDirectoryDialog;
		[Widget] Gtk.Entry addWorkingDirectoryDialog_dir;
		
		public string _path = "";
		
		private fileSelection fileselection;
		
		private pfc.gsvn app;
		
		private WaitWindow pleasewaitwindow;
		
		public WorkingDirectoryDialog(gsvn mainApp) {		
		
			Glade.XML gxml = Glade.XML.FromAssembly ("gui.glade", "addWorkingDirectoryDialog", null);
			gxml.Autoconnect(this);			
			app = mainApp;
	
		}
		
		public void Show() {
		
			addWorkingDirectoryDialog.Show();
			
		}
		
		public void Hide() {
		
			addWorkingDirectoryDialog.Hide();
			
		}		
		
		public void Dispose() {
			
			addWorkingDirectoryDialog = null;
			addWorkingDirectoryDialog.Dispose();
			
		}
		
		public void On_addWorkingDirectoryDialog_CancelButton_Clicked(object o, EventArgs args) {
		
			addWorkingDirectoryDialog.Hide();
			addWorkingDirectoryDialog.Dispose();
			
		}
		
		public void wait() {
		
			pleasewaitwindow = new WaitWindow();
			pleasewaitwindow.Show();		
						
		}
		
		public void On_addWorkingDirectoryDialog_AddButton_Clicked(object o, EventArgs args) {
		
			_path = addWorkingDirectoryDialog_dir.Text;
			
			// comprobamos que el directorio es válido.
			if (WorkingCopy.is_Working_Copy( _path )) {
			 
 					addWorkingDirectoryDialog.Hide();
 					addWorkingDirectoryDialog.Dispose();
 					app.clearBrowseFilesWindow(); 
 					app.setWorkingDirectory(this._path);
 					app.clearBrowseDirWindow();					
 					app.createTreeCopy(this._path); 					
 					
 			}	
 								 					
 				else {	 					
 					WorkingCopyErrorDialog alertDialog; 					
 					alertDialog = new WorkingCopyErrorDialog();
 					alertDialog.Show();			
 			}         		
			
	
		}
		
		public void On_addWorkingDirectoryDialog_BrowseButton_Clicked(object o, EventArgs args) {
		
			fileselection = new fileSelection();
			fileselection.Show();
		
		}
		
		public void On_addWorkingDirectoryDialog_Focus_Activate(object o, EventArgs args) {
		
			if (fileselection != null) {
				addWorkingDirectoryDialog_dir.Text = fileselection.path;
			}		
		
		}
		
	} // closing WorkingDirectoryDialog class.	
	
	/// <summary> 
	/// This class shows the wait window to user. 
	/// </summary>	
	public class WaitWindow {
	
		[Widget] Gtk.Window pleaseWaitWindow;
	
		public WaitWindow() {
		
			Glade.XML gxml = Glade.XML.FromAssembly ("gui.glade", "pleaseWaitWindow", null);
			gxml.Autoconnect(this);
			
		}
		
		public void Show() {
			
			pleaseWaitWindow.Show();
			
		}
		
		public void Hide() {
		
			if (pleaseWaitWindow != null) {
		
			pleaseWaitWindow.Hide();
			pleaseWaitWindow.Dispose();
			
			}
			
		}
		
	} // closing WaitWindow Class	
	
	/// <summary> 
	/// This class shows the add property dialog to user.
	/// </summary>	
	public class PropertyEditorAddProperty {
	
		[Widget] Gtk.Dialog addPropertyDialog;
		[Widget] Gtk.Entry addPropertyDialog_name;
		[Widget] Gtk.TextView addPropertyDialog_value;
		
		public string propertyName = "";
		public string propertyValue = "";
		
		private PropList pAdd;
		private PropList pEdit;
		private int edit = 0;
		private ErrorPropertyNameDialog errorPropertyNameDialog;
	
		public PropertyEditorAddProperty(PropList _pAdd, PropList _pEdit, int _edit) {
		
			Glade.XML gxml = Glade.XML.FromAssembly ("gui.glade", "addPropertyDialog", null);
			gxml.Autoconnect(this);	
			
			pAdd = _pAdd;
			pEdit = _pEdit;
			
			edit = _edit; 
			
		}
		
		public void Show() {
			
			addPropertyDialog_name.Text = propertyName;
			TextBuffer buffer;
			
			buffer = addPropertyDialog_value.Buffer;
			buffer.Text = propertyValue;
			addPropertyDialog_name.Text = propertyName;
			if (edit==1)
				addPropertyDialog_name.Editable = false;
			addPropertyDialog.Show();
						
		}
		
		public void Hide() {
		
			if (addPropertyDialog != null) {
		
			addPropertyDialog.Hide();
			addPropertyDialog.Dispose();
			
			}
			
		}
		
		public void On_AddPropertyDialog_CancelButton_Clicked(object o, EventArgs args) {
		
			addPropertyDialog.Hide();
			
		}
		
		public void On_AddPropertyDialog_OkButton_Clicked(object o, EventArgs args) {
		
			System.Console.WriteLine("propiedad: "+addPropertyDialog_name.Text);
			System.Console.WriteLine("valor: "+addPropertyDialog_value.Buffer.Text);
			
			if (edit == 1) {
				pAdd.editProperty(addPropertyDialog_name.Text, addPropertyDialog_value.Buffer.Text);
				pEdit.insertProperty(addPropertyDialog_name.Text, addPropertyDialog_value.Buffer.Text);
				addPropertyDialog.Hide();
			}
			else {
				if (pAdd.exitsProperty(addPropertyDialog_name.Text)) {
					PleaseSelectOtherNameAlertDialog alertdialog;
					alertdialog = new PleaseSelectOtherNameAlertDialog();
					alertdialog.Show();
				}
				else { // propertie doesnt exits. 
					if (addPropertyDialog_name.Text.IndexOf(' ') != -1) {
						errorPropertyNameDialog = new ErrorPropertyNameDialog();
						errorPropertyNameDialog.Show();
					}
					else {
						pAdd.insertProperty(addPropertyDialog_name.Text, addPropertyDialog_value.Buffer.Text);
						pEdit.insertProperty(addPropertyDialog_name.Text, addPropertyDialog_value.Buffer.Text);
						addPropertyDialog.Hide();
					}
				}
			}
			
		}
		
		public void setProperty(string name, string _value) {
		
			this.propertyName = name;
			this.propertyValue = _value;
			
		}
		
	} // closing WaitWindow Class		
	
	/// <summary> 
	/// This class shows the property editor to user. 
	/// </summary>	
	public class PropertyEditor {
	
		[Widget] Gtk.Window propertyEditor;
		//private Gtk.TreeView propertyEditorTree;
		[Widget] Gtk.ScrolledWindow propertyEditorWindow;
		
		private PropertyEditorAddProperty addpropertydialog;
		
		private PropList proplist;
		private PropList proplistAdd;
		private PropList proplistDel;
		private PropList proplistEdit;
		
		private TreeStore propertyTree = null;
		private TreeView propertyTreeView;
		
		private int _exit = 0;
		
		private gsvn app;
			
		public PropertyEditor(PropList list, gsvn _app) {
		
			Glade.XML gxml = Glade.XML.FromAssembly ("gui.glade", "propertyEditor", null);
			gxml.Autoconnect(this);
			proplist = list;
			proplistAdd = new PropList();
			proplistDel = new PropList();
			proplistEdit = new PropList();
			app = _app;
		}
		
		public void loadPropertyTree() {	
		
			foreach (Gtk.Widget w in propertyEditorWindow.Children) {
				propertyEditorWindow.Remove(w);
			}
					
			propertyTree = new TreeStore( typeof(string), typeof(string) );
					
			// Append All Properties on a TreeStore.
			propertyTreeView = new TreeView(propertyTree);
			
			for (int i = 0; i < proplist.lengthProperties(); i++) {
				if (proplist.getPropertyName(i) != "")
					propertyTree.AppendValues(proplist.getPropertyName(i), proplist.getPropertyValue(i) );
			}				
			
			propertyTreeView.AppendColumn("Name", new CellRendererText (), "text", 0);
			propertyTreeView.AppendColumn("Value", new CellRendererText (), "text", 1);
			
			propertyTreeView.HeadersVisible = true;
			
			propertyEditorWindow.Add(propertyTreeView);
			propertyEditorWindow.ShowAll();			
			
		}
		
		public void Show() {

			propertyEditorWindow.ShowAll();
	
		}
		
		public void Hide() {
		
			if (propertyEditor != null) {
		
				propertyEditor.Hide();
				propertyEditor.Dispose();
			
			}
			
		}
		
		public void On_PropertyEditor_AddButton_Clicked(object o, EventArgs args) {
		
			addpropertydialog = new PropertyEditorAddProperty(proplist, proplistAdd, 0);			
			addpropertydialog.Show();
			
		}
		
		public void On_PropertyEditor_EditButton_Clicked(object o, EventArgs args) {
		
			TreeIter iter;
			TreeModel model;
			//string selected = "";
			
			PleaseSelectAlertDialog pleaseselect;
		
			if (propertyTreeView.Selection.GetSelected (out model, out iter)) {
            	string _name = (string) model.GetValue (iter, 0);
            	string _value = (string) model.GetValue (iter, 1);
            	addpropertydialog = new PropertyEditorAddProperty(proplist, proplistEdit, 1);
				addpropertydialog.setProperty(_name, _value); 
				addpropertydialog.Show();
			}
            else { 
            	pleaseselect = new PleaseSelectAlertDialog();
            	pleaseselect.Show(); 
            }
            			
		}
		
		public void On_PropertyEditor_DeleteButton_Clicked(object o, EventArgs args) {
		
			TreeIter iter;
			TreeModel model;
			//string selected = "";
			
			DeletePropertyOkDialog deleteok;
			PleaseSelectAlertDialog pleaseselect;
		
			if (propertyTreeView.Selection.GetSelected (out model, out iter)) {
            	string _name = (string) model.GetValue (iter, 0);
            	string _value = (string) model.GetValue (iter, 1);
            	proplist.deleteProperty(_name);
            	proplistDel.insertProperty(_name, _value);
            	deleteok = new DeletePropertyOkDialog();
            	deleteok.Show(); 
			}
            else { 
				pleaseselect = new PleaseSelectAlertDialog();
            	pleaseselect.Show(); 
            }
			
		}
	
		public void On_PropertyEditor_OkButton_Clicked(object o, EventArgs args) {
		
			_exit = 1;
			app.saveProperties(proplist);
			app.deleteProperties(proplistDel);
			app.editProperties(proplistEdit);
			propertyEditor.Hide();
			
		}
		
		public void On_PropertyEditor_CancelButton_Clicked(object o, EventArgs args) {
		
			_exit = 1;
			propertyEditor.Hide();
			
		}
		
		public void on_propertyEditor_activate_focus(object o, EventArgs args) {
		
			this.loadPropertyTree();
			this.propertyEditor.ShowAll();	
		}
			
		public void on_propertyEditor_delete_focus(object o, EventArgs args) {
			
			if (_exit == 1)
				propertyEditor.Hide();
			
		}
		
	} // closing WaitWindow Class	
	
	/// <summary> 
	/// This classs shows the revert confirm dialog to user. 
	/// </summary>	
	public class RevertConfirmDialog {
	
		[Widget] Dialog revertConfirmDialog;
		
		private gsvn app;
		
		public RevertConfirmDialog(gsvn _app) {
		
			Glade.XML gxml = Glade.XML.FromAssembly ("gui.glade","revertConfirmDialog",null);
       		gxml.Autoconnect (this);   
       		app = _app;  		
       		
       	}
       	
       	public void Show() {
       	
       		revertConfirmDialog.Show();
       		
       	}
       	
       	public void On_RevertConfirmDialog_OkButton_Clicked(object o, EventArgs args) {
       	
       		revertConfirmDialog.Hide();
       		revertConfirmDialog.Dispose();
       		app.doRevert();
       		
       	} 
       	
       	public void On_RevertConfirmDialog_CancelButton_Clicked(object o, EventArgs args) {
       	
       		revertConfirmDialog.Hide();
       		revertConfirmDialog.Dispose();
       		
       	}       	       
       	

       	
	} 
	
	/// <summary> 
	/// This class shows the view dialog to user. 
	/// </summary>	
	public class ViewDialog {
	
		[Widget] Dialog viewDialog;
		[Widget] TextView viewDialog_content;
		
		//private gsvn app;
		
		private string content;
		
		public ViewDialog(string _content) {
		
			Glade.XML gxml = Glade.XML.FromAssembly ("gui.glade","viewDialog",null);
       		gxml.Autoconnect (this);    		
       		content = _content;
       		
       	}
       	
       	public void Show() {
       	
       		TextBuffer buffer;
       		
       		buffer = viewDialog_content.Buffer;
       		
       		buffer.Text = content;
       	
       		viewDialog.Show();
       		
       	}
       	
       	public void On_ViewDialog_CloseButton_Clicked(object o, EventArgs args) {
       	
       		viewDialog.Hide();
       		viewDialog.Dispose();
       		
       	} 
       	
	} 
	
	/// <summary>
	/// This class shows the blame dialog to user. 
	/// </summary>
	public class BlameDialog {
	
		[Widget] Dialog blameDialog;
		
		public BlameDialog() {
		
			Glade.XML gxml = Glade.XML.FromAssembly ("gui.glade","blameDialog",null);
       		gxml.Autoconnect (this);    		
       		
       	}
       	
       	public void Show() {
       	
       		blameDialog.Show();
       		
       	}
       	
       	public void On_BlameDialog_CloseButton_Clicked(object o, EventArgs args) {
       	
       		blameDialog.Hide();
       		blameDialog.Dispose();
       		
       	} 
       	
	} 	
	
	/// <summary> 
	/// This class shows the copy local to local dialog to user. 
	/// </summary>	
	public class CopyLocaltoLocalDialog {
	
		[Widget] Dialog copyLocaltoLocalDialog;
		[Widget] Entry copyLocaltoLocalDialog_to;
		[Widget] TextView copyLocaltoLocalDialog_message;
		
		private fileSelection fileselection;
		private gsvn app;
		private string from;
		
		public CopyLocaltoLocalDialog(string FROM, gsvn _app) {
		
			Glade.XML gxml = Glade.XML.FromAssembly ("gui.glade","copyLocaltoLocalDialog",null);
       		gxml.Autoconnect (this);    		
       		app = _app;
       		from = FROM;
       	}
       	
       	public void Show() {
       	
       		copyLocaltoLocalDialog.Show();
       		
       	}
       	
       	public void On_CopyLocaltoLocalDialog_OkButton_Clicked(object o, EventArgs args) {
       	
       		if ( (copyLocaltoLocalDialog_to.Text != "") ) {
       			app.postCopyRemotetoRemoteDialog(from, copyLocaltoLocalDialog_to.Text, copyLocaltoLocalDialog_message.Buffer.Text);       			
       			copyLocaltoLocalDialog.Hide();
       			copyLocaltoLocalDialog.Dispose();
       		}
       		else {;}
       		
       	} 
       	
       	public void On_CopyLocaltoLocalDialog_CancelButton_Clicked(object o, EventArgs args) {
       	
       		copyLocaltoLocalDialog.Hide();
       		copyLocaltoLocalDialog.Dispose();
       		
       	}        	
       	
       	public void On_CopyLocaltoLocalDialog_BrowseButton_Clicked(object o, EventArgs args) {
       	
       		fileselection = new fileSelection();
       		fileselection.Show();
       		
       	}
       	
		public void On_Focus_Activate(object o, EventArgs args) {
		
			if (fileselection != null) {
				this.copyLocaltoLocalDialog_to.Text = fileselection.path;
			}		
		
		}             	
       	
	}
	
	/// <summary> 
	/// This class shows the Make Directory dialog to user. 
	/// </summary>	
	public class MakeDirectoryDialog {
	
		[Widget] Dialog makeDirectoryDialog;
		[Widget] Entry makeDirectoryDialog_name;
		
		private gsvn app;

		public MakeDirectoryDialog(	gsvn _app ) {
		
			Glade.XML gxml = Glade.XML.FromAssembly ("gui.glade","makeDirectoryDialog",null);
       		gxml.Autoconnect (this); 
       		app = _app;   
       		       		
       	}
       	
       	public void Show() {
       	
       		makeDirectoryDialog.Show();
       		
       	}
       	
       	public void On_MakeDirectoryDialog_OkButton_Clicked(object o, EventArgs args) {
       	
       		app.postMakeDirectoryDialog( makeDirectoryDialog_name.Text);
       		makeDirectoryDialog.Hide();
       		makeDirectoryDialog.Dispose();
       		
       	} 
       	
       	public void On_MakeDirectoryDialog_CancelButton_Clicked(object o, EventArgs args) {
       	
       		makeDirectoryDialog.Hide();
       		makeDirectoryDialog.Dispose();
       		
       	}        	
       	
	} 				 		 			 		 		
				
	/// <summary> 
	/// This class shows the rename dialog to user. 
	/// </summary>				
	public class RenameDialog {
	
		[Widget] Dialog renameDialog;
		[Widget] Entry renameDialog_newName;
		[Widget] TextView renameDialog_message;
		
		private gsvn app;
			
		public RenameDialog(gsvn _app) {
		
			Glade.XML gxml = Glade.XML.FromAssembly ("gui.glade","renameDialog",null);
       		gxml.Autoconnect (this);    		
       		app = _app;
       		
       	}
       	
       	public void Show() {
       	
       		renameDialog.Show();
       		
       	}
       	
       	public void On_RenameDialog_OkButton_Clicked(object o, EventArgs args) {
       	
       		app.postRenameDialog(renameDialog_newName.Text, renameDialog_message.Buffer.Text);
       		renameDialog.Hide();
       		renameDialog.Dispose();
       	
       	} 
       	
       	public void On_RenameDialog_CancelButton_Clicked(object o, EventArgs args) {
       	
       		renameDialog.Hide();
       		renameDialog.Dispose();
       		
       	}        	
       	
	} 	
						
	/// <summary> 
	/// This class shows the Info Dialog to user. 
	/// </summary>						
	public class InfoDialog {
	
		[Widget] Dialog infoDialog;
		[Widget] TextView fileInfoTextView;
		
		private pfc.Svn.WorkingCopy wc;
		private string path;
		
		public InfoDialog(pfc.Svn.WorkingCopy workingcopy, string selected) {
		
			Glade.XML gxml = Glade.XML.FromAssembly ("gui.glade","infoDialog",null);
       		gxml.Autoconnect (this); 
       		wc = workingcopy;   
       		path = selected;		
       		
       	}
       	
       	public void Show() {
       	
       		string fileInfo = wc.infoRaw(path);
       		
       		Gtk.TextBuffer buffer;
       		
			buffer = fileInfoTextView.Buffer;
			buffer.Text = fileInfo;
       		
       		infoDialog.Show();
       		
       	}
       	
       	public void On_InfoDialog_CloseButton_Clicked(object o, EventArgs args) {
       	
       		infoDialog.Hide();
       		infoDialog.Dispose();
       		
       	}        	
       	
	} 			
	
	/// <summary> 
	/// This class shows the log dialog to user. 
	/// </summary>
	public class LogDialog {
	
		[Widget] Dialog logDialog;
		
		public LogDialog() {
		
			Glade.XML gxml = Glade.XML.FromAssembly ("gui.glade","logDialog",null);
       		gxml.Autoconnect (this);    		
       		
       	}
       	
       	public void Show() {
       	
       		logDialog.Show();
       		
       	}
       	
       	public void On_LogDialog_CloseButton_Clicked(object o, EventArgs args) {
       	
       		logDialog.Hide();
       		logDialog.Dispose();
       		
       	}        	
       	
	}
	
	/// <summary> 
	/// This class shows the move local dialog to user. 
	/// </summary>	
	public class MoveLocalDialog {
	
		[Widget] Dialog moveLocalDialog;
		[Widget] Entry moveLocalDialog_from;
		[Widget] Entry moveLocalDialog_to;
		[Widget] TextView moveLocalDialog_message;
		
		private gsvn app;
		private string element = "";
		private fileSelection fileselection;
		
		public MoveLocalDialog(gsvn _app, string _dir) {
		
			Glade.XML gxml = Glade.XML.FromAssembly ("gui.glade","moveLocalDialog",null);
       		gxml.Autoconnect (this);  
       		app = _app; 
       		element = _dir; 		
       		
       	}
       	
       	public void Show() {
       	
       		moveLocalDialog.Show();
       		moveLocalDialog_from.Text = element;
       		
       	}
       	
       	public void On_MoveLocalDialog_CancelButton_Clicked(object o, EventArgs args) {
       	
       		moveLocalDialog.Hide();
       		moveLocalDialog.Dispose();
       		
       	}        	
       	 
       	public void On_MoveLocalDialog_BrowseButton_Clicked(object o, EventArgs args) {
       	
       		fileselection = new fileSelection();
       		fileselection.Show();
       		
       	}
       	
       	public void On_MoveLocalDialog_OkButton_Clicked(object o, EventArgs args) {
       		
       		if ( this.moveLocalDialog_to.Text != "") {
       			app.postMoveOnRepository( moveLocalDialog_from.Text, moveLocalDialog_to.Text, moveLocalDialog_message.Buffer.Text);
       			moveLocalDialog.Hide();
       			moveLocalDialog.Dispose();
       		}
       		else {;}
       		
       	}        	 
       	
		public void On_Focus_Activate(object o, EventArgs args) {
		
			if (fileselection != null) {
				this.moveLocalDialog_to.Text = fileselection.path;
			}		
		
		}             	          	
       	
	} 

	/// <summary> 
	/// This class shows the move remote dialog to user. 
	/// </summary>	
	public class MoveRemoteDialog {
	
		[Widget] Dialog moveRemoteDialog;
		[Widget] Entry 	moveRemoteDialog_from;
		[Widget] Entry 	moveRemoteDialog_to;
		[Widget] TextView moveRemoteDialog_message;
		
		private gsvn app;
		
		public MoveRemoteDialog(gsvn _app) {
		
			Glade.XML gxml = Glade.XML.FromAssembly ("gui.glade","moveRemoteDialog",null);
       		gxml.Autoconnect (this);
       		
       		app = _app;    		
       		
       	}
       	
       	public void Show() {
       	
       		moveRemoteDialog.Show();
       		
       	}
       	
       	public void On_MoveRemoteDialog_CancelButton_Clicked(object o, EventArgs args) {
       	
       		moveRemoteDialog.Hide();
       		moveRemoteDialog.Dispose();
       		
       	}    
       	
       	public void On_MoveRemoteDialog_OkButton_Clicked(object o, EventArgs args) {
       	
       		if ( (moveRemoteDialog_from.Text != "") && (moveRemoteDialog_to.Text != "") ) {      	
       			app.postMoveOnRepository(moveRemoteDialog_from.Text, moveRemoteDialog_to.Text, moveRemoteDialog_message.Buffer.Text);
       			moveRemoteDialog.Hide();
       			moveRemoteDialog.Dispose();
       		}
       		else {
       			;
       		}
       		
       	}       	    	    	
       	
	}
	
	/// <summary> 
	/// This class shows the Copy Remote to local dialog to user. 
	/// </summary>	
	public class CopyRemotetoLocalDialog {
	
		[Widget] Dialog 	copyRemotetoLocalDialog;
		[Widget] Entry		copyRemotetoLocalDialog_from;
		[Widget] Entry 		copyRemotetoLocalDialog_to;
		[Widget] TextView	copyRemotetoLocalDialog_message;
		
		private fileSelection fileselection;
		private gsvn app;
		
		public CopyRemotetoLocalDialog(gsvn _app, string DIR_TO) {
		
			Glade.XML gxml = Glade.XML.FromAssembly ("gui.glade","copyRemotetoLocalDialog",null);
       		gxml.Autoconnect (this);    		
       		app = _app;
       		copyRemotetoLocalDialog_to.Text = DIR_TO;
       		
       	}
       	
       	public void Show() {
       	
       		copyRemotetoLocalDialog.Show();
       		
       	}
       	
       	public void On_CopyRemoteLocalDialog_CancelButton_Clicked(object o, EventArgs args) {
       	
       		copyRemotetoLocalDialog.Hide();
       		copyRemotetoLocalDialog.Dispose();
       		
       	}    
       	
       	public void On_CopyRemoteLocalDialog_OkButton_Clicked(object o, EventArgs args) {
       	
       		if ( (copyRemotetoLocalDialog_from.Text != "") && (copyRemotetoLocalDialog_to.Text != "") ) {
       			app.postCopyRemotetoRemoteDialog( copyRemotetoLocalDialog_from.Text, copyRemotetoLocalDialog_to.Text,
       					copyRemotetoLocalDialog_message.Buffer.Text);
       			copyRemotetoLocalDialog.Hide();
       			copyRemotetoLocalDialog.Dispose();
       		}
       		else {;}
       		
       	}      
       	
       	public void On_CopyRemoteLocalDialog_BrowseButton_Clicked(object o, EventArgs args) {
       	
       		fileselection = new fileSelection();
       		fileselection.Show();
       		
      	} 
      	
		public void On_Focus_Activate(object o, EventArgs args) {
		
			if (fileselection != null) {
				this.copyRemotetoLocalDialog_to.Text = fileselection.path;
			}		
		
		}      		    	    		    	    	
       	
	}
	
	/// <summary> 
	/// This class shows the copy local to remote dialog to user. 
	/// </summary>	
	public class CopyLocaltoRemoteDialog {
	
		[Widget] Dialog copyLocaltoRemoteDialog;
		//[Widget] Entry	copyLocaltoRemoteDialog_from;
		[Widget] Entry 	copyLocaltoRemoteDialog_to;
		[Widget] TextView copyLocaltoRemoteDialog_message;

		private gsvn app;
		private string from;
		
		public CopyLocaltoRemoteDialog(string FROM, gsvn _app) {
		
			Glade.XML gxml = Glade.XML.FromAssembly ("gui.glade","copyLocaltoRemoteDialog",null);
       		gxml.Autoconnect (this);    		
       		
       		app = _app;
       		from = FROM;
       		
       	}
       	
       	public void Show() {
       	
       		copyLocaltoRemoteDialog.Show();
       		
       	}
       	
       	public void On_CopyLocaltoRemoteDialog_CancelButton_Clicked(object o, EventArgs args) {
       	
       		copyLocaltoRemoteDialog.Hide();
       		copyLocaltoRemoteDialog.Dispose();
       		
       	}    
       	
       	public void On_CopyLocaltoRemoteDialog_OkButton_Clicked(object o, EventArgs args) {
       	
       		if (this.copyLocaltoRemoteDialog_to.Text != "") {
       			app.postCopyRemotetoRemoteDialog(from, copyLocaltoRemoteDialog_to.Text, copyLocaltoRemoteDialog_message.Buffer.Text);       	
	       		copyLocaltoRemoteDialog.Hide();
	       		copyLocaltoRemoteDialog.Dispose();
	       	}
	       	else {;}
       		
       	}   
       	
	}
	
	/// <summary> 
	/// This class shows the command output dialog to user. 
	/// </summary>
	public class CommandOutputWindow {
	
		[Widget] Window commandOutputWindow;
		[Widget] ScrolledWindow commandOutputWindow_ScrolledWindow;
		//[Widget] ScrolledWindow commandOutputWindow_ScrolledErrorWindow;
		[Widget] Button commandOutputWindow_closeButton;
		[Widget] ProgressBar commandOutputWindow_progressBar;
		[Widget] Notebook commandOutputWindow_notebook;
		//[Widget] Label commandOutputWindow_errorLabel;

		private TextView commandOutputWindow_textView;
		//private TextView commandOutputWindow_errorView;
		
		private string CommandMessage = "";
		private string ErrorMessage = "";
		private TextView errorView;
		
		public CommandOutputWindow() {
		
			Glade.XML gxml = Glade.XML.FromAssembly ("gui.glade","commandOutputWindow",null);
       		gxml.Autoconnect (this);  
       		
       		commandOutputWindow_closeButton.Sensitive = false;    
       		  		
       		commandOutputWindow_textView = new TextView();   
      	    		
       		commandOutputWindow_ScrolledWindow.Add(commandOutputWindow_textView); 
     		
			commandOutputWindow_ScrolledWindow.ShowAll();
			      		       		
       	}
       	
 	
       	public void Show() {
       	
       		commandOutputWindow_ScrolledWindow.ShowAll();
       		commandOutputWindow.Show();
       		
       	}
       	
       	public void On_CommandOutputWindow_CloseButton_Clicked(object o, EventArgs args) {
       	
       		commandOutputWindow.Hide();
       		commandOutputWindow.Dispose();
       		
       	}   
       	
       	public void finalizeCommandOutput() {
       	
       		this.commandOutputWindow_progressBar.BarStyle =  Gtk.ProgressBarStyle.Discrete;
       		commandOutputWindow_closeButton.Sensitive = true;    
       		commandOutputWindow_progressBar.Text = "Done!";		
       		
    	}	
    	
    	public void refreshCommandBuffer() {
    	
      		Gtk.TextBuffer buffer;
       		
       		buffer = commandOutputWindow_textView.Buffer;
       		
       		buffer.Insert(buffer.EndIter, this.CommandMessage);
       		
       		System.Console.WriteLine("Refresco el báfer");
       		
       		this.CommandMessage = "";
       		
       		TextMark endMark = buffer.CreateMark("end", buffer.EndIter, false);
       		
			commandOutputWindow_textView.ScrollToMark(endMark, 0.4, true, 0.0, 1.0);
			buffer.DeleteMark(endMark);
			
			commandOutputWindow_progressBar.Pulse();
			commandOutputWindow_progressBar.Text = "Please wait...";
       		
       	} 
       	
       	public void refreshErrorBuffer() {
       	
       		if (commandOutputWindow_notebook.NPages == 1) {
       			createErrorPage();
       		}
       	
     		Gtk.TextBuffer buffer;
       		
       		buffer = errorView.Buffer;
       		
       		buffer.Insert(buffer.EndIter, this.ErrorMessage);
       		
       		System.Console.WriteLine("Refresco el báfer de error");
       		
       		this.ErrorMessage = "";
       		
       	}       		
       	
       	public void setCommandMessage(string _message) {
       	
       		CommandMessage += _message + "\n";       		
       		
       	}
       	
       	public void setErrorMessage(string _message) {
       	
       		ErrorMessage += _message + "\n";
       		
       	}
       	
       	public void createErrorPage() {
       	
       		ScrolledWindow w;
       		Gtk.TextBuffer buffer;
       		
       		w = new ScrolledWindow();
       		errorView = new TextView();
       		buffer = errorView.Buffer;
       		buffer.Text = "";
       		errorView.Editable = false;
       		
       		w.VscrollbarPolicy = Gtk.PolicyType.Automatic;
       		w.HscrollbarPolicy = Gtk.PolicyType.Automatic;
       		
       		w.Add(errorView);
       		
       		commandOutputWindow_notebook.AppendPage(w, new Label("Error output") );
       		
       		commandOutputWindow_notebook.Page = 2;
       		
       		commandOutputWindow_notebook.ShowAll();
       	
       	}
       	
	}	
	
	/// <summary>
	/// This class shows the Switch Dialog to user. 
	/// </summary>
	public class SwitchDialog {
	
		[Widget] Dialog switchDialog;
		[Widget] Entry switchDialog_newURL;
		
		private gsvn app;
		
		public SwitchDialog(gsvn _app) {
		
		Glade.XML gxml = Glade.XML.FromAssembly ("gui.glade","switchDialog",null);
      		gxml.Autoconnect (this);   		
       		app = _app;
       		
       	}
       	
       	public void Show() {
       	
       		switchDialog.Show();
       		
       	}
       	
       	public void On_SwitchDialog_CancelButton_Clicked(object o, EventArgs args) {
       	
       		switchDialog.Hide();
       		switchDialog.Dispose();
       		
       	}
       	
       	public void On_SwitchDialog_OkButton_Clicked(object o, EventArgs args) {
       	
       		app.postSwitchDialog( switchDialog_newURL.Text );
       		switchDialog.Hide();
       		switchDialog.Dispose();
       		
       	}  
       	
	} 	
	
	/// <summary>
	/// This class shows the Copy Remote to Remote Dialog to user. 
	/// </summary>
	public class CopyRemotetoRemoteDialog {
	
		[Widget] Dialog 	copyRemotetoRemoteDialog;
		[Widget] Entry 		copyRemotetoRemoteDialog_from;
		[Widget] Entry 		copyRemotetoRemoteDialog_to;
		[Widget] TextView 	copyRemotetoRemoteDialog_message;
		
		private gsvn app;
		
		public CopyRemotetoRemoteDialog(gsvn _app) {
		
			Glade.XML gxml = Glade.XML.FromAssembly ("gui.glade","copyRemotetoRemoteDialog",null);
      		gxml.Autoconnect (this);   		
       		app = _app;
       		
       	}
       	
       	public void Show() {
       	
       		copyRemotetoRemoteDialog.Show();
       		
       	}
       	
       	public void On_CopyRemotetoRemoteDialog_CancelButton_Clicked(object o, EventArgs args) {
       	
       		copyRemotetoRemoteDialog.Hide();
       		copyRemotetoRemoteDialog.Dispose();
       		
       	}
       	
       	public void On_CopyRemotetoRemoteDialog_OkButton_Clicked(object o, EventArgs args) {
       		
       		if ( (copyRemotetoRemoteDialog_from.Text != "") && (copyRemotetoRemoteDialog_to.Text != "") ) {
       			app.postCopyRemotetoRemoteDialog(copyRemotetoRemoteDialog_from.Text, copyRemotetoRemoteDialog_to.Text, copyRemotetoRemoteDialog_message.Buffer.Text);    		
       			copyRemotetoRemoteDialog.Hide();
       			copyRemotetoRemoteDialog.Dispose();
       		}
       		else {;}
       		
       	}  
       	
	}
	
	/// <summary>
	/// This class shows the preferences Window to user. 
	/// </summary>	
	public class PreferencesDialog {
	
		[Widget] Dialog preferencesDialog;
		
		[Widget] Entry preferences_svntool;
		[Widget] Entry preferences_svnadmintool;
		[Widget] Entry preferences_svnlooktool;
		[Widget] Entry preferences_svnservetool;
		
		[Widget] Entry preferences_tartool;
		[Widget] Entry preferences_gziptool;
		[Widget] Entry preferences_ziptool;
		
		[Widget] Entry preferences_edittool;
		[Widget] Entry preferences_browsertool;
		
		
		private SvnConfig svnconfig;
		
		public PreferencesDialog(SvnConfig _svnconfig) {
		
			Glade.XML gxml = Glade.XML.FromAssembly ("gui.glade","preferencesDialog",null);
      		gxml.Autoconnect (this); 
      		svnconfig = _svnconfig;
      		    		
       	}
       	
       	public void Show() {
       	
       		preferences_svntool.Text = svnconfig.getSvn();
       		preferences_svnlooktool.Text = svnconfig.getSvnlook();
       		preferences_svnadmintool.Text = svnconfig.getSvnadmin();
       		preferences_svnservetool.Text = svnconfig.getSvnserve();
       		
       		preferences_tartool.Text = svnconfig.getTar();
       		preferences_ziptool.Text = svnconfig.getZip();
       		preferences_gziptool.Text = svnconfig.getGzip();
       		
       		preferences_edittool.Text = svnconfig.getEdit();
       		preferences_browsertool.Text = svnconfig.getBrowser();
       		
       		preferencesDialog.Show();
       		
       	}
       	
       	public void On_PreferencesDialog_CancelButton_Clicked(object o, EventArgs args) {
       	
       		preferencesDialog.Hide();
       		preferencesDialog.Dispose();
       		
       	}
       	
       	public void On_PreferencesDialog_OkButton_Clicked(object o, EventArgs args) {
       	
       		svnconfig.setSvn( preferences_svntool.Text );
       		svnconfig.setSvnlook( preferences_svnlooktool.Text );
       		svnconfig.setSvnadmin( preferences_svnadmintool.Text);
       		svnconfig.setSvnserve ( preferences_svnservetool.Text );
       		
       		svnconfig.setTar( preferences_tartool.Text );
       		svnconfig.setZip( preferences_ziptool.Text );
       		svnconfig.setGzip( preferences_gziptool.Text );
       		
       		svnconfig.setEdit( preferences_edittool.Text );
       		svnconfig.setBrowser( preferences_browsertool.Text );
       		
       		preferencesDialog.Hide();
       		preferencesDialog.Dispose();
       		
       	}    
       	
	}
	
	/// <summary>
	/// This class shows the Create Repository Dialog to user. 
	/// </summary>	
	public class CreateRepositoryDialog {
	
		[Widget] Dialog createRepositoryDialog;
		[Widget] Entry createRepositoryDialog_name;
		
		private gsvn app;
		
		public CreateRepositoryDialog(gsvn _app) {
		
		Glade.XML gxml = Glade.XML.FromAssembly ("gui.glade","createRepositoryDialog",null);
      		gxml.Autoconnect (this);   		
       		app = _app;
       		
       	}
       	
       	public void Show() {
       	
       		createRepositoryDialog.Show();
       		
       	}
       	
       	public void On_CreateRepository_CancelButton_Clicked(object o, EventArgs args) {
       	
       		createRepositoryDialog.Hide();
       		createRepositoryDialog.Dispose();
       		
       	}
       	
       	public void On_CreateRepository_OkButton_Clicked(object o, EventArgs args) {
       	
       		app.postCreateDialog(createRepositoryDialog_name.Text);
       		createRepositoryDialog.Hide();
       		createRepositoryDialog.Dispose();
       		
       	}  
       	
	} 																																			 																																	 																																	 				 																																	 																																			 																																	 																																	 				 																																	 																																			 																																	 																																													 																																			 																																	 																																	 				 																																	 																																			 																																	 																																	 				 																																	 																																			 																																	 																																	 		 															 																																			 																																	 																																	 				 																																	 																																			 																																	 																																	 				 																																	 																																			 																																	 																																													 																																			 																																	 																																	 				 																																	 																																			 																																	 																																	 				 																																	 																																			 																																	 																																	

	
} //closing namespace  	