#region

//        Copyright (C) 2004, 2005. Antonio Gutiérrez Mayoral
//                 <agutierr@gsyc.escet.urjc.es>
// 
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License as
//  published by the Free Software Foundation; either version 2 of the
//  License, or (at your option) any later version.
// 
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  General Public License for more details.
// 
//  You should have received a copy of the GNU General Public
//  License along with this program; if not, write to the
//  Free Software Foundation, Inc., 59 Temple Place - Suite 330,
//  Boston, MA 02111-1307, USA.

#endregion

using System;
using System.IO;
using System.Xml;

using Gtk;
using Glade;

using pfc.Svn;

namespace pfc {

	/// <summary> This class is used for add Icons to Gsvn app... defines the
	/// path to all the icons used on App. </summary>
	public class GsvnIcons {
	
		//private static string projectHome = "/home/agutierr/pfc/src/";
		private static string projectHome = "";
	
		private static string iconFolder = "icons/";		
		
		private static string genericFolder = "Folder.png";
		
		private static string genericDocument = "Document.png";
		
		private static string genericServer = "Server.png";
		
		private static string generic = "Generic/";
		
		private static string docs = "DocumentTypes/";
		
		private static string images = "ImageTypes/";
		
		private static string sound = "SoundTypes/";
		
		private static string source = "SourceTypes/";
		
		/// <summary> Extensions for Document types. </summary>
		private static string docType = "doc.png";
		private static string dviType = "dvi.png";
		private static string pdfType = "pdf.png";
		private static string pptType = "ppt.png";
		private static string psType = "ps.png";
		private static string rtfType = "rtf.png";
		private static string texType = "tex.png";
		private static string xlsType = "xls.png";
		/// <summary> Extensions for Image types. </summary>
		private static string bmpType = "bmp.png";
		private static string epsType = "eps.png";
		private static string gifType = "gif.png";
		private static string jpgType = "jpg.png";
		private static string pbmType = "pbm.png";
		private static string pngType = "png.png";
		private static string psdType = "psd.png";
		private static string svgType = "svg.png";
		private static string tgaType = "tga.png";
		private static string xwdType = "xwd.png";
		/// <summary> Extensions for Audio types. </summary>
		private static string ac3Type = "ac3.png";
		private static string aifType = "aif.png";
		private static string auType = "au.png";
		private static string midType = "mid.png";
		private static string modType = "mod.png";
		private static string mp3Type = "mp3.png";
		private static string oggType = "ogg.png";
		private static string rmType = "rm.png";
		private static string raType = "ra.png";
		private static string wavType = "wav.png";
		/// <summary> Extensions for source types. </summary>
		private static string bakType = "bak.png";
		private static string classType = "class.png";
		private static string cType = "c.png";
		private static string cssType = "css.png";
		private static string jarType = "jar.png";
		private static string hType = "h.png";
		private static string cppType = "cpp.png";
		private static string phpType = "php.png";
		private static string plType = "pl.png";
		private static string pycType = "pyc.png";
		private static string pyType = "py.png";
		private static string shType = "sh.png";
		private static string sqlType = "sql.png";
		private static string xmlType = "xml.png";

		/// <sumary> Inicialize path for icons </sumary>
		public static void getPath() {
			
			string[] argv;
			argv = Environment.GetCommandLineArgs();
			
			projectHome = argv[0].Substring(0, argv[0].LastIndexOf('/') + 1);
		}

		/// <summary> Return The size of the icon. </summary>
		/// <returns> Size of the icon </returns>
		private static string getIconSize(Config config) {
						
			if ( config.getIconSize() )
				return "normal/";
			else
				return "mini/";
		}

		/// <summary> Return the folder for the icons </summary>
		/// <returns> Path to icon's folder. </returns>
		public static string Folder(Config c) {
		
			getPath();
		
			return (projectHome + iconFolder + generic + getIconSize(c) + genericFolder);
			
		}		
		
		/// <summary> Returns the 'Server' Icon. </summary>
		/// <returns> Returns the 'Server' Icon, </returns>
		public static string Server(Config c) {
			
			getPath();
		
			return (projectHome + iconFolder + generic + getIconSize(c) + genericServer);
			
		}
		
		/// <summary> Returns the 'Document' Icon. </summary>
		/// <returns> Returns the 'Document' Icon, </returns>		
		public static string Document(Config c) {
		
			getPath();
		
			return (projectHome + iconFolder + generic + getIconSize(c) + genericDocument);
			
		}
		
		/// <summary> Returns the asociated icon for a determinated extension. </summary>
		/// <returns> Path to icon. </returns>		
		public static string getIcon(string name, Config c) {
		
			string ext;
			string iconPath = "";
			
			getPath();
			
			try {
				ext = name.Substring(name.IndexOf('.'), name.Length-name.IndexOf('.'));
			}
			catch (Exception e) {
				return (projectHome + iconFolder + genericDocument);
			}		
			
			iconPath = (projectHome + iconFolder + generic + getIconSize(c) + genericDocument);
			
			if (ext == ".doc")
				iconPath = (projectHome + iconFolder + docs + getIconSize(c) +  docType);
			if (ext == ".dvi")
				iconPath = (projectHome + iconFolder + docs + getIconSize(c) +  dviType);
			if (ext == ".pdf")
				iconPath = (projectHome + iconFolder + docs + getIconSize(c) +  pdfType);
			if (ext == ".ppt")
				iconPath = (projectHome + iconFolder + docs + getIconSize(c) +  pptType);
			if (ext == ".ps")
				iconPath = (projectHome + iconFolder + docs + getIconSize(c) +  psType);
			if (ext == ".rtf")
				iconPath = (projectHome + iconFolder + docs + getIconSize(c) +  rtfType);
			if (ext == ".tex")
				iconPath = (projectHome + iconFolder + docs + getIconSize(c) +  texType);
			if (ext == ".xls")
				iconPath = (projectHome + iconFolder + docs + getIconSize(c) +  xlsType);
		
				// imagenes
			if (ext == ".bmp")
				iconPath = (projectHome + iconFolder + images + getIconSize(c) +  bmpType);
			if (ext == ".eps")
				iconPath = (projectHome + iconFolder + images + getIconSize(c) +  epsType);
			if (ext == ".gif")
				iconPath = (projectHome + iconFolder + images + getIconSize(c) +  gifType);
			if (ext == ".jpg")
				iconPath = (projectHome + iconFolder + images + getIconSize(c) +  jpgType);
			if (ext == ".pbm")
				iconPath = (projectHome + iconFolder + images + getIconSize(c) +  pbmType);
			if (ext == ".png")
				iconPath = (projectHome + iconFolder + images + getIconSize(c) +  pngType);
			if (ext == ".psd")
				iconPath = (projectHome + iconFolder + images + getIconSize(c) +  psdType);
			if (ext == ".svg")
				iconPath = (projectHome + iconFolder + images + getIconSize(c) +  svgType);
			if (ext == ".tga")
				iconPath = (projectHome + iconFolder + images + getIconSize(c) +  tgaType);
			if (ext == ".xwd")
				iconPath = (projectHome + iconFolder + images + getIconSize(c) +  xwdType);
	
				// sonido
			if (ext == ".ac3")
				iconPath = (projectHome + iconFolder + sound + getIconSize(c) +  ac3Type);
			if (ext == ".aif")
				iconPath = (projectHome + iconFolder + sound + getIconSize(c) +  aifType);
			if (ext == ".au")
				iconPath = (projectHome + iconFolder + sound + getIconSize(c) +  auType);
			if (ext == ".mid")
				iconPath = (projectHome + iconFolder + sound + getIconSize(c) +  midType);
			if (ext == ".mod")
				iconPath = (projectHome + iconFolder + sound + getIconSize(c) +  modType);
			if (ext == ".mp3")
				iconPath = (projectHome + iconFolder + sound + getIconSize(c) +  mp3Type);
			if (ext == ".ogg")
				iconPath = (projectHome + iconFolder + sound + getIconSize(c) +  oggType);
			if (ext == ".rm")
				iconPath = (projectHome + iconFolder + sound + getIconSize(c) +  rmType);
			if (ext == ".ra")
				iconPath = (projectHome + iconFolder + sound + getIconSize(c) +  raType);
			if (ext == ".wav")
				iconPath = (projectHome + iconFolder + sound + getIconSize(c) +  wavType);

				// ficheros fuente
			if (ext == ".bak")
				iconPath = (projectHome + iconFolder + source + getIconSize(c) +  bakType);
			if (ext == ".class")
				iconPath = (projectHome + iconFolder + source + getIconSize(c) +  classType);
			if (ext == ".c")
				iconPath = (projectHome + iconFolder + source + getIconSize(c) +  cType);
			if (ext == ".css")
				iconPath = (projectHome + iconFolder + source + getIconSize(c) +  cssType);
			if (ext == ".jar")
				iconPath = (projectHome + iconFolder + source +  getIconSize(c) +  jarType);
			if (ext == ".cpp")
				iconPath = (projectHome + iconFolder + source + getIconSize(c) +  cppType);
			if (ext == ".h")
				iconPath = (projectHome + iconFolder + source + getIconSize(c) +  hType);
			if (ext == ".php")
				iconPath = (projectHome + iconFolder + source + getIconSize(c) +  phpType);
			if (ext == ".pl")
				iconPath = (projectHome + iconFolder + source + getIconSize(c) +  plType);
			if (ext == ".py")
				iconPath = (projectHome + iconFolder + source + getIconSize(c) +  pyType);
			if (ext == ".pyc")
				iconPath = (projectHome + iconFolder + source + getIconSize(c) +  pycType);
			if (ext == ".sh")
				iconPath = (projectHome + iconFolder + source + getIconSize(c) +  shType);
			if (ext == ".sql")
				iconPath = (projectHome + iconFolder + source + getIconSize(c) +  sqlType);
			if (ext == ".xml")
				iconPath = (projectHome + iconFolder + source + getIconSize(c) +  xmlType);
				
			return iconPath;
			
		}		
		
	} // closing GsvnIcons class
	
} // Closing pfc namespace