#region

//        Copyright (C) 2004, 2005. Antonio Gutiérrez Mayoral
//                 <agutierr@gsyc.escet.urjc.es>
// 
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License as
//  published by the Free Software Foundation; either version 2 of the
//  License, or (at your option) any later version.
// 
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  General Public License for more details.
// 
//  You should have received a copy of the GNU General Public
//  License along with this program; if not, write to the
//  Free Software Foundation, Inc., 59 Temple Place - Suite 330,
//  Boston, MA 02111-1307, USA.

#endregion

using Gtk;
using System;

namespace pfc {

	/// <summary> This class is used for log operations on Gsvn main log console </summary>
	public class Log {
	
		/// <summary> Enum type of log level </summary>
		private enum logLevel : int {
		
			All = 1,
			Debug = 2,
			Information = 3,
			Warning = 4,
			Fatal = 5
			
		};			
	
		private logLevel level;
		
		private Gtk.TextView logConsole;
		
		private bool isDebugEnabled;
		private bool isInformationEnabled;
		private bool isWarningEnabled;
		private bool isFatalEnabled;
		
		/// <summary> Class Constructor </summary>
		public Log (uint level, Gtk.TextView LogConsole) {
		
			if ( (level < 1) | (level > 5) ) {
				this.level = (logLevel) 1;
			}
			else {
				this.level = (logLevel) level;
			}
					
			logConsole = LogConsole;
		
			this.isDebugEnabled = ((int)this.level == (int)level);
			this.isInformationEnabled = ((int)this.level == (int)level);
			this.isWarningEnabled = ((int)this.level == (int)level);
			this.isFatalEnabled = ((int)this.level == (int)level);
			
		}
		
		/// <summary> Return the date and time on string </summary>
		/// <returns> Return the date and time on string, formated.</summary>
		private string dateTime() {
		
			string datetime = "";
			DateTime date;
			//int day,month, year;
			//int hour, minute, second;
			
			date = DateTime.Now;
						
			datetime = string.Concat( date.Day.ToString(), "/" );
			datetime = string.Concat( datetime, date.Month.ToString() );
			datetime = string.Concat( datetime, "/" );
			datetime = string.Concat( datetime, date.Year.ToString() );
			datetime = string.Concat( datetime, "  " );
			datetime = string.Concat( datetime, date.Hour.ToString() );
			datetime = string.Concat( datetime, ":" );
			datetime = string.Concat( datetime, date.Minute.ToString("00") );
		
			return (datetime);
			
		}
		
		/// <summary> Log all messages. </summary>
		public void All(string message) {
												
			logConsole.Buffer.InsertAtCursor( string.Concat( dateTime(), " > ") );
			logConsole.Buffer.InsertAtCursor(message);
		
		}
		
		/// <summary> Log message if 'Debug' level is enabled. </summary>
		/// <param name="message"> Message to log </param>
		public void Debug(string message) {
		
			if (this.isDebugEnabled) {
				logConsole.Buffer.InsertAtCursor( string.Concat( dateTime(), " > ") );
				logConsole.Buffer.InsertAtCursor(message);
				logConsole.Buffer.InsertAtCursor("\r\n");
			}
		
		}
		
		/// <summary> Log message if 'Information' level is enabled. </summary>
		/// <param name="message"> Message to log </param>		
		public void information(string message) {
		
			if (this.isInformationEnabled) {
				logConsole.Buffer.InsertAtCursor( string.Concat( dateTime(), " > ") );			
				logConsole.Buffer.InsertAtCursor(message);
				logConsole.Buffer.InsertAtCursor("\r\n");
			}
		
		}
		
		/// <summary> Log message if 'Warning' level is enabled. </summary>
		/// <param name="message"> Message to log </param>		
		public void warning (string message) {
		
			if (this.isWarningEnabled) {
				logConsole.Buffer.InsertAtCursor( string.Concat( dateTime(), " > ") );			
				logConsole.Buffer.InsertAtCursor(message);
				logConsole.Buffer.InsertAtCursor("\r\n");				
			}
		
		}
		
		/// <summary> Log message if 'Fatal' level is enabled. </summary>
		/// <param name="message"> Message to log </param>		
		public void fatal (string message) {
		
			if (this.isFatalEnabled) {
				logConsole.Buffer.InsertAtCursor( string.Concat( dateTime(), " > ") );			
				logConsole.Buffer.InsertAtCursor(message);
				logConsole.Buffer.InsertAtCursor("\r\n");				
			}
		
		}
		
		/// <summary> Clear the log console. </summary>
		public void clearLog() {
		
			logConsole.Buffer.Clear();
			
		}	
	
	} // closing log class.
	
} // Closing pfc namespace.