#region

//        Copyright (C) 2004, 2005. Antonio Gutiérrez Mayoral
//                 <agutierr@gsyc.escet.urjc.es>
// 
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License as
//  published by the Free Software Foundation; either version 2 of the
//  License, or (at your option) any later version.
// 
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  General Public License for more details.
// 
//  You should have received a copy of the GNU General Public
//  License along with this program; if not, write to the
//  Free Software Foundation, Inc., 59 Temple Place - Suite 330,
//  Boston, MA 02111-1307, USA.

#endregion

using Gtk;
using Glade;
using System;

/// <summary> On this file we define some useful class over Gsvn, like the about class window
/// or the help window.
/// <summary>
namespace pfc {

	/// <summary> Class to use the about window on gsvn. </summary>
	public class AboutWindow {
	
		[Widget] Window	aboutWindow;
		
		public AboutWindow() {
			
			Glade.XML gladeXML = new Glade.XML ("gui.glade","aboutWindow");
       		gladeXML.Autoconnect(this);
       		aboutWindow.Hide();
		
		}
		
		/// <summary>
		///	This method shows the About window to user.
		/// </summary>	
		public void Show() {
	
			aboutWindow.Show();
			
		}
		
		/// <summary>	Method asociated with glade graphical interface.
		///				This method is invoked when user clicks on 'Ok' Button on 'AboutWindow'
		/// </summary>			
	    public void OnAboutOkButtonClicked(object o, EventArgs args)
	
		{
			//if (isDebugEnabled) {
			//	log.Debug("User click 'Ok' button on about window");
			//	log.Debug("Closing about window...");
			//}
			aboutWindow.Hide();
			aboutWindow.Dispose();
			
		}		
		
		/// <summary>
		/// This method is called when user wants to close about window.
		/// </summary>
		public void OnCloseWindow(object o, EventArgs args) {
		
			aboutWindow.Hide();
			aboutWindow.Dispose();
		
		}
		
	} // Closing about window class
	
	/// <summary> Class to use the Help window on gsvn. </summary>
	public class HelpWindow {
	
		[Widget] Window helpWindow;
		[Widget] ScrolledWindow helpWindow_topicsScrolledWindow;
		//[Widget] ScrolledWindow helpWindow_contentScrolledWindow;
		
		private TreeView HelpTopics;
		
		private void createTopics() {
		
			TreeStore tree = null;	

			tree = new TreeStore( typeof(string), typeof(string) );
			
			TreeIter iter;
			
			iter = tree.AppendValues( "Inicio Rápido" , "");
				tree.AppendValues(iter, "Añadiendo una copia de trabajo");		
				tree.AppendValues(iter, "Bajando una copia del repositorio");
				tree.AppendValues(iter, "Importando un proyecto en el servidor");			
			iter = tree.AppendValues( "Ciclo básico de trabajo", "");
				tree.AppendValues(iter, "Actualizar la copia local");
				tree.AppendValues(iter, "Realizar cambios a los ficheros locales");
				tree.AppendValues(iter, "Examinar los cambios");
				tree.AppendValues(iter, "Reflejar los cambios en el servidor");
				tree.AppendValues(iter, "Exportar una copia del directorio de trabajo");
			iter = tree.AppendValues( "Examinando el historial", "" );
				tree.AppendValues(iter, "Consultar el fichero de log");
				tree.AppendValues(iter, "Consultar diferencias entre ficheros");
				tree.AppendValues(iter, "Visualizar y editar ficheros");
			iter = tree.AppendValues( "Copiando y moviendo", "" );
				tree.AppendValues(iter, "Copiar elementos");
				tree.AppendValues(iter, "Mover elementos");
			iter = tree.AppendValues( "Otras herramientas", "" );
				tree.AppendValues(iter, "El editor de propiedades");
				tree.AppendValues(iter, "Limpiar los bloqueos");	
				tree.AppendValues(iter, "Terminal de comandos");
				tree.AppendValues(iter, "Crear un directorio");
				tree.AppendValues(iter, "Renombrar");
			
			HelpTopics = new TreeView (tree);
			
			HelpTopics.HeadersVisible = false;
			HelpTopics.RulesHint = true;	
			
			HelpTopics.AppendColumn("", new CellRendererText (), "text", 0);
																
			
		}	
		
		/// <summary> HelpWindow Class Constructor. </summary>
		public HelpWindow() {

			Glade.XML gladeXML = new Glade.XML ("gui.glade","helpWindow");
			gladeXML.Autoconnect(this);
			
		}		
		
		/// <summary>
		///	This method shows the Help window to user.
		/// </summary>			
		public void Show() {
		
			createTopics();
			helpWindow_topicsScrolledWindow.Add( HelpTopics );
			helpWindow_topicsScrolledWindow.ShowAll();
			helpWindow.Show();
			
		}
		
		/// <summary>
		/// This method is called when user wants to help about window.
		/// </summary>		
		public void OnCloseClicked (object o, EventArgs args) {
		
			helpWindow.Hide();
			helpWindow.Dispose();			
			
		}
		
	} // Closing Help Window		
	
} // closing pfc namespace.