#region

//        Copyright (C) 2004, 2005. Antonio Gutiérrez Mayoral
//                 <agutierr@gsyc.escet.urjc.es>
// 
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License as
//  published by the Free Software Foundation; either version 2 of the
//  License, or (at your option) any later version.
// 
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  General Public License for more details.
// 
//  You should have received a copy of the GNU General Public
//  License along with this program; if not, write to the
//  Free Software Foundation, Inc., 59 Temple Place - Suite 330,
//  Boston, MA 02111-1307, USA.

#endregion

/// <summary>
/// This class defines the main library over Subversion.
/// </summary>

namespace pfc.Svn {

	using System.Diagnostics;
	using System.Collections;
	using System.Collections.Specialized;
	using System.Threading;
						
								
	/// <summary> The Working Copy class. This class is used for all methods that its not neccesary to acces
	/// to repository. 
	/// </summary>
	public class WorkingCopy {
	
		// private Config config = new Config();
		
		private SvnConfig svnconfig = new SvnConfig();
		
		private static string space = " ";
	
		public string workingdirectory;
		
		public WorkingCopy(string WC) {
		
			this.workingdirectory = WC;
			
		}
		
		/// <summary> This method return true if dir is a working directory (is versioned) </summary> 
		/// <returns> Return True if DIR is a versioned resource dir </returns>
		public static bool is_Working_Copy(string DIR) {
			
			Process pcs = new Process();
			StringDictionary enviroment;
			
			enviroment = new StringDictionary();
			enviroment.Add("LC_ALL", "C");
			
			if (System.IO.Directory.Exists(DIR) == false) return false;
			
			try {
				pcs.StartInfo.FileName = "/usr/bin/svn";
			}
			catch {
				return false;
			}
			

			pcs.StartInfo.Arguments = SvnSubcommands.svnInfo;
			pcs.StartInfo.Arguments += space;			
			pcs.StartInfo.Arguments += DIR;		

			pcs.StartInfo.EnvironmentVariables.Add("LC_ALL","C");
			pcs.StartInfo.UseShellExecute = false;
			pcs.StartInfo.RedirectStandardOutput = true;
			pcs.StartInfo.RedirectStandardInput = true;
			pcs.StartInfo.CreateNoWindow = true;
			pcs.Start();
			pcs.WaitForExit();
			string tmpstr = pcs.StandardOutput.ReadToEnd();
			
			System.Console.WriteLine(tmpstr);		
			
			int _exit = pcs.ExitCode;
			
			pcs.Close();			
			
			return (_exit == 0);					
		}
		
		/// <summary> This method is used for consult info about an element. </summary>
		/// <returns> Returns an info element for PATH </returns>
		public InfoList info(string PATH) {
		
			InfoList info = new InfoList();		

			Process pcs = new Process();

			pcs.StartInfo.FileName = svnconfig.getSvnExec();		

			pcs.StartInfo.Arguments = SvnSubcommands.svnInfo;
			pcs.StartInfo.Arguments += " ";			
			pcs.StartInfo.Arguments += PATH;		

			pcs.StartInfo.EnvironmentVariables.Add("LC_ALL","C") ;
			pcs.StartInfo.UseShellExecute = false;
			pcs.StartInfo.RedirectStandardOutput = true;
			pcs.StartInfo.RedirectStandardInput = true;
			pcs.StartInfo.RedirectStandardError = true;
			pcs.StartInfo.CreateNoWindow = true;
			pcs.Start();
			pcs.WaitForExit();
			
			string info_out = pcs.StandardOutput.ReadToEnd();	
			string info_error = pcs.StandardError.ReadToEnd();		
		
			if ( (info_out.IndexOf("Not a versioned resource") == -1) && (info_error.IndexOf("is not a working copy") == -1) ) {
			
				info.setInfoElement("Status","versioned");

				string[] elements;
				string eof = "\n";
				string eof_ = "\n";
				
				elements = info_out.Split(eof.ToCharArray(), 100);
				
				foreach (string e in elements) {

					try {
						info.setInfoElement( e.Substring(0, e.IndexOf(':')),
											 e.Substring(e.IndexOf(':')+2, e.Length-e.IndexOf(':')-2) );
					}
					catch	{
						;
					}
						
				}					
			
			}
			
			else { // not a versioned element. All info elements will be set null.
				info.setInfoElement("Status","unversioned");
			}				
			
			pcs.WaitForExit();			
			
			return info;
		
		} // Closing InfoList Method
		
		/// <summary> Return a property Value </summary>
		/// <param name="PATH"> Element path </param>
		/// <param name="NAME"> Name of property </param>
		/// <returns> Return the property value with name NAME on PATH</returns>
		public string propGet(string PATH, string NAME) {
		
			string _value = "";
			string eof ="\n";
			string space = " ";

			Process pcs = new Process();

			pcs.StartInfo.FileName = svnconfig.getSvnExec();
			pcs.StartInfo.Arguments += SvnSubcommands.svnPropget;
			pcs.StartInfo.Arguments += space;
			pcs.StartInfo.Arguments += NAME.Replace(eof,"");	
			pcs.StartInfo.Arguments += space;
			pcs.StartInfo.Arguments += PATH.Replace(eof,"");	

			pcs.StartInfo.EnvironmentVariables.Add("LC_ALL","C") ;
			pcs.StartInfo.UseShellExecute = false;
			pcs.StartInfo.RedirectStandardOutput = true;
			pcs.StartInfo.RedirectStandardInput = true;
			pcs.StartInfo.CreateNoWindow = true;
			pcs.Start();	
				
			string info_out = pcs.StandardOutput.ReadToEnd();pcs.WaitForExit();

			return info_out.Replace(eof,"");
						
		} // Closing PropGet Method
		
		/// <summary> Set the value of a property </summary>
		/// <param name="PATH"> Element path </param>
		/// <param name="NAME"> Name of property </param>
		/// <param name="VALUE"> Value of property </param>
		public void propSet(string PATH, string NAME, string VALUE) {
		
			string eof ="\n";
			string space = " ";
			
			Process pcs = new Process();

			pcs.StartInfo.FileName = svnconfig.getSvnExec();
			pcs.StartInfo.Arguments += SvnSubcommands.svnPropset;
			pcs.StartInfo.Arguments += space;
			pcs.StartInfo.Arguments += NAME.Replace(eof,"");	
			pcs.StartInfo.Arguments += space;
			pcs.StartInfo.Arguments += "'" + VALUE + "'";
			pcs.StartInfo.Arguments += space;
			pcs.StartInfo.Arguments += PATH.Replace(eof,"");	

			pcs.StartInfo.EnvironmentVariables.Add("LC_ALL","C") ;
			pcs.StartInfo.UseShellExecute = false;
			pcs.StartInfo.RedirectStandardOutput = true;
			pcs.StartInfo.RedirectStandardInput = true;
			pcs.StartInfo.CreateNoWindow = true;
			pcs.Start();	
			
			pcs.WaitForExit();
		
		} // Closing propSet Method
		
		/// <summary> Edit the value of a property </summary>
		/// <param name="PATH"> Element path </param>
		/// <param name="NAME"> Name of property </param>
		/// <param name="VALUE"> Value of property </param>		
		public void propEdit(string PATH, string NAME, string VALUE) {
		
			string eof ="\n";
			string space = " ";			

			Process pcs = new Process();

			pcs.StartInfo.FileName = svnconfig.getSvnExec();
			pcs.StartInfo.Arguments += SvnSubcommands.svnPropedit;
			pcs.StartInfo.Arguments += space;
			pcs.StartInfo.Arguments += NAME.Replace(eof,"");	
			pcs.StartInfo.Arguments += space;
			pcs.StartInfo.Arguments += "'" + VALUE + "'";
			pcs.StartInfo.Arguments += space;
			pcs.StartInfo.Arguments += PATH.Replace(eof,"");	

			pcs.StartInfo.EnvironmentVariables.Add("LC_ALL","C") ;
			pcs.StartInfo.UseShellExecute = false;
			pcs.StartInfo.RedirectStandardOutput = true;
			pcs.StartInfo.RedirectStandardInput = true;
			pcs.StartInfo.CreateNoWindow = true;
			pcs.Start();	
			
			pcs.WaitForExit();
		
		} // Closing propEdit Method.
		
		/// <summary> Delete a property </summary>
		/// <param name="PATH"> Element path </param>
		/// <param name="NAME"> Name of property to delete </param>
		public void propDel(string PATH, string NAME) {
		
			string eof ="\n";
			string space = " ";
			
			// ejecuto el comando y recojo la salida.
			Process pcs = new Process();
			// COMANDO
			pcs.StartInfo.FileName = svnconfig.getSvnExec();
			pcs.StartInfo.Arguments += SvnSubcommands.svnPropdel;
			pcs.StartInfo.Arguments += space;
			pcs.StartInfo.Arguments += NAME;
			pcs.StartInfo.Arguments += space;
			pcs.StartInfo.Arguments += PATH.Replace(eof,"");	
			// options
			pcs.StartInfo.EnvironmentVariables.Add("LC_ALL","C") ;
			pcs.StartInfo.UseShellExecute = false;
			pcs.StartInfo.RedirectStandardOutput = true;
			pcs.StartInfo.RedirectStandardInput = true;
			pcs.StartInfo.CreateNoWindow = true;
			pcs.Start();	
			
			pcs.WaitForExit();
		
		} // Closing propDel Method.
		
		/// <summary> Obtains the list of properties over a file </summary>
		/// <param name="PATH"> Element path </param>
		/// <returns> The list of properties of PATH </returns>
		public PropList propList(string PATH) {
		
			PropList proplist = new PropList();
			
			Process pcs = new Process();

			pcs.StartInfo.FileName = svnconfig.getSvnExec();
			pcs.StartInfo.Arguments += SvnSubcommands.svnProplist;
			pcs.StartInfo.Arguments += space;
			pcs.StartInfo.Arguments += PATH;		

			pcs.StartInfo.EnvironmentVariables.Add("LC_ALL","C") ;
			pcs.StartInfo.UseShellExecute = false;
			pcs.StartInfo.RedirectStandardOutput = true;
			pcs.StartInfo.RedirectStandardInput = true;
			pcs.StartInfo.CreateNoWindow = true;
			pcs.Start();								
			pcs.WaitForExit();
			string info_out = pcs.StandardOutput.ReadToEnd();
			

			string eof = "\n";
			string[] elements = info_out.Split(eof.ToCharArray(), 1000);			
			
			if (elements.Length != 0) {
				for (int i=1; i<elements.Length-1;i++) {
					string _value = propGet(PATH, elements[i]);
					proplist.insertProperty(elements[i].Substring(2, elements[i].Length -2), _value);
					System.Console.WriteLine(elements[i] + _value);
				}
			}
			return proplist;		
		
		} // Closing PropList Method.
		
		/// <summary> Check if an element is versioned. (is under version system) </summary>
		/// <param name="PATH"> Element path </param>
		/// <returns> True if element PATH is versioned, FALSE in other case.
		public bool isVersioned(string PATH) {
		
			Process pcs = new Process();

			pcs.StartInfo.FileName = svnconfig.getSvnExec();
			
			pcs.StartInfo.Arguments = SvnSubcommands.svnInfo;
			pcs.StartInfo.Arguments += " ";			
			pcs.StartInfo.Arguments += PATH;		

			pcs.StartInfo.EnvironmentVariables.Add("LC_ALL","C") ;
			pcs.StartInfo.UseShellExecute = false;
			pcs.StartInfo.RedirectStandardOutput = true;
			pcs.StartInfo.RedirectStandardInput = true;
			pcs.StartInfo.CreateNoWindow = true;
			pcs.Start();
			pcs.WaitForExit();
			
			string info_out = pcs.StandardOutput.ReadToEnd();
			
			return !(info_out.IndexOf("Not a versioned resource") > 0);
			
		} // Closing isVersioned Method.
		
		/// <summary> Do revert operation over an element. </summary>
		/// <param name="PATH"> Element path </param>
		public void revert(string PATH) {
		
			Process pcs = new Process();
			pcs.StartInfo.FileName = svnconfig.getSvnExec();
		
			pcs.StartInfo.Arguments = SvnSubcommands.svnRevert;
			pcs.StartInfo.Arguments += " ";			
			pcs.StartInfo.Arguments += PATH;		

			pcs.StartInfo.EnvironmentVariables.Add("LC_ALL","C") ;
			pcs.StartInfo.UseShellExecute = false;
			pcs.StartInfo.RedirectStandardOutput = true;
			pcs.StartInfo.RedirectStandardInput = true;
			pcs.StartInfo.CreateNoWindow = true;
			pcs.Start();
			pcs.WaitForExit();
			
		}  // Closing Revert Method.
		
		/// <summary> Mark a file for deletion. </summary>
		/// <param name="PATH"> Element path to mark for deletion. </param>
		public void delete(string PATH) {
		

			Process pcs = new Process();

			pcs.StartInfo.FileName = svnconfig.getSvnExec();
			
			// arguments
			pcs.StartInfo.Arguments = SvnSubcommands.svnDelete;
			pcs.StartInfo.Arguments += space;		
			pcs.StartInfo.Arguments += PATH;		
						
			System.Console.WriteLine( pcs.StartInfo.Arguments );
			
			pcs.StartInfo.EnvironmentVariables.Add("LC_ALL","C") ;
			pcs.StartInfo.UseShellExecute = false;
			pcs.StartInfo.RedirectStandardOutput = true;
			pcs.StartInfo.RedirectStandardInput = true;
			pcs.StartInfo.CreateNoWindow = true;
			pcs.Start();
			pcs.WaitForExit();		
			
		}
		
		/// <summary> Mark a file for adition. </summary>
		/// <param name="PATH"> Element path to mark for adition. </param>		
		public void _add(string PATH, bool recursive) {
		
			Process pcs = new Process();
			pcs.StartInfo.FileName = svnconfig.getSvnExec();
			
			pcs.StartInfo.Arguments = SvnSubcommands.svnAdd;
			pcs.StartInfo.Arguments += space;					
			if (!recursive) {
				pcs.StartInfo.Arguments += space;
				pcs.StartInfo.Arguments += SvnSwitches.non_recursive;
			}		
			pcs.StartInfo.Arguments += space;
			pcs.StartInfo.Arguments += PATH;		
			
			System.Console.WriteLine( pcs.StartInfo.Arguments );
			
			pcs.StartInfo.EnvironmentVariables.Add("LC_ALL","C") ;
			pcs.StartInfo.UseShellExecute = false;
			pcs.StartInfo.RedirectStandardOutput = true;
			pcs.StartInfo.RedirectStandardInput = true;
			pcs.StartInfo.CreateNoWindow = true;
			pcs.Start();
			pcs.WaitForExit();
			
			string info_out = pcs.StandardOutput.ReadToEnd();
			
			System.Console.WriteLine(info_out);
			
		} // Closing Add Method.
			
		/// <summary> Create a new directory and mark it for adition. </summary>
		/// <param name="PATH"> Path to new Directory. </param>
		/// <param name="NAME"> Name of new Directory. </param>
		public void makeDirectory(string PATH, string NAME) {
		
			Process pcs = new Process();
			pcs.StartInfo.FileName = svnconfig.getSvnExec();
			
			pcs.StartInfo.Arguments = SvnSubcommands.svnMkdir;
			pcs.StartInfo.Arguments += space;					
			pcs.StartInfo.Arguments += "'" + PATH + "/" + NAME + "'";		
			
			System.Console.WriteLine( pcs.StartInfo.Arguments );
			
			pcs.StartInfo.EnvironmentVariables.Add("LC_ALL","C") ;
			pcs.StartInfo.UseShellExecute = false;
			pcs.StartInfo.RedirectStandardOutput = true;
			pcs.StartInfo.RedirectStandardInput = true;
			pcs.StartInfo.CreateNoWindow = true;
			pcs.Start();
			pcs.WaitForExit();
			
		}
		
		/// <summary> Obtain Info about an element. </summary>
		/// <param name="PATH"> Path to obtain Info. </param>
		public string infoRaw(string PATH) {
		
			Process pcs = new Process();
			pcs.StartInfo.FileName = svnconfig.getSvnExec();
			
			pcs.StartInfo.Arguments = SvnSubcommands.svnInfo;
			pcs.StartInfo.Arguments += space;					
			pcs.StartInfo.Arguments += PATH;		
			
			pcs.StartInfo.EnvironmentVariables.Add("LC_ALL","C") ;
			pcs.StartInfo.UseShellExecute = false;
			pcs.StartInfo.RedirectStandardOutput = true;
			pcs.StartInfo.RedirectStandardInput = true;
			pcs.StartInfo.CreateNoWindow = true;
			pcs.Start();
			pcs.WaitForExit();
			
			string info_out = pcs.StandardOutput.ReadToEnd();
			
			return info_out;
			
		}	
		
		/// <summary> Cleanup a directory. </summary>
		/// <param name="PATH"> Path to cleanup. </param>
		public void cleanup(string PATH) {		

			Process pcs = new Process();

			pcs.StartInfo.FileName = svnconfig.getSvnExec();
			

			pcs.StartInfo.Arguments = SvnSubcommands.svnCleanup;
			pcs.StartInfo.Arguments += space;					
			pcs.StartInfo.Arguments += "'" + PATH + "'" ;		
			
			pcs.StartInfo.EnvironmentVariables.Add("LC_ALL","C") ;
			pcs.StartInfo.UseShellExecute = false;
			pcs.StartInfo.RedirectStandardOutput = true;
			pcs.StartInfo.RedirectStandardInput = true;
			pcs.StartInfo.CreateNoWindow = true;
			pcs.Start();
			pcs.WaitForExit();		
		
		}
		
		/// <summary> Return true if file is a text file. </summary>
		/// <param name="PATH"> Path to check if file is a text file. </param>
		public static bool isTextFile(string PATH) {	

			Process pcs = new Process();
			
			SvnConfig svnconfig = new SvnConfig();

			pcs.StartInfo.FileName = svnconfig.fileUtil;

			pcs.StartInfo.Arguments += space;					
			pcs.StartInfo.Arguments += PATH;		
			
			pcs.StartInfo.EnvironmentVariables.Add("LC_ALL","C") ;
			pcs.StartInfo.UseShellExecute = false;
			pcs.StartInfo.RedirectStandardOutput = true;
			pcs.StartInfo.RedirectStandardInput = true;
			pcs.StartInfo.CreateNoWindow = true;
			pcs.Start();
			pcs.WaitForExit();	
			
			string output = pcs.StandardOutput.ReadToEnd();
			
			return (output.IndexOf("text") != -1);
			
		}		
		
		/// <summary> Return the content of a file. </summary>
		/// <param name="PATH"> Path to file. </param>
		/// <returns> The content of the file. </returns>		
		public string cat (string PATH) {
		
			Process pcs = new Process();
			
			pcs.StartInfo.FileName = "/bin/cat";

			pcs.StartInfo.Arguments += space;					
			pcs.StartInfo.Arguments += PATH;		
			
			System.Console.WriteLine( pcs.StartInfo.Arguments );
			
			pcs.StartInfo.EnvironmentVariables.Add("LC_ALL","C") ;
			pcs.StartInfo.UseShellExecute = false;
			pcs.StartInfo.RedirectStandardOutput = true;
			pcs.StartInfo.RedirectStandardInput = true;
			pcs.StartInfo.CreateNoWindow = true;
			pcs.Start();
			pcs.WaitForExit();	
			
			pcs.Start();
			pcs.WaitForExit();
			
			return pcs.StandardOutput.ReadToEnd();			
			
		}	
		
		/// <summary> Edit file with an external editor. </summary>
		/// <param name="PATH"> Path to file. </param>		
		public void edit(string PATH) {
		

			Process pcs = new Process();
			
			SvnConfig svnconfig = new SvnConfig();
			
			pcs.StartInfo.FileName = svnconfig.editApp;

			pcs.StartInfo.Arguments += space;					
			pcs.StartInfo.Arguments += PATH;		
			
			System.Console.WriteLine( pcs.StartInfo.Arguments );
			
			pcs.StartInfo.UseShellExecute = false;
			pcs.StartInfo.RedirectStandardOutput = false;
			pcs.StartInfo.RedirectStandardInput = false;
			pcs.StartInfo.CreateNoWindow = false;

			pcs.Start();
			
			pcs.WaitForExit();	
			
		}		
		
	} // clossing Working Copy class.				
		
	
	/// <summary> This class implements usual client subcommands over subversion.
	/// </summary>
	public class SvnClient {
	
		// <summary> working directory of subversion </summary>
		public string wc;
			
		// <summary> subversion url </summary>
		public string svnurl; 
				
		// <summary> username in subversion </summary>
		public string username; 
		
		// <summary> password in subversion </summary>
		public string password; 
		
		public string remote_dir;		
		
		private WorkingCopy workingCopy;
		private SvnConfig svnconfig;
		
		public string tmpstr = "";
		
		private CommandOutputWindow commandOutput;
		
		private static Gtk.ThreadNotify notify;
		private static Gtk.ThreadNotify notify2;
		private static Gtk.ThreadNotify notify3;
		private static Gtk.ThreadNotify notify4;
		
		private string[] _params;
		
		private gsvn app;

		/// <summary> Class constructor. </summary>
		/// <param name="WORKINGCOPY"> Working Directory </param>
		/// <param name="URL"> Subversion url </param>
		/// <param name="REMOTE_DIR"> Remote dir on repository </param>
		/// <param name="_gsvn"> Main app </param>
		public SvnClient(string WORKINGCOPY,
						 string URL,
						 string REMOTE_DIR,
						 gsvn _app) {
		
			this.wc = WORKINGCOPY;
			this.svnurl = URL;
			this.remote_dir = REMOTE_DIR;
			
			workingCopy = new WorkingCopy(WORKINGCOPY);
			svnconfig = new SvnConfig();
			
			app = _app;
			
		}
		
		/// <summary> Class constructor. </summary>
		public SvnClient(WorkingCopy _workingCopy) {
		
			this.workingCopy = _workingCopy;
			
		}
		
		/// <summary> Class constructor. </summary>
		/// <param name="WORKINGCOPY"> Working Directory </param>
		/// <param name="URL"> Subversion url </param>
		/// <param name="USER"> User loginname on subversion <param>
		/// <param name="PASSWORD"> Password on subversion </param>
		public SvnClient(	string WORKINGCOPY,
							string URL,
							string USERNAME,
							string PASSWORD) {
							
			this.svnurl = URL;
			this.username = USERNAME;
			this.password = PASSWORD;
			
		}
		
		/// <summary> Class constructor. </summary>
		public SvnClient() {
		
			svnconfig = new SvnConfig();
			
		}
		
		/// <summary> Set working Directory. </summar>
		
		
		public void Nothing() {
		
			;
			
		}
		
		private void addworkingDir() {
		
			app.createTreeCopy(this.wc);
			
		}
		
		public void execSubcommand(string COMMAND,
								   string[] __params) {
								   
			// first, create output command window
			commandOutput = new CommandOutputWindow();
			commandOutput.Show();
			// later, we add the notify threads objects.
			
			_params = __params;
			
			Thread t;
			
			notify  = new Gtk.ThreadNotify (new Gtk.ReadyEvent (commandOutput.refreshCommandBuffer) );
			notify2 = new Gtk.ThreadNotify (new Gtk.ReadyEvent (commandOutput.finalizeCommandOutput) );	
			notify3 = new Gtk.ThreadNotify (new Gtk.ReadyEvent (commandOutput.refreshErrorBuffer) );	
			notify4 = new Gtk.ThreadNotify (new Gtk.ReadyEvent (this.addworkingDir) );				
			
			switch (COMMAND) {		
			
				case "CHECKOUT":	t = new Thread ( new ThreadStart( this.checkout ) ) ;
									t.Start();
									break;
									
				case "IMPORT": 		t = new Thread ( new ThreadStart( this.import ) ) ;
									t.Start();
									break;
									
				case "EXPORT":		t = new Thread ( new ThreadStart( this.export) ) ;
									t.Start();
									break;
									
				case "COMMIT": 		t = new Thread ( new ThreadStart( this.commit) ) ;
									t.Start();
									break;
						
				case "UPDATE": 		t = new Thread ( new ThreadStart( this.update) ) ;
									t.Start();
									break;
									
				case "SWITCH":		t = new Thread ( new ThreadStart( this._switch) ) ;
									t.Start();
									break;	
									
				case "MOVE_REMOTE":	t = new Thread ( new ThreadStart( this.move_remote) ) ;
									t.Start();
									break;		
			
				case "COPY_REMOTE":	t = new Thread ( new ThreadStart( this.copy) ) ;
									t.Start();
									break;
									
				case "UPDATE_ENTRY":t = new Thread ( new ThreadStart( this.update_entry) ) ;
									t.Start();
									break;
									
				case "COMMIT_ENTRY":t = new Thread ( new ThreadStart( this.commit_entry) ) ;
									t.Start();
									break;
									
				case "CAT":			t = new Thread ( new ThreadStart( this.cat ) ) ;
									t.Start();
									break;
									
				case "BLAME":		t = new Thread ( new ThreadStart( this.blame ) ) ;
									t.Start();
									break;	
									
				case "LOG":			t = new Thread ( new ThreadStart( this.log ) ) ;
									t.Start();
									break;		
									
				case "CREATE":		t = new Thread ( new ThreadStart( this.create ) ) ;
									t.Start();
									break;																															
			
			}				
			
		}	
		
		/// <summary> Obtains a fresh copy from repository </summary>
		public void	checkout() {
								
			Gtk.TextBuffer buffer;
			string buf1, buf2;
			string tmpstr = "";
			string space = " ";
			string EOF = "\n";
			
			Process p = new Process();

			p.StartInfo.FileName = svnconfig.getSvnExec();
			p.StartInfo.Arguments += SvnSubcommands.svnCheckout;
			p.StartInfo.EnvironmentVariables.Add("LC_ALL","C");
			
			System.Console.WriteLine(this.svnurl.IndexOf("svn+ssh"));
			
			if ( (this.svnurl.IndexOf("svn+ssh") == 0) && (this._params[2] != "") )
				// insert username 
				this.svnurl = this.svnurl.Replace("svn+ssh://", "svn+ssh://"+_params[2]+"@");
			
			p.StartInfo.Arguments += space + this.svnurl;
			p.StartInfo.Arguments += this.remote_dir;
			
			p.StartInfo.Arguments += space + this.wc;
			
			if (this._params[0] == "False") {
				p.StartInfo.Arguments += space + SvnSwitches.revision;
				p.StartInfo.Arguments += space + _params[1];
			}
			
			System.Console.WriteLine( p.StartInfo.Arguments.ToString() );
			
			p.StartInfo.UseShellExecute = false;
			p.StartInfo.RedirectStandardError = true;
			p.StartInfo.RedirectStandardInput = true;
			p.StartInfo.RedirectStandardOutput = true;
			p.StartInfo.CreateNoWindow = true;
			
			p.Start();
				
			while ( ((buf1 = p.StandardOutput.ReadLine() ) != null) || ((buf2 = p.StandardError.ReadLine() ) != null ) ) {

				if (buf1 != null) {
					System.Console.WriteLine(buf1);	
					commandOutput.setCommandMessage(buf1);
					notify.WakeupMain();				
				}				
				if (buf2 != null) {
					commandOutput.setErrorMessage(buf2);
					notify3.WakeupMain();
				}
				
				notify.WakeupMain();
				p.Refresh();
				
			}
			
			p.WaitForExit();
			
			notify2.WakeupMain();
			
			if ( p.ExitCode == 0) 		
				notify4.WakeupMain();		
			
					
		}
		
		/// <summary> Send changes from your working copy to the repository </summary>
		public void commit () {
		
			string buf1, buf2;
			string tmpstr = "";
			string space = " ";
			string EOF = "\n";		
		
			Process p = new Process();

			p.StartInfo.FileName = svnconfig.getSvnExec();
			p.StartInfo.Arguments += SvnSubcommands.svnCommit + space;
			p.StartInfo.EnvironmentVariables.Add("LC_ALL","C");					
			
			_params[1] = _params[1].Replace("\n"," ");
			
			p.StartInfo.Arguments += _params[0] + space;
			p.StartInfo.Arguments += SvnSwitches.message + space + "'" + _params[1].Replace(EOF, space) + "'" + space;
			
			System.Console.WriteLine( p.StartInfo.Arguments );
			
			p.StartInfo.UseShellExecute = false;
			p.StartInfo.RedirectStandardError = true;
			p.StartInfo.RedirectStandardInput = true;
			p.StartInfo.RedirectStandardOutput = true;
			p.StartInfo.CreateNoWindow = true;			

			p.Start();
			
			while ( ((buf1 = p.StandardOutput.ReadLine() ) != null) || ((buf2 = p.StandardError.ReadLine() ) != null ) ) {

				if (buf1 != null) {
					System.Console.WriteLine(buf1);	
					commandOutput.setCommandMessage(buf1);
					notify.WakeupMain();				
				}				
				if (buf2 != null) {
					commandOutput.setErrorMessage(buf2);
					notify3.WakeupMain();
				}
				
				notify.WakeupMain();
				p.Refresh();
				
			}
			
			p.WaitForExit();			
			
			notify2.WakeupMain();	
			
		}
		
		/// <summary> Syncronize working copy with repository </summary>
		public void update() {
		
			string buf1, buf2;
			string tmpstr = "";
			string space = " ";
			string EOF = "\n";		
		
			Process p = new Process();

			p.StartInfo.FileName = svnconfig.getSvnExec();
			p.StartInfo.Arguments += SvnSubcommands.svnUpdate + space;
			p.StartInfo.EnvironmentVariables.Add("LC_ALL","C");					
			
			p.StartInfo.Arguments += _params[0] + space;
			
			if (_params[1] == "False") 
				 p.StartInfo.Arguments += SvnSwitches.revision + space + _params[2];
			
			System.Console.WriteLine( p.StartInfo.Arguments );
			
			p.StartInfo.UseShellExecute = false;
			p.StartInfo.RedirectStandardError = true;
			p.StartInfo.RedirectStandardInput = true;
			p.StartInfo.RedirectStandardOutput = true;
			p.StartInfo.CreateNoWindow = true;			

			p.Start();
			
			while ( ((buf1 = p.StandardOutput.ReadLine() ) != null) || ((buf2 = p.StandardError.ReadLine() ) != null ) ) {

				if (buf1 != null) {
					System.Console.WriteLine(buf1);	
					commandOutput.setCommandMessage(buf1);
					notify.WakeupMain();				
				}				
				if (buf2 != null) {
					commandOutput.setErrorMessage(buf2);
					notify3.WakeupMain();
				}
				
				notify.WakeupMain();
				p.Refresh();
				
			}
			
			p.WaitForExit();			
			
			notify2.WakeupMain();				
		
		}
		
		/// <summary> Sycronize an item with repository </summary>
		public void update_entry() {

			string buf1, buf2;
			string tmpstr = "";
			string space = " ";
			string EOF = "\n";		
		
			Process p = new Process();

			p.StartInfo.FileName = svnconfig.getSvnExec();
			p.StartInfo.Arguments += SvnSubcommands.svnUpdate + space;
			p.StartInfo.EnvironmentVariables.Add("LC_ALL","C");					
			
			p.StartInfo.Arguments += _params[0] + space;
			
			if (_params[1] == "False") 
				 p.StartInfo.Arguments += SvnSwitches.revision + space + _params[2];
			
			System.Console.WriteLine( p.StartInfo.Arguments );
			
			p.StartInfo.UseShellExecute = false;
			p.StartInfo.RedirectStandardError = true;
			p.StartInfo.RedirectStandardInput = true;
			p.StartInfo.RedirectStandardOutput = true;
			p.StartInfo.CreateNoWindow = true;			

			p.Start();
			
			while ( ((buf1 = p.StandardOutput.ReadLine() ) != null) || ((buf2 = p.StandardError.ReadLine() ) != null ) ) {

				if (buf1 != null) {
					System.Console.WriteLine(buf1);	
					commandOutput.setCommandMessage(buf1);
					notify.WakeupMain();				
				}				
				if (buf2 != null) {
					commandOutput.setErrorMessage(buf2);
					notify3.WakeupMain();
				}
				
				notify.WakeupMain();
				p.Refresh();
				
			}
			
			p.WaitForExit();			
			
			notify2.WakeupMain();				
		
		}

		/// <summary> Send item's changes to repository. </summary>
		public void commit_entry() {

			string buf1, buf2;
			string tmpstr = "";
			string space = " ";
			string EOF = "\n";		
		
			Process p = new Process();

			p.StartInfo.FileName = svnconfig.getSvnExec();
			p.StartInfo.Arguments += SvnSubcommands.svnCommit + space;
			p.StartInfo.EnvironmentVariables.Add("LC_ALL","C");					
			
			p.StartInfo.Arguments += _params[0] + space;
			
			p.StartInfo.Arguments += SvnSwitches.message + space + "'" + _params[1].Replace(EOF, space) +  "'";
			
			System.Console.WriteLine( p.StartInfo.Arguments );
			
			p.StartInfo.UseShellExecute = false;
			p.StartInfo.RedirectStandardError = true;
			p.StartInfo.RedirectStandardInput = true;
			p.StartInfo.RedirectStandardOutput = true;
			p.StartInfo.CreateNoWindow = true;			

			p.Start();
			
			while ( ((buf1 = p.StandardOutput.ReadLine() ) != null) || ((buf2 = p.StandardError.ReadLine() ) != null ) ) {

				if (buf1 != null) {
					System.Console.WriteLine(buf1);	
					commandOutput.setCommandMessage(buf1);
					notify.WakeupMain();				
				}				
				if (buf2 != null) {
					commandOutput.setErrorMessage(buf2);
					notify3.WakeupMain();
				}
				
				notify.WakeupMain();
				p.Refresh();
				
			}
			
			p.WaitForExit();			
			
			notify2.WakeupMain();				
		
		}		
				
		/// <summary> Move items on repository </summary>	
		public void move_remote() {
		
			string buf1, buf2;
			string tmpstr = "";
			string space = " ";
			string EOF = "\n";		
		
			Process p = new Process();

			p.StartInfo.FileName = svnconfig.getSvnExec();
			p.StartInfo.Arguments += SvnSubcommands.svnMove + space;
			p.StartInfo.EnvironmentVariables.Add("LC_ALL","C");					
			
			p.StartInfo.Arguments += _params[0] + space;
			
			p.StartInfo.Arguments += _params[1] + space;
			
			p.StartInfo.Arguments += SvnSwitches.message + space + "'" + _params[2].Replace("\n"," ") + "'" ;
			
			System.Console.WriteLine( p.StartInfo.Arguments );
			
			p.StartInfo.UseShellExecute = false;
			p.StartInfo.RedirectStandardError = true;
			p.StartInfo.RedirectStandardInput = true;
			p.StartInfo.RedirectStandardOutput = true;
			p.StartInfo.CreateNoWindow = true;			

			p.Start();
			
			while ( ((buf1 = p.StandardOutput.ReadLine() ) != null) || ((buf2 = p.StandardError.ReadLine() ) != null ) ) {

				if (buf1 != null) {
					System.Console.WriteLine(buf1);	
					commandOutput.setCommandMessage(buf1);
					notify.WakeupMain();				
				}				
				if (buf2 != null) {
					commandOutput.setErrorMessage(buf2);
					notify3.WakeupMain();
				}
				
				notify.WakeupMain();
				p.Refresh();
				
			}
			
			p.WaitForExit();			
			
			notify2.WakeupMain();					
			
		}
		
		/// <summary> Copy items on repository </summary>
		public void copy() {
		
			string buf1, buf2;
			string tmpstr = "";
			string space = " ";
			string EOF = "\n";		
		
			Process p = new Process();

			p.StartInfo.FileName = svnconfig.getSvnExec();
			p.StartInfo.Arguments += SvnSubcommands.svnCopy + space;
			p.StartInfo.EnvironmentVariables.Add("LC_ALL","C");					
			
			p.StartInfo.Arguments += _params[0] + space;
			
			p.StartInfo.Arguments += _params[1] + space;
			
			p.StartInfo.Arguments += SvnSwitches.message + space + "'" + _params[2].Replace("\n"," ") + "'" ;
			
			System.Console.WriteLine( p.StartInfo.Arguments );
			
			p.StartInfo.UseShellExecute = false;
			p.StartInfo.RedirectStandardError = true;
			p.StartInfo.RedirectStandardInput = true;
			p.StartInfo.RedirectStandardOutput = true;
			p.StartInfo.CreateNoWindow = true;			

			p.Start();
			
			while ( ((buf1 = p.StandardOutput.ReadLine() ) != null) || ((buf2 = p.StandardError.ReadLine() ) != null ) ) {

				if (buf1 != null) {
					System.Console.WriteLine(buf1);	
					commandOutput.setCommandMessage(buf1);
					notify.WakeupMain();				
				}				
				if (buf2 != null) {
					commandOutput.setErrorMessage(buf2);
					notify3.WakeupMain();
				}
				
				notify.WakeupMain();
				p.Refresh();
				
			}
			
			p.WaitForExit();			
			
			notify2.WakeupMain();						

		}
		
		/// <summary> Export a clean directory tree </summary>
		public void export() {
		
			string buf1, buf2;
			string tmpstr = "";
			string space = " ";
			string EOF = "\n";		
		
			Process p = new Process();

			p.StartInfo.FileName = svnconfig.getSvnExec();
			p.StartInfo.Arguments += SvnSubcommands.svnExport + space;
			p.StartInfo.EnvironmentVariables.Add("LC_ALL","C");		
			
			if (_params[0] == "False") { // Export from a directory: svn export PATH1 PATH2
			
				p.StartInfo.Arguments += _params[2] + space + _params[4];
			
			}
			
			else { // export from an URL: export -r rev URL PATH
			
				if (_params[3] != "") {
					p.StartInfo.Arguments += SvnSwitches.revision + space + _params[3];
				}
				
				p.StartInfo.Arguments += _params[1] + space + _params[4];
				
			}
			
			System.Console.WriteLine( p.StartInfo.Arguments);
			
			p.StartInfo.UseShellExecute = false;
			p.StartInfo.RedirectStandardError = true;
			p.StartInfo.RedirectStandardInput = true;
			p.StartInfo.RedirectStandardOutput = true;
			p.StartInfo.CreateNoWindow = true;
			
			p.Start();
			
			while ( ((buf1 = p.StandardOutput.ReadLine() ) != null) || ((buf2 = p.StandardError.ReadLine() ) != null ) ) {

				if (buf1 != null) {
					System.Console.WriteLine(buf1);	
					commandOutput.setCommandMessage(buf1);
					notify.WakeupMain();				
				}				
				if (buf2 != null) {
					commandOutput.setErrorMessage(buf2);
					notify3.WakeupMain();
				}
				
				notify.WakeupMain();
				p.Refresh();
				
			}
			
			p.WaitForExit();			
			
			notify2.WakeupMain();			
					
		}
		
		/// <summary> Switch a working copy with other Svn URL. </summary>
		public void _switch() {
		
			string buf1, buf2;
			string tmpstr = "";
			string space = " ";
			string EOF = "\n";		
		
			Process p = new Process();

			p.StartInfo.FileName = svnconfig.getSvnExec();
			p.StartInfo.Arguments += SvnSubcommands.svnSwitch + space;
			p.StartInfo.EnvironmentVariables.Add("LC_ALL","C");		
			
			p.StartInfo.Arguments += _params[0] + space + _params[1];			

			System.Console.WriteLine( p.StartInfo.Arguments);
			
			p.StartInfo.UseShellExecute = false;
			p.StartInfo.RedirectStandardError = true;
			p.StartInfo.RedirectStandardInput = true;
			p.StartInfo.RedirectStandardOutput = true;
			p.StartInfo.CreateNoWindow = true;
			
			p.Start();
			
			while ( ((buf1 = p.StandardOutput.ReadLine() ) != null) || ((buf2 = p.StandardError.ReadLine() ) != null ) ) {

				if (buf1 != null) {
					System.Console.WriteLine(buf1);	
					commandOutput.setCommandMessage(buf1);
					notify.WakeupMain();				
				}				
				if (buf2 != null) {
					commandOutput.setErrorMessage(buf2);
					notify3.WakeupMain();
				}
				
				notify.WakeupMain();
				p.Refresh();
				
			}
			
			p.WaitForExit();			
			
			notify2.WakeupMain();				
			
		}
		
		/// <summary> Import a projecto to server. </summary>
		public void import() {
		
			string buf1, buf2;
			string tmpstr = "";
			string space = " ";
			string EOF = "\n";
			
			Process p = new Process();

			p.StartInfo.FileName = svnconfig.getSvnExec();
			p.StartInfo.Arguments += SvnSubcommands.svnImport;
			p.StartInfo.EnvironmentVariables.Add("LC_ALL","C");		

			if ( (_params[0].IndexOf("svn+ssh") == 0) && (this._params[3] != "") )
				// insert username 
				_params[0] = _params[0].Replace("svn+ssh://", "svn+ssh://"+_params[4]+"@");			
			
			p.StartInfo.Arguments += space + this._params[1] ;
			p.StartInfo.Arguments += space + this._params[0] + this._params[2];	
			
			p.StartInfo.Arguments += space + SvnSwitches.message;
			p.StartInfo.Arguments += space + "'" + this._params[3] + "'";
			
			System.Console.WriteLine( p.StartInfo.Arguments );
			
			if ( _params[3] == "True") {
				p.StartInfo.Arguments += space + SvnSwitches.non_recursive;
			}			
			
			p.StartInfo.UseShellExecute = false;
			p.StartInfo.RedirectStandardError = true;
			p.StartInfo.RedirectStandardInput = true;
			p.StartInfo.RedirectStandardOutput = true;
			p.StartInfo.CreateNoWindow = true;
			
			p.Start();
					
			while ( ((buf1 = p.StandardOutput.ReadLine() ) != null) || ((buf2 = p.StandardError.ReadLine() ) != null ) ) {

				if (buf1 != null) {
					System.Console.WriteLine(buf1);	
					commandOutput.setCommandMessage(buf1);
					notify.WakeupMain();				
				}				
				if (buf2 != null) {
					commandOutput.setErrorMessage(buf2);
					notify3.WakeupMain();
				}
				
				notify.WakeupMain();
				p.Refresh();
				
			}
			
			p.WaitForExit();			
			
			notify2.WakeupMain();
			
		}
		
		/// <summary> Obtain the content of a file. </summary>		
		public void cat() {

			string buf1, buf2;
			string tmpstr = "";
			string space = " ";
			string EOF = "\n";		
		
			Process p = new Process();

			p.StartInfo.FileName = svnconfig.getSvnExec();
			p.StartInfo.Arguments += SvnSubcommands.svnCat + space;
			p.StartInfo.EnvironmentVariables.Add("LC_ALL","C");					
			
			p.StartInfo.Arguments += _params[0] + space;
			
			System.Console.WriteLine( p.StartInfo.Arguments );
			
			p.StartInfo.UseShellExecute = false;
			p.StartInfo.RedirectStandardError = true;
			p.StartInfo.RedirectStandardInput = true;
			p.StartInfo.RedirectStandardOutput = true;
			p.StartInfo.CreateNoWindow = true;			

			p.Start();
			
			while ( ((buf1 = p.StandardOutput.ReadLine() ) != null) || ((buf2 = p.StandardError.ReadLine() ) != null ) ) {

				if (buf1 != null) {
					System.Console.WriteLine(buf1);	
					commandOutput.setCommandMessage(buf1);
					notify.WakeupMain();				
				}				
				if (buf2 != null) {
					commandOutput.setErrorMessage(buf2);
					notify3.WakeupMain();
				}
				
				notify.WakeupMain();
				p.Refresh();
				
			}
			
			p.WaitForExit();			
			
			notify2.WakeupMain();				
		
		}
		
		/// <summary> Obtain authors and number of revisions inline. </summary>
		public void blame() {

			string buf1, buf2;
			string tmpstr = "";
			string space = " ";
			string EOF = "\n";		
		
			Process p = new Process();

			p.StartInfo.FileName = svnconfig.getSvnExec();
			p.StartInfo.Arguments += SvnSubcommands.svnBlame + space;
			
			p.StartInfo.EnvironmentVariables.Add("LC_ALL","C");					
			
			p.StartInfo.Arguments += _params[0] + space;
			
			System.Console.WriteLine( p.StartInfo.Arguments );
			
			p.StartInfo.UseShellExecute = false;
			p.StartInfo.RedirectStandardError = true;
			p.StartInfo.RedirectStandardInput = true;
			p.StartInfo.RedirectStandardOutput = true;
			p.StartInfo.CreateNoWindow = true;			

			p.Start();
			
			while ( ((buf1 = p.StandardOutput.ReadLine() ) != null) || ((buf2 = p.StandardError.ReadLine() ) != null ) ) {

				if (buf1 != null) {
					System.Console.WriteLine(buf1);	
					commandOutput.setCommandMessage(buf1);
					notify.WakeupMain();				
				}				
				if (buf2 != null) {
					commandOutput.setErrorMessage(buf2);
					notify3.WakeupMain();
				}
				
				notify.WakeupMain();
				p.Refresh();
				
			}
			
			p.WaitForExit();			
			
			notify2.WakeupMain();				
		
		}										
		
		/// <summary> Obtain the log of an item. </summary>
		public void log() {
		
			string buf1, buf2;
			string tmpstr = "";
			string space = " ";
			string EOF = "\n";		
		
			Process p = new Process();

			p.StartInfo.FileName = svnconfig.getSvnExec();
			p.StartInfo.Arguments += SvnSubcommands.svnLog + space;
			p.StartInfo.EnvironmentVariables.Add("LC_ALL","C");		
			
			p.StartInfo.Arguments += _params[0];			

			System.Console.WriteLine( p.StartInfo.Arguments);
			
			p.StartInfo.UseShellExecute = false;
			p.StartInfo.RedirectStandardError = true;
			p.StartInfo.RedirectStandardInput = true;
			p.StartInfo.RedirectStandardOutput = true;
			p.StartInfo.CreateNoWindow = true;
			
			p.Start();
			
			while ( ((buf1 = p.StandardOutput.ReadLine() ) != null) || ((buf2 = p.StandardError.ReadLine() ) != null ) ) {

				if (buf1 != null) {
					System.Console.WriteLine(buf1);	
					commandOutput.setCommandMessage(buf1);
					notify.WakeupMain();				
				}				
				if (buf2 != null) {
					commandOutput.setErrorMessage(buf2);
					notify3.WakeupMain();
				}
				
				notify.WakeupMain();
				p.Refresh();
				
			}
			
			p.WaitForExit();			
			
			notify2.WakeupMain();				
			
		}
		
		/// <summary> Create a repository. </summary>
		public void create() {
		
			string buf1, buf2;
			string tmpstr = "";
			string space = " ";
			string EOF = "\n";		
		
			Process p = new Process();

			p.StartInfo.FileName = svnconfig.getSvnadmin();
			p.StartInfo.Arguments += SvnSubcommands.svnCreate + space;
			p.StartInfo.EnvironmentVariables.Add("LC_ALL","C");		
			
			p.StartInfo.Arguments += _params[0];			

			System.Console.WriteLine( p.StartInfo.Arguments );
			
			p.StartInfo.UseShellExecute = false;
			p.StartInfo.RedirectStandardError = true;
			p.StartInfo.RedirectStandardInput = true;
			p.StartInfo.RedirectStandardOutput = true;
			p.StartInfo.CreateNoWindow = true;
			
			p.Start();
			
			while ( ((buf1 = p.StandardOutput.ReadLine() ) != null) || ((buf2 = p.StandardError.ReadLine() ) != null ) ) {

				if (buf1 != null) {
					System.Console.WriteLine(buf1);	
					commandOutput.setCommandMessage(buf1);
					notify.WakeupMain();				
				}				
				if (buf2 != null) {
					commandOutput.setErrorMessage(buf2);
					notify3.WakeupMain();
				}
				
				notify.WakeupMain();
				p.Refresh();
				
			}
			
			p.WaitForExit();			
			
			notify2.WakeupMain();				
			
		}		
	
	} // closing client class.

} // closing pfc.Svn namespace.
